@@ -65,15 +65,38 @@
 #include <stdio.h>
 #endif
 
-
 #include <string.h> // for malloc, memset, memcpy
 #include <stdlib.h> // for free
 
+#ifndef WEED_NO_FAST_APPEND
+# define WEED_FAST_APPEND
+#endif
+
+
+inline void weed_free(void *p) {
+  free(p);
+}
+
+
+inline void *weed_memset(void *p, int c, size_t n) {
+  return memset(p, c, n);
+}
+
+
+inline void *weed_malloc(size_t n) {
+  return malloc(n);
+}
+
+
+inline void *weed_memcpy(void *dest, const void *src, size_t n) {
+  return memcpy(dest, src, n);
+}
+
 
 #if !GLIB_CHECK_VERSION(2, 14, 0)
 static inline gpointer g_slice_copy(gsize bsize, gconstpointer block) {
   gpointer ret = g_slice_alloc(bsize);
-  memcpy(ret, block, bsize);
+  weed_memcpy(ret, block, bsize);
   return ret;
 }
 #endif
@@ -86,18 +109,21 @@ static inline size_t weed_strlen(const char *string) {
   return len;
 }
 
+
 static inline char *weed_strdup(const char *string) {
   size_t len;
-  char *ret = (char *)malloc((len = weed_strlen(string)) + 1);
+  char *ret = (char *)weed_malloc((len = weed_strlen(string)) + 1);
 
-  memcpy(ret, string, len + 1);
+  weed_memcpy(ret, string, len + 1);
   return ret;
 }
 
+
 static inline char *weed_slice_strdup(const char *string) {
   return (char *)g_slice_copy(weed_strlen(string) + 1, string);
 }
 
+
 static inline int weed_strcmp(const char *st1, const char *st2) {
   while (!(*st1 == 0 && *st2 == 0)) {
     if (*(st1) == 0 || *(st2) == 0 || *(st1++) != *(st2++)) return 1;
@@ -106,11 +132,20 @@ static inline int weed_strcmp(const char *st1, const char *st2) {
 }
 
 
+static inline uint32_t weed_hash(const char *string) {
+  char c;
+  uint32_t hash = 5381;
+  while ((c = *(string++)) != 0) hash += (hash << 5) + c;
+  return hash;
+}
+
+
 static inline int weed_seed_is_ptr(int seed) {
   return (seed != WEED_SEED_BOOLEAN && seed != WEED_SEED_INT && seed != WEED_SEED_DOUBLE && seed != WEED_SEED_STRING && seed !=
           WEED_SEED_INT64) ? 1 : 0;
 }
 
+
 static inline size_t weed_seed_get_size(int seed, void *value) {
   return weed_seed_is_ptr(seed) ? (sizeof(void *)) : \
          (seed == WEED_SEED_BOOLEAN || seed == WEED_SEED_INT) ? 4 : \
@@ -119,6 +154,7 @@ static inline size_t weed_seed_get_size(int seed, void *value) {
          (seed == WEED_SEED_STRING) ? weed_strlen((const char *)value) : 0;
 }
 
+
 static inline void weed_data_free(weed_data_t **data, int num_elems, int seed_type) {
   register int i;
   for (i = 0; i < num_elems; i++) {
@@ -129,6 +165,7 @@ static inline void weed_data_free(weed_data_t **data, int num_elems, int seed_ty
   g_slice_free1(num_elems * sizeof(weed_data_t *), data);
 }
 
+
 static inline weed_data_t **weed_data_new(int seed_type, int num_elems, void *value) {
   register int i;
   weed_data_t **data = NULL;
@@ -164,20 +201,25 @@ static inline weed_data_t **weed_data_new(int seed_type, int num_elems, void *va
   return data;
 }
 
+
 static inline weed_leaf_t *weed_find_leaf(weed_plant_t *leaf, const char *key) {
+  uint32_t hash = weed_hash(key);
   while (leaf != NULL) {
-    if (!weed_strcmp((char *)leaf->key, (char *)key)) return leaf;
+    if (hash == leaf->key_hash)
+      if (!weed_strcmp((char *)leaf->key, (char *)key)) return leaf;
     leaf = leaf->next;
   }
   return NULL;
 }
 
+
 static inline void weed_leaf_free(weed_leaf_t *leaf) {
   weed_data_free(leaf->data, leaf->num_elements, leaf->seed_type);
   g_slice_free1(weed_strlen(leaf->key) + 1, (void *)leaf->key);
   g_slice_free(weed_leaf_t, leaf);
 }
 
+
 static inline weed_leaf_t *weed_leaf_new(const char *key, int seed) {
   weed_leaf_t *leaf = g_slice_new(weed_leaf_t);
   if (leaf == NULL) return NULL;
@@ -185,6 +227,7 @@ static inline weed_leaf_t *weed_leaf_new(const char *key, int seed) {
     g_slice_free(weed_leaf_t, leaf);
     return NULL;
   }
+  leaf->key_hash = weed_hash(key);
   leaf->seed_type = seed;
   leaf->data = NULL;
   leaf->next = NULL;
@@ -192,7 +235,13 @@ static inline weed_leaf_t *weed_leaf_new(const char *key, int seed) {
   return leaf;
 }
 
+
 static inline void weed_leaf_append(weed_plant_t *leaf, weed_leaf_t *newleaf) {
+#ifdef WEED_FAST_APPEND
+  newleaf->next = leaf->next;
+  leaf->next = newleaf;
+  return;
+#endif
   weed_leaf_t *leafnext;
   while (leaf != NULL) {
     if ((leafnext = leaf->next) == NULL) {
@@ -203,6 +252,7 @@ static inline void weed_leaf_append(weed_plant_t *leaf, weed_leaf_t *newleaf) {
   }
 }
 
+
 void weed_plant_free(weed_plant_t *leaf) {
   weed_leaf_t *leafnext;
   while (leaf != NULL) {
@@ -215,13 +265,16 @@ void weed_plant_free(weed_plant_t *leaf) {
 
 int weed_leaf_delete(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = plant, *leafnext;
+  uint32_t hash = weed_hash(key);
   while (leaf->next != NULL) {
-    if (!weed_strcmp((char *)leaf->next->key, (char *)key)) {
-      if (leaf->next->flags & WEED_LEAF_READONLY_HOST) return WEED_ERROR_LEAF_READONLY;
-      leafnext = leaf->next;
-      leaf->next = leaf->next->next;
-      weed_leaf_free(leafnext);
-      return WEED_NO_ERROR;
+    if (leaf->next->key_hash == hash) {
+      if (!weed_strcmp((char *)leaf->next->key, (char *)key)) {
+        if (leaf->next->flags & WEED_LEAF_READONLY_HOST) return WEED_ERROR_LEAF_READONLY;
+        leafnext = leaf->next;
+        leaf->next = leaf->next->next;
+        weed_leaf_free(leafnext);
+        return WEED_NO_ERROR;
+      }
     }
     leaf = leaf->next;
   }
@@ -236,6 +289,7 @@ int weed_leaf_set_flags(weed_plant_t *plant, const char *key, int flags) {
   return WEED_NO_ERROR;
 }
 
+
 weed_plant_t *weed_plant_new(int plant_type) {
   weed_leaf_t *leaf;
   if ((leaf = weed_leaf_new("type", WEED_SEED_INT)) == NULL) return NULL;
@@ -250,6 +304,7 @@ weed_plant_t *weed_plant_new(int plant_type) {
   return leaf;
 }
 
+
 char **weed_plant_list_leaves(weed_plant_t *plant) {
   weed_leaf_t *leaf = plant;
   char **leaflist;
@@ -257,12 +312,12 @@ char **weed_plant_list_leaves(weed_plant_t *plant) {
   for (; leaf != NULL; i++) {
     leaf = leaf->next;
   }
-  if ((leaflist = (char **)malloc(i * sizeof(char *))) == NULL) return NULL;
+  if ((leaflist = (char **)weed_malloc(i * sizeof(char *))) == NULL) return NULL;
   i = 0;
   for (leaf = plant; leaf != NULL; leaf = leaf->next) {
     if ((leaflist[i] = weed_strdup(leaf->key)) == NULL) {
-      for (--i; i >= 0; i--) free(leaflist[i]);
-      free(leaflist);
+      for (--i; i >= 0; i--) weed_free(leaflist[i]);
+      weed_free(leaflist);
       return NULL;
     }
     i++;
@@ -271,6 +326,7 @@ char **weed_plant_list_leaves(weed_plant_t *plant) {
   return leaflist;
 }
 
+
 static inline int _weed_leaf_set_caller(weed_plant_t *plant, const char *key, int seed_type, int num_elems,
                                         void *value, int caller) {
 #ifdef CHECK_CALLER
@@ -322,74 +378,63 @@ int weed_default_get(weed_plant_t *plant, const char *key, int idx, void *value)
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL || idx > leaf->num_elements) return WEED_ERROR_NOSUCH_LEAF;
   if (value == NULL) return WEED_NO_ERROR;
-  if (weed_seed_is_ptr(leaf->seed_type)) memcpy(value, &leaf->data[idx]->value, sizeof(void *));
+  if (weed_seed_is_ptr(leaf->seed_type)) weed_memcpy(value, &leaf->data[idx]->value, sizeof(void *));
   else {
     if (leaf->seed_type == WEED_SEED_STRING) {
       size_t size = leaf->data[idx]->size;
       char **valuecharptrptr = (char **)value;
-      if (size > 0) memcpy(*valuecharptrptr, leaf->data[idx]->value, size);
-      memset(*valuecharptrptr + size, 0, 1);
-    } else memcpy(value, leaf->data[idx]->value, weed_seed_get_size(leaf->seed_type, leaf->data[idx]->value));
+      if (size > 0) weed_memcpy(*valuecharptrptr, leaf->data[idx]->value, size);
+      weed_memset(*valuecharptrptr + size, 0, 1);
+    } else weed_memcpy(value, leaf->data[idx]->value, weed_seed_get_size(leaf->seed_type, leaf->data[idx]->value));
   }
   return WEED_NO_ERROR;
-
 }
 
 
-
 int weed_leaf_get(weed_plant_t *plant, const char *key, int idx, void *value) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL || idx > leaf->num_elements) return WEED_ERROR_NOSUCH_LEAF;
   if (value == NULL) return WEED_NO_ERROR;
-  if (weed_seed_is_ptr(leaf->seed_type)) memcpy(value, &leaf->data[idx]->value, sizeof(void *));
+  if (weed_seed_is_ptr(leaf->seed_type)) weed_memcpy(value, &leaf->data[idx]->value, sizeof(void *));
   else {
     if (leaf->seed_type == WEED_SEED_STRING) {
       size_t size = leaf->data[idx]->size;
       char **valuecharptrptr = (char **)value;
-      if (size > 0) memcpy(*valuecharptrptr, leaf->data[idx]->value, size);
-      memset(*valuecharptrptr + size, 0, 1);
-    } else memcpy(value, leaf->data[idx]->value, weed_seed_get_size(leaf->seed_type, leaf->data[idx]->value));
+      if (size > 0) weed_memcpy(*valuecharptrptr, leaf->data[idx]->value, size);
+      weed_memset(*valuecharptrptr + size, 0, 1);
+    } else weed_memcpy(value, leaf->data[idx]->value, weed_seed_get_size(leaf->seed_type, leaf->data[idx]->value));
   }
   return WEED_NO_ERROR;
 }
 
+
 int weed_leaf_num_elements(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL) return 0;
   return leaf->num_elements;
 }
 
+
 size_t weed_leaf_element_size(weed_plant_t *plant, const char *key, int idx) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL || idx > leaf->num_elements) return 0;
   return leaf->data[idx]->size;
 }
 
+
 int weed_leaf_seed_type(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL) return 0;
   return leaf->seed_type;
 }
 
+
 int weed_leaf_get_flags(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL) return 0;
   return leaf->flags;
 }
 
-
-void weed_free(void *p) {
-  free(p);
-}
-
-void *weed_memset(void *p, int c, size_t n) {
-  return memset(p, c, n);
-}
-
-void *weed_malloc(size_t n) {
-  return malloc(n);
-}
-
-void *weed_memcpy(void *dest, const void *src, size_t n) {
-  return memcpy(dest, src, n);
-}
+#ifndef WEED_NO_FAST_APPEND
+# undef WEED_FAST_APPEND
+#endif
