@@ -55,6 +55,21 @@
 #include "weed-host.h"
 #endif
 
+#if defined __GNUC__ && !defined WEED_IGN_GNUC_OPT
+#  define GNU_FLATTEN  __attribute__((flatten)) // inline all function calls
+#  define GNU_CONST  __attribute__((const))
+#  define GNU_HOT  __attribute__((hot))
+#  define GNU_PURE  __attribute__((pure))
+#else
+#  define GNU_FLATTEN
+#  define GNU_CONST
+#  define GNU_HOT
+#  define GNU_PURE
+#endif
+
+#ifndef WEED_NO_FAST_APPEND
+# define WEED_FAST_APPEND
+#endif
 
 extern weed_default_getter_f weed_default_get;
 extern weed_leaf_get_f weed_leaf_get;
@@ -80,20 +95,32 @@ extern weed_free_f weed_free;
 extern weed_memcpy_f weed_memcpy;
 extern weed_memset_f weed_memset;
 
-static weed_plant_t *_weed_plant_new(int plant_type);
-static char **_weed_plant_list_leaves(weed_plant_t *plant);
+static weed_plant_t *_weed_plant_new(int plant_type) GNU_FLATTEN;
+static char **_weed_plant_list_leaves(weed_plant_t *plant) GNU_FLATTEN;
 static int _weed_leaf_set_caller(weed_plant_t *plant, const char *key, int seed_type, int num_elems,
-                                 void *value, int caller);
-static int _weed_leaf_get(weed_plant_t *plant, const char *key, int idx, void *value);
-static int _weed_leaf_num_elements(weed_plant_t *plant, const char *key);
-static size_t _weed_leaf_element_size(weed_plant_t *plant, const char *key, int idx);
-static int _weed_leaf_seed_type(weed_plant_t *plant, const char *key);
-static int _weed_leaf_get_flags(weed_plant_t *plant, const char *key);
+                                 void *value, int caller) GNU_FLATTEN;
+static int _weed_leaf_set_plugin(weed_plant_t *plant, const char *key, int seed_type, int num_elems, void *value) GNU_FLATTEN;
+static int _weed_leaf_get(weed_plant_t *plant, const char *key, int idx, void *value) GNU_HOT;
+static int _weed_leaf_set(weed_plant_t *plant, const char *key, int seed_type, int num_elems, void *value) GNU_FLATTEN;
+static int _weed_leaf_num_elements(weed_plant_t *plant, const char *key) GNU_FLATTEN;
+static size_t _weed_leaf_element_size(weed_plant_t *plant, const char *key, int idx) GNU_FLATTEN;
+static int _weed_leaf_seed_type(weed_plant_t *plant, const char *key) GNU_FLATTEN;
+static int _weed_leaf_get_flags(weed_plant_t *plant, const char *key) GNU_FLATTEN;
+static size_t weed_seed_get_size(int seed, void *value) GNU_FLATTEN GNU_HOT GNU_PURE;
+static weed_leaf_t *weed_find_leaf(weed_plant_t *leaf, const char *key) GNU_FLATTEN GNU_HOT;
+static weed_leaf_t *weed_leaf_new(const char *key, int seed) GNU_FLATTEN;
+
+static int weed_seed_is_ptr(int seed) GNU_CONST;
+
+static int weed_strcmp(const char *st1, const char *st2) GNU_HOT;
+static size_t weed_strlen(const char *string) GNU_PURE;
+static uint32_t weed_hash(const char *string) GNU_PURE;
 
 /* host only functions */
-static void _weed_plant_free(weed_plant_t *plant);
-static int _weed_leaf_set_flags(weed_plant_t *plant, const char *key, int flags);
-static int _weed_leaf_delete(weed_plant_t *plant, const char *key);
+static void _weed_plant_free(weed_plant_t *plant) GNU_FLATTEN;
+static int _weed_leaf_set_flags(weed_plant_t *plant, const char *key, int flags) GNU_FLATTEN;
+static int _weed_leaf_delete(weed_plant_t *plant, const char *key) GNU_FLATTEN;
+
 
 static inline size_t weed_strlen(const char *string) {
   size_t len = 0;
@@ -102,6 +129,7 @@ static inline size_t weed_strlen(const char *string) {
   return len;
 }
 
+
 static inline char *weed_strdup(const char *string) {
   size_t len;
   char *ret = weed_malloc((len = weed_strlen(string)) + 1);
@@ -109,8 +137,8 @@ static inline char *weed_strdup(const char *string) {
   return ret;
 }
 
-static inline int weed_strcmp(const char *st1, const char *st2) {
 
+static inline int weed_strcmp(const char *st1, const char *st2) {
   while (!(*st1 == 0 && *st2 == 0)) {
     if (*(st1) == 0 || *(st2) == 0 || *(st1++) != *(st2++)) return 1;
   }
@@ -118,11 +146,20 @@ static inline int weed_strcmp(const char *st1, const char *st2) {
 }
 
 
+static inline uint32_t weed_hash(const char *string) {
+  char c;
+  uint32_t hash = 5381;
+  while ((c = *(string++)) != 0) hash += (hash << 5) + c;
+  return hash;
+}
+
+
 static inline int weed_seed_is_ptr(int seed) {
   return (seed != WEED_SEED_BOOLEAN && seed != WEED_SEED_INT && seed != WEED_SEED_DOUBLE && seed != WEED_SEED_STRING &&
           seed != WEED_SEED_INT64) ? 1 : 0;
 }
 
+
 static inline size_t weed_seed_get_size(int seed, void *value) {
   return weed_seed_is_ptr(seed) ? (sizeof(void *)) : \
          (seed == WEED_SEED_BOOLEAN || seed == WEED_SEED_INT) ? 4 : \
@@ -131,6 +168,7 @@ static inline size_t weed_seed_get_size(int seed, void *value) {
          (seed == WEED_SEED_STRING) ? weed_strlen((const char *)value) : 0;
 }
 
+
 static inline void weed_data_free(weed_data_t **data, int num_elems, int seed_type) {
   register int i;
   if (data == NULL) return;
@@ -141,6 +179,7 @@ static inline void weed_data_free(weed_data_t **data, int num_elems, int seed_ty
   weed_free(data);
 }
 
+
 static inline weed_data_t **weed_data_new(int seed_type, int num_elems, void *value) {
   register int i;
   weed_data_t **data = NULL;
@@ -176,20 +215,25 @@ static inline weed_data_t **weed_data_new(int seed_type, int num_elems, void *va
   return data;
 }
 
+
 static inline weed_leaf_t *weed_find_leaf(weed_plant_t *leaf, const char *key) {
+  uint32_t hash = weed_hash(key);
   while (leaf != NULL) {
-    if (!weed_strcmp((char *)leaf->key, (char *)key)) return leaf;
+    if (hash == leaf->key_hash)
+      if (!weed_strcmp((char *)leaf->key, (char *)key)) return leaf;
     leaf = leaf->next;
   }
   return NULL;
 }
 
+
 static inline void weed_leaf_free(weed_leaf_t *leaf) {
   weed_data_free(leaf->data, leaf->num_elements, leaf->seed_type);
   weed_free((char *)leaf->key);
   weed_free(leaf);
 }
 
+
 static inline weed_leaf_t *weed_leaf_new(const char *key, int seed) {
   weed_leaf_t *leaf = (weed_leaf_t *)weed_malloc(sizeof(weed_leaf_t));
   if (leaf == NULL) return NULL;
@@ -197,6 +241,7 @@ static inline weed_leaf_t *weed_leaf_new(const char *key, int seed) {
     weed_free(leaf);
     return NULL;
   }
+  leaf->key_hash = weed_hash(key);
   leaf->seed_type = seed;
   leaf->data = NULL;
   leaf->next = NULL;
@@ -204,7 +249,14 @@ static inline weed_leaf_t *weed_leaf_new(const char *key, int seed) {
   return leaf;
 }
 
+
 static inline void weed_leaf_append(weed_plant_t *leaf, weed_leaf_t *newleaf) {
+#ifdef WEED_FAST_APPEND
+  newleaf->next = leaf->next;
+  leaf->next = newleaf;
+  return;
+#endif
+
   weed_leaf_t *leafnext;
   while (leaf != NULL) {
     if ((leafnext = leaf->next) == NULL) {
@@ -215,6 +267,7 @@ static inline void weed_leaf_append(weed_plant_t *leaf, weed_leaf_t *newleaf) {
   }
 }
 
+
 static void _weed_plant_free(weed_plant_t *leaf) {
   weed_leaf_t *leafnext;
   while (leaf != NULL) {
@@ -227,13 +280,16 @@ static void _weed_plant_free(weed_plant_t *leaf) {
 
 static int _weed_leaf_delete(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = plant, *leafnext;
+  uint32_t hash = weed_hash(key);
   while (leaf->next != NULL) {
-    if (!weed_strcmp((char *)leaf->next->key, (char *)key)) {
-      if (leaf->next->flags & WEED_LEAF_READONLY_HOST) return WEED_ERROR_LEAF_READONLY;
-      leafnext = leaf->next;
-      leaf->next = leaf->next->next;
-      weed_leaf_free(leafnext);
-      return WEED_NO_ERROR;
+    if (leaf->next->key_hash == hash) {
+      if (!weed_strcmp((char *)leaf->next->key, (char *)key)) {
+        if (leaf->next->flags & WEED_LEAF_READONLY_HOST) return WEED_ERROR_LEAF_READONLY;
+        leafnext = leaf->next;
+        leaf->next = leaf->next->next;
+        weed_leaf_free(leafnext);
+        return WEED_NO_ERROR;
+      }
     }
     leaf = leaf->next;
   }
@@ -248,6 +304,7 @@ static int _weed_leaf_set_flags(weed_plant_t *plant, const char *key, int flags)
   return WEED_NO_ERROR;
 }
 
+
 static weed_plant_t *_weed_plant_new(int plant_type) {
   weed_leaf_t *leaf;
   if ((leaf = weed_leaf_new("type", WEED_SEED_INT)) == NULL) return NULL;
@@ -262,6 +319,7 @@ static weed_plant_t *_weed_plant_new(int plant_type) {
   return leaf;
 }
 
+
 static char **_weed_plant_list_leaves(weed_plant_t *plant) {
   weed_leaf_t *leaf = plant;
   char **leaflist;
@@ -283,6 +341,7 @@ static char **_weed_plant_list_leaves(weed_plant_t *plant) {
   return leaflist;
 }
 
+
 static inline int _weed_leaf_set_caller(weed_plant_t *plant, const char *key, int seed_type, int num_elems,
                                         void *value, int caller) {
   weed_data_t **data = NULL;
@@ -342,7 +401,6 @@ static int _weed_default_get(weed_plant_t *plant, const char *key, int idx, void
     } else memcpy(value, leaf->data[idx]->value, weed_seed_get_size(leaf->seed_type, leaf->data[idx]->value));
   }
   return WEED_NO_ERROR;
-
 }
 
 
@@ -370,31 +428,34 @@ static int _weed_leaf_num_elements(weed_plant_t *plant, const char *key) {
   return leaf->num_elements;
 }
 
+
 static size_t _weed_leaf_element_size(weed_plant_t *plant, const char *key, int idx) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL || idx >= leaf->num_elements) return 0;
   return leaf->data[idx]->size;
 }
 
+
 static int _weed_leaf_seed_type(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL) return 0;
   return leaf->seed_type;
 }
 
+
 static int _weed_leaf_get_flags(weed_plant_t *plant, const char *key) {
   weed_leaf_t *leaf = weed_find_leaf(plant, key);
   if (leaf == NULL) return 0;
   return leaf->flags;
 }
 
+
 void weed_init(int api, weed_malloc_f _mallocf, weed_free_f _freef, weed_memcpy_f _memcpyf, weed_memset_f _memsetf) {
   // this is called by the host in order for it to set its version of the functions
 
   // *the plugin should never call this, instead the plugin functions are passed to the plugin
   // from the host in the "host_info" plant*
 
-
   switch (api) {
   // higher API versions may use different functions, or add to them
 
@@ -427,5 +488,8 @@ void weed_init(int api, weed_malloc_f _mallocf, weed_free_f _freef, weed_memcpy_
   else weed_memcpy = (weed_memcpy_f)memcpy;
   if (_memsetf != NULL) weed_memset = _memsetf;
   else weed_memset = (weed_memset_f)memset;
-
 }
+
+#ifndef WEED_NO_FAST_APPEND
+# undef WEED_FAST_APPEND
+#endif
