@@ -65,20 +65,20 @@ const char *plugin_version = "LiVES asf/wmv decoder version 1.1";
 
 #include "asf_decoder.h"
 
-static enum CodecID ff_codec_get_id(const AVCodecTag *tags, unsigned int tag) {
+static enum AVCodecID ff_codec_get_id(const AVCodecTag *tags, unsigned int tag) {
   int i;
-  for (i = 0; tags[i].id != CODEC_ID_NONE; i++) {
+  for (i = 0; tags[i].id != AV_CODEC_ID_NONE; i++) {
     if (tag == tags[i].tag)
       return tags[i].id;
   }
-  for (i = 0; tags[i].id != CODEC_ID_NONE; i++) {
+  for (i = 0; tags[i].id != AV_CODEC_ID_NONE; i++) {
     if (toupper((tag >> 0) & 0xFF) == toupper((tags[i].tag >> 0) & 0xFF)
         && toupper((tag >> 8) & 0xFF) == toupper((tags[i].tag >> 8) & 0xFF)
         && toupper((tag >> 16) & 0xFF) == toupper((tags[i].tag >> 16) & 0xFF)
         && toupper((tag >> 24) & 0xFF) == toupper((tags[i].tag >> 24) & 0xFF))
       return tags[i].id;
   }
-  return CODEC_ID_NONE;
+  return AV_CODEC_ID_NONE;
 }
 
 static index_container_t **indices;
@@ -439,7 +439,7 @@ static int get_next_video_packet(const lives_clip_data_t *cdata, int tfrag, int6
 
 
   if (tdts == -1) {
-    priv->avpkt.size = priv->def_packet_size + FF_INPUT_BUFFER_PADDING_SIZE;
+    priv->avpkt.size = priv->def_packet_size + AV_INPUT_BUFFER_PADDING_SIZE;
     priv->avpkt.data = malloc(priv->avpkt.size);
     memset(priv->avpkt.data, 0, priv->avpkt.size);
   }
@@ -714,10 +714,10 @@ static int get_next_video_packet(const lives_clip_data_t *cdata, int tfrag, int6
             fprintf(stderr, "asf_decoder: buffer overflow reading vid packet (%d + %d > %d),\n increasing buffer size\n",
                     asf->packet_frag_offset, asf->packet_frag_size, priv->avpkt.size);
 
-            priv->avpkt.data = realloc(priv->avpkt.data, priv->def_packet_size * 2 + FF_INPUT_BUFFER_PADDING_SIZE);
+            priv->avpkt.data = realloc(priv->avpkt.data, priv->def_packet_size * 2 + AV_INPUT_BUFFER_PADDING_SIZE);
             memset(priv->avpkt.data + priv->avpkt.size, 0, priv->def_packet_size);
             priv->def_packet_size *= 2;
-            priv->avpkt.size = priv->def_packet_size + FF_INPUT_BUFFER_PADDING_SIZE;
+            priv->avpkt.size = priv->def_packet_size + AV_INPUT_BUFFER_PADDING_SIZE;
           }
 
           if (read(priv->fd, priv->avpkt.data + asf->packet_frag_offset, asf->packet_frag_size)
@@ -917,7 +917,7 @@ static void detach_stream(lives_clip_data_t *cdata) {
     av_free(priv->ctx);
   }
 
-  if (priv->picture != NULL) av_frame_unref(&priv->picture);
+  if (priv->picture != NULL) av_frame_unref(priv->picture);
 
   priv->ctx = NULL;
   priv->picture = NULL;
@@ -1436,8 +1436,8 @@ seek_skip:
           if (!got_vidst) {
             priv->st->codec->extradata_size = size - 40;
 
-            priv->st->codec->extradata = malloc(priv->st->codec->extradata_size + FF_INPUT_BUFFER_PADDING_SIZE);
-            memset(priv->st->codec->extradata, 0, priv->st->codec->extradata_size + FF_INPUT_BUFFER_PADDING_SIZE);
+            priv->st->codec->extradata = malloc(priv->st->codec->extradata_size + AV_INPUT_BUFFER_PADDING_SIZE);
+            memset(priv->st->codec->extradata, 0, priv->st->codec->extradata_size + AV_INPUT_BUFFER_PADDING_SIZE);
 
             if (read(priv->fd, priv->st->codec->extradata,
                      priv->st->codec->extradata_size) < priv->st->codec->extradata_size) {
@@ -2106,28 +2106,28 @@ seek_skip:
   }
 
   switch (vidst->codec->codec_id) {
-  case CODEC_ID_WMV1:
+  case AV_CODEC_ID_WMV1:
     snprintf(cdata->video_name, 16, "wmv1");
     break;
-  case CODEC_ID_WMV2:
+  case AV_CODEC_ID_WMV2:
     snprintf(cdata->video_name, 16, "wmv2");
     break;
-  case CODEC_ID_WMV3:
+  case AV_CODEC_ID_WMV3:
     snprintf(cdata->video_name, 16, "wmv3");
     break;
-  case CODEC_ID_DVVIDEO:
+  case AV_CODEC_ID_DVVIDEO:
     snprintf(cdata->video_name, 16, "dv");
     break;
-  case CODEC_ID_MPEG4:
+  case AV_CODEC_ID_MPEG4:
     snprintf(cdata->video_name, 16, "mpeg4");
     break;
-  case CODEC_ID_H264:
+  case AV_CODEC_ID_H264:
     snprintf(cdata->video_name, 16, "h264");
     break;
-  case CODEC_ID_MPEG1VIDEO:
+  case AV_CODEC_ID_MPEG1VIDEO:
     snprintf(cdata->video_name, 16, "mpeg1");
     break;
-  case CODEC_ID_MPEG2VIDEO:
+  case AV_CODEC_ID_MPEG2VIDEO:
     snprintf(cdata->video_name, 16, "mpeg2");
     break;
   default:
@@ -2532,7 +2532,7 @@ static lives_clip_data_t *asf_clone(lives_clip_data_t *cdata) {
 
   }
 
-  if (dpriv->picture != NULL) av_frame_unref(&dpriv->picture);
+  if (dpriv->picture != NULL) av_frame_unref(dpriv->picture);
   dpriv->picture = NULL;
 
   return clone;
@@ -2608,7 +2608,7 @@ lives_clip_data_t *get_clip_data(const char *URI, lives_clip_data_t *cdata) {
   cdata->asigned = TRUE;
   cdata->ainterleaf = TRUE;
 
-  if (priv->picture != NULL) av_frame_unref(&priv->picture);
+  if (priv->picture != NULL) av_frame_unref(priv->picture);
   priv->picture = NULL;
 
   return cdata;
@@ -2734,7 +2734,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 
   if (tframe != priv->last_frame) {
 
-    if (priv->picture != NULL) av_frame_unref(&priv->picture);
+    if (priv->picture != NULL) av_frame_unref(priv->picture);
     priv->picture = NULL;
 
     if (priv->last_frame == -1 || (tframe < priv->last_frame) || (tframe - priv->last_frame > rescan_limit)) {
@@ -2814,7 +2814,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 
       // otherwise discard this frame
       if (got_picture) {
-        av_frame_unref(&priv->picture);
+        av_frame_unref(priv->picture);
         priv->picture = NULL;
         tfrag = -1;
         nextframe++;
