@@ -65,15 +65,15 @@ static void lives_avcodec_unlock(void) {
 }
 
 
-static int stream_peek(int fd, unsigned char *str, size_t len) {
-  off_t cpos = lseek(fd, 0, SEEK_CUR); // get current posn
-  int rv = pread(fd, str, len, cpos); // read len bytes without changing cpos
+/* static int stream_peek(int fd, unsigned char *str, size_t len) { */
+/*   off_t cpos = lseek(fd, 0, SEEK_CUR); // get current posn */
+/*   int rv = pread(fd, str, len, cpos); // read len bytes without changing cpos */
 
-  if (rv == -1) {
-    fprintf(stderr, "err is %d\n", errno);
-  }
-  return rv;
-}
+/*   if (rv == -1) { */
+/*     fprintf(stderr, "err is %d\n", errno); */
+/*   } */
+/*   return rv; */
+/* } */
 
 
 void get_samps_and_signed(enum AVSampleFormat sfmt, int *asamps, boolean *asigned) {
@@ -96,34 +96,58 @@ static int64_t get_current_ticks(void) {
 }
 
 
+#define FRAMES_GUESS 32768
+#define HALF_ROUND_UP(x) (x > 0 ? (int)(x / 2. + .5) : (int)(x / 2. - .5))
+
 static int64_t get_real_last_frame(lives_clip_data_t *cdata, boolean allow_longer_seek) {
   int64_t diff = 0;
-  int64_t olframe = (cdata->nframes == 0 ? 0 : cdata->nframes - 1), lframe = olframe;
+
+  // lframe is the frame we will check, olframe is the current seek base frame, maxframes is the largest found
+  int64_t olframe = cdata->nframes - 1, lframe = olframe;
   int64_t timex, tottime = 0;
-
-  int nseeks = 0;
-
   long no_seek_limit = NO_SEEK_LIMIT;
 
-  if (allow_longer_seek) no_seek_limit = LNO_SEEK_LIMIT;
+  int nseeks = 0;
+  boolean have_upper_bound = FALSE;
+  boolean have_lower_bound = FALSE;
 
+  if (allow_longer_seek) no_seek_limit = LNO_SEEK_LIMIT;
   cdata->seek_flag = LIVES_SEEK_FAST;
 
+  // check we can get at least one frame
+  if (!get_frame(cdata, 0, NULL, 0, NULL)) {
+    return -1;
+  }
+
+  if (cdata->nframes == 0) {
+    // if we got a broken file with 0 last frame, search for it
+    olframe = lframe = FRAMES_GUESS - 1;
+    diff = -FRAMES_GUESS;
+    have_lower_bound = TRUE;
+  }
+
   while (1) {
     nseeks++;
-    //    #define DEBUG_RLF
+    //#define DEBUG_RLF
 #ifdef DEBUG_RLF
-    fprintf(stderr, "will check frame %ld of %ld...", lframe + 1, cdata->nframes);
+    fprintf(stderr, "will check frame %ld of (allegedly) %ld...", lframe + 1, cdata->nframes);
 #endif
 
     timex = get_current_ticks();
+
+    // see if we can find lframe
     if (!get_frame(cdata, lframe, NULL, 0, NULL)) {
+      // lframe not found
+
+      have_upper_bound = TRUE; // we got an upper bound
+
       timex = get_current_ticks() - timex;
 
       if (timex > no_seek_limit) {
-        //#ifdef DEBUG_RLF
-        fprintf(stderr, "avcodec_decoder: seek was %ld, longer than limit of %d; giving up.\n", timex, NO_SEEK_LIMIT);
-        //#endif
+        // seek took too long, give up
+#ifdef DEBUG_RLF
+        fprintf(stderr, "avcodec_decoder: seek was %ld, longer than limit of %ld; giving up.\n", timex, no_seek_limit);
+#endif
         return -1;
       }
 
@@ -131,28 +155,34 @@ static int64_t get_real_last_frame(lives_clip_data_t *cdata, boolean allow_longe
       fprintf(stderr, "no (%ld)\n", timex);
 #endif
       if (diff == 1) {
+        // this is the ideal situation, we got lframe - 1, but not lframe
         tottime /= nseeks;
 #ifdef DEBUG_RLF
-        fprintf(stderr, "av positive seek was %ld\n", tottime);
+        fprintf(stderr, "av seek was %ld\n", tottime);
 #endif
         return lframe - 1;
       }
-      if (diff < 0) diff *= 2;
-      else if (diff == 0) diff = -1;
-      else diff /= 2;
-      lframe = olframe + diff;
-      if (lframe < 0) {
-        diff = olframe;
-        lframe = 0;
-        if (diff == 0) diff = 1;
+      if (diff == 0) {
+        diff = -1; // initial frame not found, start a backwards scan
+        have_lower_bound = FALSE;
+      } else {
+        if (!have_lower_bound) diff *= 2; // found no frame yet, search backwards more
+        else diff = HALF_ROUND_UP(diff); // found a frame, or started with a guess, search backwards less
       }
+
+      if (diff > 0) diff = -diff;
+
+      if (diff < -olframe) diff = -olframe; // can't jump to a negative frame !
     } else {
+      // we did find a frame
+      have_lower_bound = TRUE;
+
       timex = get_current_ticks() - timex;
 
-      if (timex > NO_SEEK_LIMIT) {
-        //#ifdef DEBUG_RLF
-        fprintf(stderr, "avcodec_decoder: seek was %ld, longer than limit of %d; giving up.\n", timex, NO_SEEK_LIMIT);
-        //#endif
+      if (timex > no_seek_limit) {
+#ifdef DEBUG_RLF
+        fprintf(stderr, "avcodec_decoder: seek was %ld, longer than limit of %ld; giving up.\n", timex, no_seek_limit);
+#endif
         return -1;
       }
 
@@ -163,20 +193,26 @@ static int64_t get_real_last_frame(lives_clip_data_t *cdata, boolean allow_longe
 #ifdef DEBUG_RLF
       fprintf(stderr, "yes ! (%ld)\n", timex);
 #endif
-      if (diff < 2 && diff > -4) {
-        tottime /= nseeks;
+      if (diff == 0) diff = 1; // initial frame found, start a forward scan
+      else {
+        if (diff < 2 && diff > -2) {
+          // this is the ideal situation, we got lframe, but not lframe + 1
+          tottime /= nseeks;
 #ifdef DEBUG_RLF
-        fprintf(stderr, "av seek was %ld\n", tottime);
+          fprintf(stderr, "av seek was %ld\n", tottime);
 #endif
-        if (tottime > FAST_SEEK_LIMIT)
-          cdata->seek_flag |= LIVES_SEEK_NEEDS_CALCULATION;
-        return lframe;
+          if (tottime > FAST_SEEK_LIMIT)
+            cdata->seek_flag |= LIVES_SEEK_NEEDS_CALCULATION;
+          return lframe;
+        }
+        if (have_upper_bound) diff = HALF_ROUND_UP(diff); // we found a null frame
+        else diff *= 2;
+        if (diff < 0) diff = -diff; // we were searching backwards, now we want to go forwards
       }
-      diff /= 2;
-      if (diff < 0) diff = -diff / 2;
-      olframe = lframe;
-      lframe += diff;
+
     }
+    olframe = lframe;
+    lframe += diff;
   }
 }
 
@@ -184,20 +220,19 @@ static int64_t get_real_last_frame(lives_clip_data_t *cdata, boolean allow_longe
 static boolean attach_stream(lives_clip_data_t *cdata, boolean isclone) {
   // open the file and get a handle
   lives_av_priv_t *priv = cdata->priv;
-  AVProbeData pd = {0};
   AVInputFormat *fmt;
-
   AVCodec *vdecoder;
-
   AVStream *s;
   AVCodecContext *cc;
+  AVFormatContext *fmt_ctx;
 
   int64_t i_start_time = 0;
 
   boolean is_partial_clone = FALSE;
 
-  register int i;
+  int err;
 
+  register int i;
 
   if (isclone && !priv->inited) {
     isclone = FALSE;
@@ -206,28 +241,21 @@ static boolean attach_stream(lives_clip_data_t *cdata, boolean isclone) {
     else priv->longer_seek = TRUE;
   }
 
-  if ((priv->fd = open(cdata->URI, O_RDONLY)) == -1) {
-    fprintf(stderr, "avformat_decoder: unable to open %s\n", cdata->URI);
-    return FALSE;
-  }
-
   if (isclone) goto skip_probe;
 
-  pd.filename = cdata->URI;
-
-  pd.buf = calloc(128, 32);
-
-  if ((pd.buf_size = stream_peek(priv->fd, pd.buf, AVPROBE_PADDING_SIZE)) < AVPROBE_PADDING_SIZE) {
-    fprintf(stderr, "couldn't peek stream %d\n", pd.buf_size);
+  fmt_ctx = avformat_alloc_context();
+  if (!fmt_ctx) {
+    fprintf(stderr, "No fmt_ctx\n");
     return FALSE;
   }
 
-  if (!(fmt = av_probe_input_format(&pd, TRUE))) {
-    fprintf(stderr, "couldn't guess format\n");
+  if ((err = avformat_open_input(&fmt_ctx, cdata->URI,
+                                 NULL, NULL)) < 0) {
+    fprintf(stderr, "avf_open failed\n");
     return FALSE;
   }
 
-  free(pd.buf);
+  fmt = fmt_ctx->iformat;
 
   if (!strcmp(fmt->name, "redir") ||
       !strcmp(fmt->name, "sdp")) {
@@ -252,6 +280,8 @@ static boolean attach_stream(lives_clip_data_t *cdata, boolean isclone) {
   priv->fmt = fmt;
   priv->inited = TRUE;
 
+  avformat_close_input(&fmt_ctx);
+
 skip_probe:
 
   priv->ic = NULL;
@@ -290,7 +320,6 @@ skip_probe:
     goto skip_init;
   }
 
-
   // fill cdata
 
   cdata->nclips = 1;
@@ -366,7 +395,6 @@ skip_init:
       priv->astream = i;
       break;
 
-
     case AVMEDIA_TYPE_VIDEO:
 
       if (!isclone && priv->vstream != -1) {
@@ -382,16 +410,15 @@ skip_init:
       cdata->frame_width = cdata->width = cc->width;
       cdata->frame_height = cdata->height = cc->height;
 
-
       /*if( cc->palctrl )
       {
       fmt.video.p_palette = malloc( sizeof(video_palette_t) );
       *fmt.video.p_palette = *(video_palette_t *)cc->palctrl;
       }*/
 
-
       cdata->YUV_subspace = WEED_YUV_SUBSPACE_YCBCR;
       cdata->YUV_sampling = WEED_YUV_SAMPLING_DEFAULT;
+      cdata->YUV_clamping = WEED_YUV_CLAMPING_CLAMPED;
 
       cdata->palettes[0] = avi_pix_fmt_to_weed_palette(cc->pix_fmt, &cdata->YUV_clamping);
       cdata->palettes[1] = WEED_PALETTE_END;
@@ -426,11 +453,6 @@ skip_init:
 
       priv->ctx = cc;
 
-      if (priv->ctx->ticks_per_frame == 2) {
-        // needs checking
-        cdata->interlace = LIVES_INTERLACE_BOTTOM_FIRST;
-      }
-
 #ifdef DEBUG
       fprintf(stderr, "fps is %.4f %d %d %d %d %d\n", cdata->fps, s->time_base.den, s->time_base.num, cc->time_base.den, cc->time_base.num,
               priv->ctx->ticks_per_frame);
@@ -448,8 +470,6 @@ skip_init:
 
     /*        case CODEC_TYPE_SUBTITLE:
 
-
-
     if( strncmp( p_sys->ic->iformat->name, "matroska", 8 ) == 0 &&
               cc->codec_id == AV_CODEC_ID_DVD_SUBTITLE &&
               cc->extradata != NULL &&
@@ -609,13 +629,9 @@ skip_init:
     return FALSE;
   }
 
-
   if (priv->ic->start_time != (int64_t)AV_NOPTS_VALUE)
     i_start_time = priv->ic->start_time * 1000000 / AV_TIME_BASE;
 
-
-
-
   fprintf(stderr, "AVFormat supported stream\n");
   fprintf(stderr, "    - format = %s (%s)\n",
           priv->fmt->name, priv->fmt->long_name);
@@ -624,8 +640,6 @@ skip_init:
           (priv->ic->duration != (int64_t)AV_NOPTS_VALUE) ?
           priv->ic->duration * 1000000 / AV_TIME_BASE : -1);
 
-
-
 #ifdef HAVE_FFMPEG_CHAPTERS
   /*    if( p_sys->ic->nb_chapters > 0 )
         p_sys->p_title = vlc_input_title_New();
@@ -647,16 +661,13 @@ skip_init:
   }*/
 #endif
 
-
   return TRUE;
 }
 
 
-
 static void detach_stream(lives_clip_data_t *cdata) {
   // close the file, free the decoder
   lives_av_priv_t *priv = cdata->priv;
-  if (priv->fd > -1) close(priv->fd);
 
   // will close and free the context
   if (priv->ic != NULL) {
@@ -680,15 +691,12 @@ static void detach_stream(lives_clip_data_t *cdata) {
 
   priv->astream = -1;
   priv->vstream = -1;
-
 }
 
 
 //////////////////////////////////////////
 // std functions
 
-
-
 const char *module_check_init(void) {
   lives_avcodec_lock();
   av_register_all(); /* Can be called several times */
@@ -702,7 +710,6 @@ const char *version(void) {
 }
 
 
-
 static lives_clip_data_t *init_cdata(void) {
   lives_av_priv_t *priv;
   lives_clip_data_t *cdata = (lives_clip_data_t *)malloc(sizeof(lives_clip_data_t));
@@ -715,8 +722,6 @@ static lives_clip_data_t *init_cdata(void) {
 
   cdata->priv = priv = malloc(sizeof(lives_av_priv_t));
 
-  priv->fd = -1;
-
   priv->ic = NULL;
 
   priv->astream = -1;
@@ -737,31 +742,33 @@ static lives_clip_data_t *init_cdata(void) {
   memset(cdata->author, 0, 1);
   memset(cdata->title, 0, 1);
   memset(cdata->comment, 0, 1);
-
+#ifdef TEST_CACHING
+  priv->cachemax = 128;
+  priv->cache = NULL;
+#endif
   return cdata;
 }
 
 
-
 static lives_clip_data_t *avf_clone(lives_clip_data_t *cdata) {
   lives_clip_data_t *clone = init_cdata();
   lives_av_priv_t *dpriv, *spriv;
 
   // copy from cdata to clone, with a new context for clone
-  clone->URI = strdup(cdata->URI);
-  clone->nclips = cdata->nclips;
+  clone->URI = strdup(cdata->URI);  // VALID for pclone
+  clone->nclips = cdata->nclips; // ???
   snprintf(clone->container_name, 512, "%s", cdata->container_name);
   clone->current_clip = cdata->current_clip;
   clone->width = cdata->width;
   clone->height = cdata->height;
-  clone->nframes = cdata->nframes;
+  clone->nframes = cdata->nframes;   // valid for pclone
   clone->interlace = cdata->interlace;
   clone->offs_x = cdata->offs_x;
   clone->offs_y = cdata->offs_y;
   clone->frame_width = cdata->frame_width;
   clone->frame_height = cdata->frame_height;
   clone->par = cdata->par;
-  clone->fps = cdata->fps;
+  clone->fps = cdata->fps;   // valid ? for pclone
   if (cdata->palettes != NULL) clone->palettes[0] = cdata->palettes[0];
   clone->current_palette = cdata->current_palette;
   clone->YUV_sampling = cdata->YUV_sampling;
@@ -813,7 +820,6 @@ static lives_clip_data_t *avf_clone(lives_clip_data_t *cdata) {
 }
 
 
-
 lives_clip_data_t *get_clip_data(const char *URI, lives_clip_data_t *cdata) {
   // the first time this is called, caller should pass NULL as the cdata
   // subsequent calls to this should re-use the same cdata
@@ -827,6 +833,7 @@ lives_clip_data_t *get_clip_data(const char *URI, lives_clip_data_t *cdata) {
   if (URI == NULL && cdata != NULL) {
     // create a clone of cdata - we also need to be able to handle a "fake" clone with only URI, nframes and fps set (priv == NULL)
     cdata = avf_clone(cdata);
+    if (cdata == NULL) return NULL;
     priv = cdata->priv;
     if (priv->longer_seek) goto rescan;
     return cdata;
@@ -868,10 +875,9 @@ rescan:
 
   real_frames = get_real_last_frame(cdata, priv->longer_seek) + 1;
 
-  if (cdata->nframes > 100 && real_frames < cdata->nframes * SEEK_SUCCESS_MIN_RATIO) {
+  if (real_frames <= 0) {
     fprintf(stderr,
-            "avformat_decoder: ERROR - could only seek to %ld frames out of %ld\navformat_decoder: I will pass on this file as it may be broken.\n",
-            real_frames, cdata->nframes);
+            "avformat_decoder: ERROR - could not find the last frame\navformat_decoder: I will pass on this file as it may be broken.\n");
     detach_stream(cdata);
     free(cdata->URI);
     cdata->URI = NULL;
@@ -879,9 +885,10 @@ rescan:
     return NULL;
   }
 
-  if (real_frames <= 0) {
+  if (cdata->nframes > 100 && real_frames < cdata->nframes * SEEK_SUCCESS_MIN_RATIO) {
     fprintf(stderr,
-            "avformat_decoder: ERROR - could not find the last frame\navformat_decoder: I will pass on this file as it may be broken.\n");
+            "avformat_decoder: ERROR - could only seek to %ld frames out of %ld\navformat_decoder: I will pass on this file as it may be broken.\n",
+            real_frames, cdata->nframes);
     detach_stream(cdata);
     free(cdata->URI);
     cdata->URI = NULL;
@@ -904,7 +911,6 @@ rescan:
 }
 
 
-
 static size_t write_black_pixel(unsigned char *idst, int pal, int npixels, int y_black) {
   unsigned char *dst = idst;
   register int i;
@@ -954,7 +960,80 @@ static size_t write_black_pixel(unsigned char *idst, int pal, int npixels, int y
   return idst - dst;
 }
 
+#ifdef TEST_CACHING
+#define DEF_CACHEFRAMES_MAX 16
 
+#ifndef ABS(a)
+#define ABS(a) (a >= 0ll ? a : -a)
+#endif
+
+static AVFrame *get_from_cache(lives_av_priv_t *priv, int64_t pts) {
+  priv_cache_t *cache = priv->cache;
+  while (cache != NULL) {
+    //fprintf(stderr, "CF %ld %ld %ld\n", cache->pts, pts, ABS(cache->pts - pts));
+    if (ABS(cache->pts - pts) <= 100) {
+      return cache->frame;
+    }
+    cache = cache->next;
+  }
+  return NULL;
+}
+
+
+static void remove_cache_above(priv_cache_t *cache, int maxe) {
+  priv_cache_t *lastcache = cache,  *nextcache;
+  int count = 0;
+
+  while (cache != NULL) {
+    nextcache = cache->next;
+    if (count++ >= maxe) {
+      if (lastcache != NULL) lastcache->next = NULL;
+      av_frame_unref(cache->frame);
+      free(cache);
+      lastcache = NULL;
+    } else lastcache = cache;
+    cache = nextcache;
+  }
+}
+
+
+static void add_to_cache(lives_av_priv_t *priv, int64_t pts) {
+  // if we have a frame with this pts in the list, ignore
+  // otherwise free the tail of the list and add this at the head
+
+  // NOTE: void av_frame_move_ref(AVFrame *dst, AVFrame *src); int av_frame_copy(AVFrame *dst, const AVFrame *src);
+  // int av_frame_ref(AVFrame *dst, const AVFrame *src); AVFrame *av_frame_clone(const AVFrame *src);
+  if (priv->cachemax < 1) return;
+  if (get_from_cache(priv, pts) != NULL) {
+    fprintf(stderr, "Dupe\n");
+    return;
+  } else {
+    priv_cache_t *cache = (priv_cache_t *)malloc(sizeof(priv_cache_t));
+    if (cache == NULL) return;
+    cache->frame = priv->pFrame;
+    cache->pts = pts;
+    cache->next = priv->cache;
+    priv->cache = cache;
+    remove_cache_above(cache, priv->cachemax);
+  }
+}
+
+
+static void free_cache(lives_av_priv_t *priv) {
+  // walk the list and free each element
+  remove_cache_above(priv->cache, 0);
+  priv->cache = NULL;
+}
+
+
+int begin_caching(const lives_clip_data_t *cdata, int maxframes) {
+  lives_av_priv_t *priv = cdata->priv;
+  if (maxframes == -1) maxframes = DEF_CACHEFRAMES_MAX;
+  priv->cachemax = maxframes;
+  if (maxframes == 0) free_cache(priv);
+  return maxframes;
+}
+#endif
 
 // tune this so small jumps forward are efficient
 #define JUMP_FRAMES_SLOW 64
@@ -978,6 +1057,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
   unsigned char black[4] = {0, 0, 0, 255};
 
   boolean hit_target = FALSE;
+  //boolean did_seek = FALSE;
 
   int gotFrame;
 
@@ -991,7 +1071,8 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 
   register int p, i;
 
-  if (tframe < 0 || tframe >= cdata->nframes || cdata->fps == 0.) return FALSE;
+  // if pixel_data is NULL, just check if the frame exists
+  if (tframe < 0 || ((tframe >= cdata->nframes || cdata->fps == 0.) && pixel_data != NULL)) return FALSE;
 
   cc = s->codec;
 
@@ -1046,22 +1127,31 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
       bleft = bright = 0;
     }
   }
+  //#define DEBUG
+
+  time = (double)tframe / cdata->fps;
+  target_pts = time * (double)AV_TIME_BASE;
+#ifdef TEST_CACHING
+  priv->pFrame = get_from_cache(priv, target_pts);
+#ifdef DEBUG
+  if (priv->pFrame != NULL) fprintf(stderr, "got frame from cache for target %ld %p %d\n", target_pts, priv->pFrame, pal);
+  else fprintf(stderr, "got frame from cache FAIL for target %ld %p %d\n", target_pts, priv->pFrame, pal);
+#endif
+  if (priv->pFrame != NULL) goto framedone2;
+#endif
 
   if (priv->pFrame == NULL || tframe != priv->last_frame) {
-    // same frame -> we reuse priv-pFrame;
+    // same frame -> we reuse priv-pFrame if we have it; otherwise we do this
 
-    //#define DEBUG
 #ifdef DEBUG
     fprintf(stderr, "pt a1 %d %ld\n", priv->last_frame, tframe);
 #endif
 
+#ifndef TEST_CACHING
     if (priv->pFrame != NULL) av_frame_unref(priv->pFrame);
+#endif
     priv->pFrame = NULL;
 
-    time = (double)tframe / cdata->fps;
-
-    target_pts = time * (double)AV_TIME_BASE;
-
     if (cdata->seek_flag & LIVES_SEEK_FAST) jump_frames = JUMP_FRAMES_FAST;
     else jump_frames = JUMP_FRAMES_SLOW;
 
@@ -1079,12 +1169,13 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
       seek_target = av_rescale_q(xtarget_pts, AV_TIME_BASE_Q, s->time_base);
       av_seek_frame(priv->ic, priv->vstream, seek_target, AVSEEK_FLAG_BACKWARD);
 #ifdef DEBUG
-      fprintf(stderr, "pt a2 %d %ld\n", priv->last_frame, seek_target);
+      fprintf(stderr, "new seek: %d %ld %ld\n", priv->last_frame, seek_target, priv->ic->start_time);
 #endif
       avcodec_flush_buffers(cc);
       priv->black_fill = FALSE;
       MyPts = -1;
       priv->needs_packet = TRUE;
+      //did_seek = TRUE;
     } else {
       MyPts = (priv->last_frame + 1.) / cdata->fps * (double)AV_TIME_BASE;
     }
@@ -1094,7 +1185,6 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
     do {
       if (priv->needs_packet) {
         do {
-
           ret = av_read_frame(priv->ic, &priv->packet);
 
 #ifdef DEBUG
@@ -1107,15 +1197,13 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
             priv->black_fill = TRUE;
             goto framedone;
           }
-
         } while (priv->packet.stream_index != priv->vstream);
-
       }
 
       if (MyPts == -1) {
         MyPts = priv->packet.pts;
         MyPts = av_rescale_q(MyPts, s->time_base, AV_TIME_BASE_Q) - priv->ic->start_time;
-        priv->found_pts = MyPts; // PTS of end of frame, set so start_time is zero
+        priv->found_pts = MyPts; // PTS of start of frame, set so start_time is zero
       }
 
 #ifdef DEBUG
@@ -1136,7 +1224,15 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 #endif
 
 #ifdef DEBUG
-      fprintf(stderr, "pt 1 %ld %d %ld\n", tframe, gotFrame, MyPts);
+      fprintf(stderr, "pt 1 %ld %d %ld %ld %d %d\n", tframe, gotFrame, MyPts, gotFrame ? priv->pFrame->best_effort_timestamp : 0,
+              priv->pFrame->color_trc, priv->pFrame->color_range);
+#endif
+
+#ifdef TEST_CACHING
+      if (gotFrame && priv->cachemax > 0) {
+        add_to_cache(priv, MyPts);
+        fprintf(stderr, "adding to cache: %ld %p %p\n", MyPts, priv->pFrame, priv->pFrame->data);
+      }
 #endif
 
       if (priv->packet.size == 0) {
@@ -1150,26 +1246,68 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
       // otherwise discard this frame
       if (gotFrame) {
         MyPts += (double)AV_TIME_BASE / cdata->fps;
+#ifndef TEST_CACHING
         av_frame_unref(priv->pFrame);
+#endif
         priv->pFrame = NULL;
       }
-
-
     } while (!(hit_target && gotFrame));
   }
 
 framedone:
   timex = get_current_ticks() - timex;
+
+  // TODO **: if we did a seek and had to decode multiple frames, set needs_calc; if seek direct don't
+  // then set fast depending on the time limit
   if (timex > FAST_SEEK_LIMIT)((lives_clip_data_t *)cdata)->seek_flag = LIVES_SEEK_NEEDS_CALCULATION;
+  //else if (did_seek) ((lives_clip_data_t *)cdata)->seek_flag = LIVES_SEEK_FAST;
 
   priv->last_frame = tframe;
 
+#ifdef TEST_CACHING
+framedone2:
+#endif
   if (priv->pFrame == NULL || pixel_data == NULL) return TRUE;
 
-  //height=cdata->height;
-
   if (priv->black_fill) btop = cdata->frame_height;
+  else {
+    // we are allowed to cast away const-ness just for this (and eventually gamma)
+    // yuv_subspace, yuv_clamping, yuv_sampling and interlace
 
+    if (priv->pFrame->interlaced_frame) {
+      if (priv->pFrame->top_field_first)((lives_clip_data_t *)cdata)->interlace = LIVES_INTERLACE_TOP_FIRST;
+      else ((lives_clip_data_t *)cdata)->interlace = LIVES_INTERLACE_BOTTOM_FIRST;
+    } else ((lives_clip_data_t *)cdata)->interlace = LIVES_INTERLACE_NONE;
+
+    ((lives_clip_data_t *)cdata)->YUV_sampling = WEED_YUV_SAMPLING_DEFAULT;
+
+    if (priv->pFrame->chroma_location == AVCHROMA_LOC_LEFT)
+      ((lives_clip_data_t *)cdata)->YUV_sampling = WEED_YUV_SAMPLING_JPEG;
+
+    if (priv->pFrame->chroma_location == AVCHROMA_LOC_CENTER)
+      ((lives_clip_data_t *)cdata)->YUV_sampling = WEED_YUV_SAMPLING_MPEG;
+
+    if (priv->pFrame->chroma_location == AVCHROMA_LOC_TOPLEFT)
+      ((lives_clip_data_t *)cdata)->YUV_sampling = WEED_YUV_SAMPLING_DVNTSC;
+
+    if (priv->pFrame->colorspace == AVCOL_SPC_BT709)
+      ((lives_clip_data_t *)cdata)->YUV_subspace = WEED_YUV_SUBSPACE_BT709;
+    else
+      ((lives_clip_data_t *)cdata)->YUV_subspace = WEED_YUV_SUBSPACE_YCBCR;
+
+
+    if (priv->pFrame->color_range == AVCOL_RANGE_JPEG)
+      ((lives_clip_data_t *)cdata)->YUV_clamping = WEED_YUV_CLAMPING_UNCLAMPED;
+    else
+      ((lives_clip_data_t *)cdata)->YUV_clamping = WEED_YUV_CLAMPING_CLAMPED;
+    y_black = (cdata->YUV_clamping == WEED_YUV_CLAMPING_CLAMPED) ? 16 : 0;
+
+#ifdef TEST_GAMMA
+    ((lives_clip_data_t *)cdata)->gamma = WEED_GAMMA_LINEAR;
+    if (priv->pFrame->color_trc == AVCOL_TRC_IEC61966_2_1)
+      ((lives_clip_data_t *)cdata)->gamma = WEED_GAMMA_SRGB;
+#endif
+  }
   for (p = 0; p < nplanes; p++) {
     dst = pixel_data[p];
     src = priv->pFrame->data[p];
@@ -1218,14 +1356,10 @@ framedone:
     }
   }
 
-
   return TRUE;
 }
 
 
-
-
-
 void clip_data_free(lives_clip_data_t *cdata) {
 
   if (cdata->palettes != NULL) free(cdata->palettes);
@@ -1238,7 +1372,6 @@ void clip_data_free(lives_clip_data_t *cdata) {
 
   free(cdata->priv);
 
-
   free(cdata);
 }
 
