@@ -5,12 +5,13 @@
 
 # (C) G. Finch 2011 - released under GPL v3 or higher
 
+use POSIX;     # Needed for setlocale()
+setlocale(LC_NUMERIC, "C");
 
 if (!defined($ARGV[0])) {exit 1;}
 
 my $command=$ARGV[0];
 
-
 if ($command eq "get_formats") {
     # list of formats we can handle, separated by |
     # see lives/src/plugins.h for values
@@ -21,8 +22,6 @@ if ($command eq "get_formats") {
     exit 0;
 }
 
-
-
 if ($command eq "check") {
     my $chkform=$ARGV[1];
     if ($chkform == 3) {
@@ -46,9 +45,6 @@ if ($command eq "check") {
     exit 1;
 }
 
-
-
-
 if ($command eq "play") {
     my $result;
     
@@ -88,11 +84,8 @@ if ($command eq "cleanup") {
     my $outfifo=$ARGV[2];
 
     unlink $outfifo;
-
 }
 
-
-
 ########################################33333333
 
 sub location {
