@@ -182,7 +182,7 @@ const int *get_audio_fmts() {
 }
 
 
-const char *get_init_rfx(void) {
+const char *get_init_rfx(int intention) {
   return \
          "<define>\\n\
 |1.7\\n\
