@@ -76,7 +76,7 @@ static int file_filter(const struct dirent *a) {
 #define MAX_DEVICES 65
 
 static char **get_vloopback2_devices(void) {
-  char devname[256];
+  char devname[512];
   struct dirent **namelist;
   int n;
   int fd;
@@ -182,7 +182,7 @@ const int *get_audio_fmts() {
 const char rfx[32768];
 
 
-const char *get_init_rfx(void) {
+const char *get_init_rfx(int intention) {
   char **vdevs = get_vloopback2_devices();
   char devstr[30000];
 
