@@ -31,6 +31,9 @@ static int package_version = 2; // version of this package
 #endif
 
 #include "../weed-utils-code.c" // optional
+
+#define NEED_RANDOM
+
 #include "../weed-plugin-utils.c" // optional
 
 /////////////////////////////////////////////////////////////
@@ -59,10 +62,10 @@ static int is_big_endian() {
   return 1;
 }
 
-
 // defines for configure dialog elements
 enum DlgControls {
   P_TEXT = 0,
+  P_RAND,
   P_MODE,
   P_END
 };
@@ -73,7 +76,6 @@ typedef struct {
   int blue;
 } rgb_t;
 
-
 typedef enum {
   TEXT_TYPE_ASCII, // 1 byte charset [DEFAULT]
   TEXT_TYPE_UTF8, //  1 - 4 byte charset
@@ -81,7 +83,6 @@ typedef enum {
   TEXT_TYPE_UTF32 // 4 byte charset
 } pt_ttext_t;
 
-
 typedef enum {
   PT_LETTER_MODE, // proctext() is called per letter xtext from start -> start+length-1
   PT_WORD_MODE, // proctext() is called with per word xtext from start -> start+length-1
@@ -89,7 +90,6 @@ typedef enum {
   PT_ALL_MODE // proctext called once with NULL xtext pointing to sdata->text (or NULL if length is 0)
 } pt_tmode_t;
 
-
 typedef enum {
   PT_SPIRAL_TEXT = 0,
   PT_SPINNING_LETTERS,
@@ -97,13 +97,9 @@ typedef enum {
   PT_WORD_COALESCE
 } pt_op_mode_t;
 
-
 // for future use
 #define NTHREADS 1
 
-
-
-
 typedef struct {
   int size;
 
@@ -128,10 +124,6 @@ typedef struct {
 
 } pt_letter_data_t;
 
-
-
-
-
 // static data per instance
 typedef struct {
   int count; // proctext counter
@@ -142,13 +134,18 @@ typedef struct {
 
   gboolean alarm; // event wake up
 
-  off_t start; // start glyph (inclusive) in text (0 based) for string/word/letter modes
-  int64_t length; // length of string in text [0 to all] for string/word/letter modes
+  off_t start; // start glyph (inclusive) in current string (0 based) for string/word/letter modes
+  int64_t length; // length of substring in current string [0 to all] for string/word/letter modes
 
   pt_ttext_t text_type;
-  char *text; // text loaded in from file
+
+  int nstrings; // number of strings loaded
+  int cstring; // currently selected string
+  char **strings; // text loaded in from file
+  char *text; // shortcut for strings[cstring]
   size_t tlength; // length of text in characters
   int wlength; // length in words
+  gboolean rndorder; // true to select random strings, otherwise they are selected in order
 
   // offsets of text in layer (0,0) == top left
   int x_text;
@@ -177,21 +174,21 @@ typedef struct {
 
   // per glyph/mode private data
   pt_letter_data_t *letter_data;
+
+  guchar *pixel_data;
+
 } sdata_t;
 
-
-
-
 typedef struct {
   size_t start;
   size_t length;
 } pt_subst_t;
 
-
 /////////////////////////////////////////////
 static int unal[256][256];
 static int al[256][256];
 
+
 static void init_unal(void) {
   // premult to postmult and vice-versa
 
@@ -205,6 +202,7 @@ static void init_unal(void) {
   }
 }
 
+
 static void alpha_unpremult(guchar *ptr, int width, int height, int rowstride, int pal, int un) {
   int aoffs, coffs, psizel;
   int alpha;
@@ -248,143 +246,16 @@ static void alpha_unpremult(guchar *ptr, int width, int height, int rowstride, i
       ptr += rowstride;
     }
   }
-
-}
-
-
-
-static cairo_t *channel_to_cairo(weed_plant_t *channel) {
-  // convert a weed channel to cairo
-  // the channel shares pixel_data with cairo
-  // so it should be copied before the cairo is destroyed
-
-  // "width","rowstrides" and "current_palette" of channel may all change
-
-  int irowstride, orowstride;
-  int width, widthx;
-  int height;
-  int pal;
-  int error;
-  int flags = 0;
-
-  register int i;
-
-  guchar *src, *dst, *pixel_data;
-
-  cairo_surface_t *surf;
-  cairo_t *cairo;
-  cairo_format_t cform = CAIRO_FORMAT_ARGB32;
-
-  width = weed_get_int_value(channel, "width", &error);
-  height = weed_get_int_value(channel, "height", &error);
-  pal = weed_get_int_value(channel, "current_palette", &error);
-  irowstride = weed_get_int_value(channel, "rowstrides", &error);
-
-  widthx = width * 4;
-
-  orowstride = cairo_format_stride_for_width(cform, width);
-
-  src = (guchar *)weed_get_voidptr_value(channel, "pixel_data", &error);
-
-  pixel_data = (guchar *)weed_malloc(height * orowstride);
-
-  if (pixel_data == NULL) return NULL;
-
-  if (irowstride == orowstride) {
-    weed_memcpy((void *)pixel_data, (void *)src, irowstride * height);
-  } else {
-    dst = pixel_data;
-    for (i = 0; i < height; i++) {
-      weed_memcpy((void *)dst, (void *)src, widthx);
-      weed_memset((void *)dst + widthx, 0, widthx - orowstride);
-      dst += orowstride;
-      src += irowstride;
-    }
-  }
-
-  if (weed_plant_has_leaf(channel, "flags")) flags = weed_get_int_value(channel, "flags", &error);
-  if (!(flags & WEED_CHANNEL_ALPHA_PREMULT)) {
-    // if we have post-multiplied alpha, pre multiply
-    alpha_unpremult(pixel_data, widthx, height, orowstride, pal, FALSE);
-  }
-
-  surf = cairo_image_surface_create_for_data(pixel_data,
-         cform,
-         width, height,
-         orowstride);
-
-  if (surf == NULL) {
-    weed_free(pixel_data);
-    return NULL;
-  }
-
-  cairo = cairo_create(surf);
-  cairo_surface_destroy(surf);
-  weed_free(pixel_data);
-
-  return cairo;
-}
-
-
-
-static void cairo_to_channel(cairo_t *cairo, weed_plant_t *channel) {
-  // updates a weed_channel from a cairo_t
-  cairo_surface_t *surface = cairo_get_target(cairo);
-
-  cairo_format_t cform = CAIRO_FORMAT_ARGB32;
-
-  int error;
-
-  guchar *src, *dst, *pixel_data = (guchar *)weed_get_voidptr_value(channel, "pixel_data", &error);
-
-  int flags = 0;
-
-  int height = weed_get_int_value(channel, "height", &error);
-  int irowstride, orowstride = weed_get_int_value(channel, "rowstrides", &error);
-  int width = weed_get_int_value(channel, "width", &error), widthx = width * 4;
-
-  register int i;
-
-  // flush to ensure all writing to the image was done
-  cairo_surface_flush(surface);
-
-  src = cairo_image_surface_get_data(surface);
-
-  irowstride = cairo_format_stride_for_width(cform, width);
-
-  if (irowstride == orowstride) {
-    weed_memcpy((void *)pixel_data, (void *)src, irowstride * height);
-  } else {
-    dst = pixel_data;
-    for (i = 0; i < height; i++) {
-      weed_memcpy((void *)dst, (void *)src, widthx);
-      weed_memset((void *)(dst + widthx), 0, widthx - orowstride);
-      dst += orowstride;
-      src += irowstride;
-    }
-  }
-
-
-  if (weed_plant_has_leaf(channel, "flags")) flags = weed_get_int_value(channel, "flags", &error);
-  if (!(flags & WEED_CHANNEL_ALPHA_PREMULT)) {
-    int pal = weed_get_int_value(channel, "current_palette", &error);
-
-    // un-premultiply the alpha
-    alpha_unpremult(pixel_data, widthx, height, orowstride, pal, TRUE);
-  }
-
-
 }
 
 
 static char **fonts_available = NULL;
 static int num_fonts_available = 0;
 
-// work around some **weirdness** with strndup
 static char *stringdup(const char *s, size_t n) {
   char *ret;
   size_t len = strlen(s);
-  if (len >= n) len = n - 1;
+  if (len > n) len = n;
   ret = weed_malloc(len + 1);
   weed_memcpy(ret, s, len);
   weed_memset(ret + len, 0, 1);
@@ -392,7 +263,6 @@ static char *stringdup(const char *s, size_t n) {
 }
 
 
-
 static size_t utf8len(char *text) {
   register size_t toffs = 0;
   while (text[toffs] != '\0') {
@@ -449,6 +319,145 @@ static size_t get_utf8_word_length(char *text) {
 }
 
 
+static void getastring(sdata_t *sdata) {
+  if (!sdata->rndorder) {
+    sdata->cstring++;
+    if (sdata->cstring >= sdata->nstrings) sdata->cstring = 0;
+  } else {
+    sdata->cstring = (int)drand((double)sdata->nstrings);
+  }
+
+  sdata->text = sdata->strings[sdata->cstring];
+
+  if (sdata->text_type == TEXT_TYPE_ASCII) {
+    sdata->tlength = strlen(sdata->text);
+    sdata->wlength = get_ascii_word_length(sdata->text);
+  } else {
+    sdata->tlength = utf8len(sdata->text);
+    sdata->wlength = get_utf8_word_length(sdata->text);
+  }
+}
+
+
+static cairo_t *channel_to_cairo(sdata_t *sdata, weed_plant_t *channel) {
+  // convert a weed channel to cairo
+  // the channel shares pixel_data with cairo
+  // so it should be copied before the cairo is destroyed
+
+  // "width","rowstrides" and "current_palette" of channel may all change
+
+  int irowstride, orowstride;
+  int width, widthx;
+  int height;
+  int pal;
+  int error;
+  int flags = 0;
+
+  register int i;
+
+  guchar *src, *dst, *pixel_data;
+
+  cairo_surface_t *surf;
+  cairo_t *cairo;
+  cairo_format_t cform = CAIRO_FORMAT_ARGB32;
+
+  width = weed_get_int_value(channel, "width", &error);
+  height = weed_get_int_value(channel, "height", &error);
+  pal = weed_get_int_value(channel, "current_palette", &error);
+  irowstride = weed_get_int_value(channel, "rowstrides", &error);
+
+  widthx = width * 4;
+
+  orowstride = cairo_format_stride_for_width(cform, width);
+
+  src = (guchar *)weed_get_voidptr_value(channel, "pixel_data", &error);
+
+  sdata->pixel_data = pixel_data = (guchar *)weed_malloc(height * orowstride);
+
+  if (pixel_data == NULL) return NULL;
+
+  if (irowstride == orowstride) {
+    weed_memcpy((void *)pixel_data, (void *)src, irowstride * height);
+  } else {
+    dst = pixel_data;
+    for (i = 0; i < height; i++) {
+      weed_memcpy((void *)dst, (void *)src, widthx);
+      weed_memset((void *)dst + widthx, 0, widthx - orowstride);
+      dst += orowstride;
+      src += irowstride;
+    }
+  }
+
+  if (weed_plant_has_leaf(channel, "flags")) flags = weed_get_int_value(channel, "flags", &error);
+  if (!(flags & WEED_CHANNEL_ALPHA_PREMULT)) {
+    // if we have post-multiplied alpha, pre multiply
+    alpha_unpremult(pixel_data, widthx, height, orowstride, pal, FALSE);
+  }
+
+  surf = cairo_image_surface_create_for_data(pixel_data,
+         cform,
+         width, height,
+         orowstride);
+
+  if (surf == NULL) {
+    weed_free(pixel_data);
+    return NULL;
+  }
+
+  cairo = cairo_create(surf);
+  cairo_surface_destroy(surf);
+
+  return cairo;
+}
+
+
+static void cairo_to_channel(cairo_t *cairo, weed_plant_t *channel) {
+  // updates a weed_channel from a cairo_t
+  cairo_surface_t *surface = cairo_get_target(cairo);
+
+  cairo_format_t cform = CAIRO_FORMAT_ARGB32;
+
+  int error;
+
+  guchar *src, *dst, *pixel_data = (guchar *)weed_get_voidptr_value(channel, "pixel_data", &error);
+
+  int flags = 0;
+
+  int height = weed_get_int_value(channel, "height", &error);
+  int irowstride, orowstride = weed_get_int_value(channel, "rowstrides", &error);
+  int width = weed_get_int_value(channel, "width", &error), widthx = width * 4;
+
+  register int i;
+
+  // flush to ensure all writing to the image was done
+  cairo_surface_flush(surface);
+
+  src = cairo_image_surface_get_data(surface);
+
+  irowstride = cairo_format_stride_for_width(cform, width);
+
+  if (irowstride == orowstride) {
+    weed_memcpy((void *)pixel_data, (void *)src, irowstride * height);
+  } else {
+    dst = pixel_data;
+    for (i = 0; i < height; i++) {
+      weed_memcpy((void *)dst, (void *)src, widthx);
+      weed_memset((void *)(dst + widthx), 0, widthx - orowstride);
+      dst += orowstride;
+      src += irowstride;
+    }
+  }
+
+  if (weed_plant_has_leaf(channel, "flags")) flags = weed_get_int_value(channel, "flags", &error);
+  if (!(flags & WEED_CHANNEL_ALPHA_PREMULT)) {
+    int pal = weed_get_int_value(channel, "current_palette", &error);
+
+    // un-premultiply the alpha
+    alpha_unpremult(pixel_data, widthx, height, orowstride, pal, TRUE);
+  }
+}
+
+
 static pt_subst_t *get_nth_word_utf8(char *text, int idx) {
   // get nth word, return start (bytes) and length (bytes)
   // idx==0 for first word, etc
@@ -509,8 +518,6 @@ static pt_subst_t *get_nth_word_ascii(char *text, int idx) {
 }
 
 
-
-
 static void getlsize(PangoLayout *layout, double *pw, double *ph) {
   // calculate width and height of layout
   int w_, h_;
@@ -536,10 +543,10 @@ static void setxypos(double dwidth, double dheight, double x, double y, int *x_t
 }
 
 
-
-
 // set font size
 static void set_font_size(PangoLayout *layout, PangoFontDescription *font, int font_size) {
+  font_size = (int)((double)(font_size + 1.) / 4.) * 4;
+
   pango_font_description_set_size(font, font_size * PANGO_SCALE);
   pango_layout_set_font_description(layout, font);
 }
@@ -565,6 +572,7 @@ static void colour_copy(rgb_t *col1, rgb_t *col2) {
   weed_memcpy(col1, col2, sizeof(rgb_t));
 }
 
+
 static inline double rand_angle(void) {
   // return a random double between 0. and 2*M_PI
   return (double)rand() / (double)RAND_MAX * 2.*M_PI;
@@ -587,6 +595,7 @@ static void letter_data_free(sdata_t *sdata) {
   sdata->letter_data = NULL;
 }
 
+
 static void rotate_text(cairo_t *cairo, PangoLayout *layout, int x_center, int y_center, double radians) {
   cairo_translate(cairo, x_center, y_center);
   cairo_rotate(cairo, radians);
@@ -612,9 +621,10 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
 
     if (sdata->start > -1) {
       set_font_size(layout, font, 64);
-      // get pixel size of word
-      getlsize(layout, &dwidth, &dheight);
 
+      // get pixel size of word
+      pango_layout_set_text(layout, (char *)xtext, -1);
+      getlsize(layout, &dwidth, &dheight);
       setxypos(dwidth, dheight, width / 2, height / 2, &sdata->x_text, &sdata->y_text);
 
       sdata->dbl1 -= sdata->dbl1 / 3.5 + .5; // blur shrink
@@ -636,6 +646,8 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
           sdata->start = 0;
           sdata->length = 0;
           sdata->dbl1 = 0.;
+          if (!sdata->rndorder) getastring(sdata);
+          // time before next repeat
           pt_set_alarm(sdata, 1000); // milliseconds
         }
       } else {
@@ -645,6 +657,7 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
 
       if (sdata->length > 0) {
         if (sdata->start == sdata->wlength - 1) {
+          // time between each phrase
           pt_set_alarm(sdata, 1000); // milliseconds
         } else {
           // peek at last char of next word
@@ -658,10 +671,11 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
 
           weed_free(xsubst);
 
-          if (nxlast == '.' || nxlast == '!' || nxlast == '?') pt_set_alarm(sdata, 1000); // milliseconds
-          else if (nxlast == ',') pt_set_alarm(sdata, 700); // milliseconds
-          else if (nxlast == ';') pt_set_alarm(sdata, 800); // milliseconds
-          else pt_set_alarm(sdata, 500); // milliseconds
+          // hold time
+          if (nxlast == '.' || nxlast == '!' || nxlast == '?') pt_set_alarm(sdata, 4000); // milliseconds
+          else if (nxlast == ',') pt_set_alarm(sdata, 2800); // milliseconds
+          else if (nxlast == ';') pt_set_alarm(sdata, 1600); // milliseconds
+          else pt_set_alarm(sdata, 1000); // milliseconds
         }
       }
     }
@@ -669,10 +683,13 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
     break;
 
   case (PT_LETTER_STARFIELD):
+    // LETTER_MODE:
+    // this is called for each letter from sdata->start to sdata->start + sdata->length - 1
+    // with sdata->count set to the current letter number
+
     if (sdata->timer == 0.) {
       sdata->start = 0;
       sdata->length = 0;
-
       sdata->tmode = PT_LETTER_MODE;
 
       sdata->letter_data = letter_data_create(sdata->tlength);
@@ -699,7 +716,6 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
         colour_copy(&sdata->fg, &ldt->colour);
 
       } else pango_layout_set_text(layout, "", -1);
-
     }
 
     if (sdata->alarm) {
@@ -732,10 +748,13 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
       else pt_set_alarm(sdata, -1);
     }
 
-
     break;
 
   case (PT_SPINNING_LETTERS):
+    // LETTER_MODE:
+    // this is called for each letter from sdata->start to sdata->start + sdata->length - 1
+    // with sdata->count set to the current letter number
+
     if (sdata->timer == 0.) {
       sdata->int1 = 0;
 
@@ -767,7 +786,7 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
     // get pixel size of letter/word
     getlsize(layout, &dwidth, &dheight);
 
-    sdata->dbl1 += dwidth + 10.;
+    sdata->dbl1 += dwidth + 50.;
 
     setxypos(0, dheight, sdata->x_text, sdata->y_text, &sdata->x_text, &sdata->y_text);
 
@@ -783,6 +802,9 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
     break;
 
   case (PT_SPIRAL_TEXT):
+    // LETTER_MODE:
+    // this is called for each letter from sdata->start to sdata->start + sdata->length - 1
+    // with sdata->count set to the current letter number
 
     if (sdata->timer == 0.) {
       sdata->int1 = 0;
@@ -791,6 +813,7 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
     }
 
     set_font_size(layout, font, 2560. / (sdata->count + 19.));
+    pango_layout_set_text(layout, xtext, -1);
 
     // get pixel size of letter/word
     getlsize(layout, &dwidth, &dheight);
@@ -826,11 +849,13 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
     if (!strncmp(xtext, ".", 1)) sdata->int1++;
 
     if (sdata->alarm) {
-      pt_set_alarm(sdata, 250); // milliseconds
+      pt_set_alarm(sdata, 80); // milliseconds to get next letter
       if (sdata->start + sdata->length < sdata->tlength) {
         // add an extra letter
         sdata->length++;
       } else {
+        // no more letters...
+        // shrink from head to tail
         sdata->length -= 2;
         sdata->start += 2;
         sdata->dbl3 -= .0002;
@@ -838,46 +863,56 @@ static void proctext(sdata_t *sdata, weed_timecode_t tc, char *xtext, cairo_t *c
         sdata->dbl1 = sdata->dbl1 * .97;
         sdata->dbl2 = radY = sdata->dbl2 * .97;
 
-        // trip to lissajou
+        // trip to lissajou (unused for now)
         if (!sdata->bool1) sdata->bool1 = TRUE;
 
         if (sdata->length <= 0) {
+          // all letters gone - restart cycle
+          if (!sdata->rndorder) getastring(sdata);
           sdata->length = 1;
           sdata->start = 0;
           sdata->bool1 = FALSE;
-          sdata->dbl3 = -sdata->dbl3 / 12.;
-        } else pt_set_alarm(sdata, 50); // milliseconds
-
+          //sdata->dbl3 = -sdata->dbl3 / 12.;
+          sdata->int1 = 0;
+        } else pt_set_alarm(sdata, 40); // milliseconds (string disappearing)
       }
-
     }
     break;
-
   } // end switch
-
 }
 
 
 int puretext_init(weed_plant_t *inst) {
-  int error, fd;
+  int error, fd, canstart;
+
   gboolean erropen = FALSE;
+
   weed_plant_t **in_params = weed_get_plantptr_array(inst, "in_parameters", &error);
+
   sdata_t *sdata;
 
   char buff[65536];
+
   size_t b_read;
 
   char *textfile = weed_get_string_value(in_params[P_TEXT], "value", &error);
 
+  register int i, j = 0;
+
+  weed_free(in_params);
+
   // open file and read in text
 
   if ((fd = open(textfile, O_RDONLY)) == -1) {
     erropen = TRUE;
-    g_snprintf(buff, 512, "Error opening file %s", textfile);
+    fprintf(stderr, "Error opening file %s\n", textfile);
+    weed_free(textfile);
+    return WEED_ERROR_INIT_ERROR;
   }
 
   weed_free(textfile);
-  weed_free(in_params);
+
+  seed_rand();
 
   sdata = (sdata_t *)weed_malloc(sizeof(sdata_t));
 
@@ -888,6 +923,8 @@ int puretext_init(weed_plant_t *inst) {
   sdata->alarm_time = 0.;
   sdata->alarm = FALSE;
 
+  sdata->pixel_data = NULL;
+
   sdata->text_type = TEXT_TYPE_UTF8;
 
   if (!erropen) {
@@ -896,17 +933,48 @@ int puretext_init(weed_plant_t *inst) {
     close(fd);
   }
 
-  sdata->text = strdup(buff);
+  canstart = 0;
+  sdata->nstrings = 0;
+
+  // parse the text
+  for (i = 0; i < b_read; i++) {
+    if ((uint8_t)buff[i] == 0x0A || (uint8_t)buff[i] == 0x0D) {
+      if (canstart < i) sdata->nstrings++;
+      canstart = i + 1;
+    }
+  }
+  if (canstart < i) sdata->nstrings++;
+
+  if (sdata->nstrings == 0) {
+    fprintf(stderr, "No strings found in file.\n");
+    return WEED_ERROR_INIT_ERROR;
+  }
+
+  sdata->strings = (char **)weed_malloc(sdata->nstrings * sizeof(char *));
+
+  canstart = 0;
+
+  for (i = 0; i < sdata->nstrings; i++) {
+    // parse the text
+    for (; j < b_read; j++) {
+      if ((uint8_t)buff[j] == 0x0A || (uint8_t)buff[i] == 0x0D) {
+        if (canstart < j) {
+          sdata->strings[i] = stringdup(&buff[canstart], j - canstart);
+          canstart = j + 1;
+          break;
+        }
+        canstart = j + 1;
+      }
+    }
+    if (canstart < j) {
+      sdata->strings[i] = stringdup(&buff[canstart], j - canstart);
+    }
+  }
 
   sdata->start = sdata->length = 0;
-
-  if (sdata->text_type == TEXT_TYPE_ASCII) {
-    sdata->tlength = strlen(sdata->text);
-    sdata->wlength = get_ascii_word_length(sdata->text);
-  } else {
-    sdata->tlength = utf8len(sdata->text);
-    sdata->wlength = get_utf8_word_length(sdata->text);
-  }
+  sdata->cstring = -1;
+  sdata->text = NULL;
+  sdata->rndorder = TRUE;
 
   sdata->int1 = 0;
   sdata->dbl1 = sdata->dbl2 = sdata->dbl3 = -1.;
@@ -922,23 +990,22 @@ int puretext_init(weed_plant_t *inst) {
 }
 
 
-
 int puretext_deinit(weed_plant_t *inst) {
   int error;
+  int i;
   sdata_t *sdata = (sdata_t *)weed_get_voidptr_value(inst, "plugin_internal", &error);
 
   if (sdata != NULL) {
     if (sdata->letter_data != NULL) letter_data_free(sdata);
-    if (sdata->text != NULL) free(sdata->text);
-    free(sdata);
+    for (i = 0; i < sdata->nstrings; i++) weed_free(sdata->strings[i]);
+    weed_free(sdata->strings);
+    weed_free(sdata);
   }
 
   return WEED_NO_ERROR;
-
 }
 
 
-
 int puretext_process(weed_plant_t *inst, weed_timecode_t tc) {
   int error;
 
@@ -969,6 +1036,7 @@ int puretext_process(weed_plant_t *inst, weed_timecode_t tc) {
 
   register int i, j;
 
+  sdata->rndorder = weed_get_boolean_value(in_params[P_RAND], "value", &error);
 
   weed_free(in_params); // must weed free because we got an array
 
@@ -1003,17 +1071,16 @@ int puretext_process(weed_plant_t *inst, weed_timecode_t tc) {
     weed_memcpy(bgdata, dst, height * orowstride);
   }
 
-
-
   // THINGS TO TO WITH TEXTS AND PANGO
   if ((!in_channel) || (in_channel == out_channel))
-    cairo = channel_to_cairo(out_channel);
+    cairo = channel_to_cairo(sdata, out_channel);
   else
-    cairo = channel_to_cairo(in_channel);
+    cairo = channel_to_cairo(sdata, in_channel);
+
+  if (sdata->text == NULL) getastring(sdata);
 
   if (cairo) {
     // TODO - get real offset of start in bytes
-
     if (sdata->text_type == TEXT_TYPE_ASCII) {
       toffs = sdata->start;
     } else {
@@ -1027,13 +1094,6 @@ int puretext_process(weed_plant_t *inst, weed_timecode_t tc) {
         PangoFontDescription *font;
         char *xtext;
 
-        font = pango_font_description_new();
-
-        // send letter or word to proctext
-
-        //if((num_fonts_available) && (fontnum >= 0) && (fontnum < num_fonts_available) && (fonts_available[fontnum]))
-        pango_font_description_set_family(font, "Serif");
-
         if (sdata->length == 0) {
           xtext = weed_malloc(1);
           weed_memset(xtext, 0, 1);
@@ -1069,60 +1129,65 @@ int puretext_process(weed_plant_t *inst, weed_timecode_t tc) {
             break;
           }
         }
+
+        font = pango_font_description_new();
+        pango_font_description_set_family(font, "Serif");
         pango_layout_set_font_description(layout, font);
-        pango_layout_set_text(layout, (char *)xtext, -1);
 
         // default colour - opaque white
         sdata->fg.red = sdata->fg.green = sdata->fg.blue = 255.;
-        sdata->fg_alpha = 1.;
+        sdata->fg_alpha = .5;
 
         cairo_save(cairo);
 
         // get size, position, and colour
         proctext(sdata, tc, (char *)xtext, cairo, layout, font, width, height);
 
-        free(xtext);
-
         cairo_move_to(cairo, sdata->x_text, sdata->y_text);
 
         cairo_set_source_rgba(cairo, sdata->fg.red / 255.0, sdata->fg.green / 255.0, sdata->fg.blue / 255.0,
                               sdata->fg_alpha);
 
+        pango_layout_set_text(layout, (char *)xtext, -1);
+
         pango_cairo_show_layout(cairo, layout);
 
         cairo_restore(cairo);
 
         pango_font_description_free(font);
         g_object_unref(layout);
+        weed_free(xtext);
       }
 
       sdata->count++;
-
     } // end loop
 
     cairo_to_channel(cairo, out_channel);
+    if (sdata->pixel_data != NULL) {
+      weed_free(sdata->pixel_data);
+      sdata->pixel_data = NULL;
+    }
     cairo_destroy(cairo);
   }
 
-
   if (sdata->mode == PT_WORD_COALESCE) {
     if (sdata->dbl1 > 0.) {
       guchar *b_data = bgdata;
-      int width3 = width * 3;
+      int width4 = width * 4;
       guchar *dstx = dst = weed_get_voidptr_value(out_channel, "pixel_data", &error);
 
       for (i = 0; i < height; i++) {
-        for (j = 0; j < width3; j += 3) {
+        for (j = 0; j < width4; j += 4) {
           if (dst[j] != b_data[j] || dst[j + 1] != b_data[j + 1] || dst[j + 2] != b_data[j + 2]) {
             // move point by sdata->dbl1 pixels
             double angle = rand_angle();
-            int x = j / 3 + sin(angle) * sdata->dbl1;
+            int x = j / 4 + sin(angle) * sdata->dbl1;
             int y = i + cos(angle) * sdata->dbl1;
             if (x > 0 && x < width && y > 0 && y < height) {
               // blur 1 pixel
-              memcpy(&dstx[y * orowstride + x * 3], &dst[j], 3);
+              memcpy(&dstx[y * orowstride + x * 4], &dst[j], 3);
               // protect blurred pixel
-              if (y >= i) memcpy(&bgdata[y * orowstride + x * 3], &dst[j], 3);
+              if (y >= i) memcpy(&bgdata[y * orowstride + x * 4], &dst[j], 3);
             }
             // replace original pixel
             memcpy(&dst[j], &b_data[j], 3);
@@ -1138,10 +1203,6 @@ int puretext_process(weed_plant_t *inst, weed_timecode_t tc) {
   return WEED_NO_ERROR;
 }
 
-
-
-
-
 /*
 static int font_compare(const void *p1, const void *p2) {
   const char *s1 = (const char *)(*(char **)p1);
@@ -1188,7 +1249,6 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 
     init_unal();
 
-
     // this section contains code
     // for configure fonts available
     num_fonts_available = 0;
@@ -1230,6 +1290,8 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
     flags |= WEED_PARAMETER_REINIT_ON_VALUE_CHANGE;
     weed_set_int_value(in_params[P_TEXT], "flags", flags);
 
+    in_params[P_RAND] = weed_switch_init("rand", "Select text lines _randomly", WEED_TRUE);
+
     in_params[P_MODE] = weed_string_list_init("mode", "Effect _mode", 0, modes);
     flags = 0;
     if (weed_plant_has_leaf(in_params[P_MODE], "flags"))
@@ -1251,25 +1313,21 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
     weed_plugin_info_add_filter_class(plugin_info, filter_class);
 
     weed_set_int_value(plugin_info, "version", package_version);
-
   }
 
   return plugin_info;
 }
 
 
-
 void weed_desetup(void) {
   // clean up what we reserve for font family names
   if (num_fonts_available && fonts_available) {
     int i;
     for (i = 0; i < num_fonts_available; ++i) {
-      free((void *)fonts_available[i]);
+      weed_free((void *)fonts_available[i]);
     }
     weed_free((void *)fonts_available);
   }
   num_fonts_available = 0;
   fonts_available = NULL;
 }
-
-
