@@ -34,18 +34,19 @@ static int package_version = 1; // version of this package
 #include "weed-utils-code.c" // optional
 #include "weed-plugin-utils.c" // optional
 
-
 /////////////////////////////////////////////////////////////
 
 #define USE_DBLBUF 1
 
-
 #include <libprojectM/projectM.hpp>
 
 #include <GL/gl.h>
 
-#include <SDL/SDL.h>
+#include <SDL.h>
+
+#ifndef HAVE_SDL2
 #include <SDL_syswm.h>
+#endif
 
 #include <pthread.h>
 
@@ -54,19 +55,21 @@ static int package_version = 1; // version of this package
 #include <sys/time.h>
 
 #include <errno.h>
+#include <unistd.h>
 
 #include "projectM-ConfigFile.h"
 #include "projectM-getConfigFilename.h"
 
-#define TARGET_FPS 30.
+#define TARGET_FPS 35.
+#define MESHSIZE 128
+#define DEF_TEXTURESIZE 1024
 
 static int copies = 0;
 
-static pthread_cond_t cond;
-static pthread_mutex_t cond_mutex;
+static pthread_cond_t cond  = PTHREAD_COND_INITIALIZER;
+static pthread_mutex_t cond_mutex = PTHREAD_MUTEX_INITIALIZER;
 static struct timespec ts;
 
-
 typedef struct {
   projectM *globalPM;
   GLubyte *fbuffer;
@@ -77,7 +80,7 @@ typedef struct {
   volatile int pidx;
   int opidx;
   volatile int nprs;
-  volatile char **prnames;
+  char **volatile prnames;  // volatile ptr to non-volatile strings !!
   pthread_mutex_t mutex;
   pthread_mutex_t pcm_mutex;
   pthread_t thread;
@@ -88,15 +91,19 @@ typedef struct {
   volatile bool failed;
   volatile bool update_size;
   volatile bool rendering;
+#ifdef HAVE_SDL2
+  SDL_Window *win;
+  SDL_GLContext glCtx;
+#endif
 } _sdata;
 
-
 static _sdata *statsd;
 
 static int maxwidth, maxheight;
 
 static int inited = 0;
 
+#ifndef HAVE_SDL2
 static void winhide() {
   SDL_SysWMinfo info;
 
@@ -119,7 +126,6 @@ static void winhide() {
 }
 
 
-
 static int resize_display(int width, int height) {
   int flags = SDL_OPENGL | SDL_HWSURFACE | SDL_RESIZABLE;
 
@@ -133,21 +139,23 @@ static int resize_display(int width, int height) {
 
   return 0;
 }
-
+#endif
 
 static int change_size(_sdata *sdata) {
-  int ret;
+  int ret = 0;
   sdata->globalPM->projectM_resetGL(sdata->width, sdata->height);
   if (sdata->fbuffer != NULL) weed_free(sdata->fbuffer);
+#ifdef HAVE_SDL2
+  SDL_SetWindowSize(sdata->win, sdata->width, sdata->height);
+#else
   ret = resize_display(sdata->width, sdata->height);
+#endif
   sdata->fbuffer = (GLubyte *)weed_malloc(sizeof(GLubyte) * sdata->width * sdata->height * 3);
   return ret;
 }
 
 
 static int init_display(_sdata *sd) {
-  const SDL_VideoInfo *info;
-
   int defwidth = sd->width;
   int defheight = sd->height;
 
@@ -159,7 +167,13 @@ static int init_display(_sdata *sd) {
   }
 
   /* Let's get some video information. */
-  info = SDL_GetVideoInfo();
+#ifdef HAVE_SDL2
+  SDL_Rect rect;
+  SDL_GetDisplayBounds(0, &rect);
+  maxwidth = rect.w;
+  maxheight = rect.h;
+#else
+  const SDL_VideoInfo *info = SDL_GetVideoInfo();
   if (!info) {
     /* This should probably never happen. */
     fprintf(stderr, "Video query failed: %s\n",
@@ -167,27 +181,31 @@ static int init_display(_sdata *sd) {
 
     return 2;
   }
-
-  //printf("Screen Resolution: %d x %d\n", info->current_w, info->current_h);
   maxwidth = info->current_w;
   maxheight = info->current_h;
+#endif
+  printf("Screen Resolution: %d x %d\n", maxwidth, maxheight);
 
-  if (defwidth > maxwidth) defwidth = maxwidth;
-  if (defheight > maxheight) defheight = maxheight;
+  // if (defwidth > maxwidth) defwidth = maxwidth;
+  // if (defheight > maxheight) defheight = maxheight;
 
   SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
   SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
   SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, USE_DBLBUF);
 
+#ifdef HAVE_SDL2
+  sd->win = SDL_CreateWindow("projectM", SDL_WINDOWPOS_UNDEFINED , SDL_WINDOWPOS_UNDEFINED, defwidth, defheight,
+                             SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE | SDL_WINDOW_HIDDEN);
+  sd->glCtx = SDL_GL_CreateContext(sd->win);
+#else
   if (resize_display(defwidth, defheight)) return 3;
-
+#endif
   //if (change_size(sd)) return 4;
 
   return 0;
 }
 
 
-
 static int render_frame(_sdata *sdata) {
   sdata->globalPM->renderFrame();
 
@@ -235,7 +253,11 @@ static int render_frame(_sdata *sdata) {
 #if USE_DBLBUF
   glReadPixels(0, 0, sdata->width, sdata->height, GL_RGB, GL_UNSIGNED_BYTE, sdata->fbuffer);
   pthread_mutex_lock(&sdata->mutex);
+#ifdef HAVE_SDL2
+  SDL_GL_SwapWindow(sdata->win);
+#else
   SDL_GL_SwapBuffers();
+#endif
   pthread_mutex_unlock(&sdata->mutex);
 #else
   pthread_mutex_lock(&sdata->mutex);
@@ -258,10 +280,8 @@ static void do_exit(void) {
 
 
 static void *worker(void *data) {
-  std::string config_filename = getConfigFilename();
   std::string prname;
 
-  ConfigFile config(config_filename);
   projectM::Settings settings;
 
   bool rerand = true;
@@ -270,36 +290,51 @@ static void *worker(void *data) {
 
   register int i = 0;
 
+  float hwratio = (float)sd->height / (float)sd->width;
+
   if (init_display(sd)) {
-    //sd->worker_ready=true;
     sd->failed = true;
+    sd->worker_ready = true;
 
     // tell main thread we are ready
     pthread_mutex_lock(&cond_mutex);
     pthread_cond_signal(&cond);
-    pthread_mutex_lock(&cond_mutex);
-
-    goto fail;
+    pthread_mutex_unlock(&cond_mutex);
+    SDL_Quit();
+    return NULL;
   }
 
-
   atexit(do_exit);
 
-  // can fail here
-  sd->globalPM = new projectM(config_filename);
+  settings.windowWidth = sd->width;
+  settings.windowHeight = sd->height;
+  settings.meshX = MESHSIZE;
+  settings.meshY = settings.meshX * hwratio;
+  settings.fps = sd->fps;
+  settings.smoothPresetDuration = 2;
+  settings.presetDuration = 10;
+  settings.beatSensitivity = .5;
+  settings.aspectCorrection = 1;
+  settings.softCutRatingsEnabled = 0;
+  settings.shuffleEnabled = 1;
+  settings.presetURL = "/usr/share/projectM/presets";
+  settings.menuFontURL = "/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf";
+  settings.titleFontURL = "/usr/share/fonts/truetype/ttf-dejavu/DejaVuSansMono.ttf";
+  settings.easterEgg = 1;
+  settings.textureSize = DEF_TEXTURESIZE;
 
-  settings = sd->globalPM->settings();
+  // can fail here
+  sd->globalPM = new projectM(settings, 0);
 
   sd->textureHandle = sd->globalPM->initRenderToTexture();
 
   sd->nprs = sd->globalPM->getPlaylistSize() + 1;
 
-  sd->prnames = (volatile char **)weed_malloc(sd->nprs * sizeof(char *));
-
-  sd->prnames[0] = (volatile char *)"- Random -";
+  sd->prnames = (char **volatile)weed_malloc(sd->nprs * sizeof(char *));
+  sd->prnames[0] = strdup("- Random -");
 
   for (i = 1; i < sd->nprs; i++) {
-    sd->prnames[i] = const_cast<volatile char *>((sd->globalPM->getPresetName(i - 1)).c_str());
+    sd->prnames[i] = strdup((sd->globalPM->getPresetName(i - 1)).c_str());
   };
 
   // tell main thread we are ready
@@ -341,17 +376,13 @@ static void *worker(void *data) {
       sd->update_size = false;
     }
     pthread_mutex_unlock(&sd->mutex);
-    settings.fps = sd->fps;
     render_frame(sd);
   }
 
-  if (sd->globalPM != NULL) delete(sd->globalPM);
-
-
-fail:
+  // TODO : segfault
+  //if (sd->globalPM != NULL) delete(sd->globalPM);
 
   SDL_Quit();
-
   return NULL;
 }
 
@@ -370,7 +401,6 @@ static int projectM_deinit(weed_plant_t *inst) {
 }
 
 
-
 static int projectM_init(weed_plant_t *inst) {
   _sdata *sd;
 
@@ -380,8 +410,7 @@ static int projectM_init(weed_plant_t *inst) {
   copies++;
 
   if (!inited) {
-    int rc;
-    struct timeval tv;
+    int rc = 0;
 
     weed_plant_t *out_channel = weed_get_plantptr_value(inst, "out_channels", &error);
     weed_plant_t *iparam = weed_get_plantptr_value(inst, "in_parameters", &error);
@@ -426,28 +455,28 @@ static int projectM_init(weed_plant_t *inst) {
     sd->nprs = 0;
     sd->prnames = NULL;
     sd->worker_ready = false;
-
-    pthread_mutex_init(&cond_mutex, NULL);
-    pthread_cond_init(&cond, NULL);
+    sd->rendering = false;
 
     // kick off a thread to init screean and render
     pthread_create(&sd->thread, NULL, worker, sd);
 
-    gettimeofday(&tv, NULL);
-    ts.tv_sec = tv.tv_sec + 30;
+    clock_gettime(CLOCK_REALTIME, &ts);
+    ts.tv_sec += 30;
 
-    pthread_mutex_lock(&cond_mutex);
-    rc = pthread_cond_timedwait(&cond, &cond_mutex, &ts);
-    pthread_mutex_unlock(&cond_mutex);
+    // wait for worker thread ready
+    while (!sd->worker_ready && rc == 0) {
+      pthread_mutex_lock(&cond_mutex);
+      rc = pthread_cond_timedwait(&cond, &cond_mutex, &ts);
+      pthread_mutex_unlock(&cond_mutex);
+    }
 
-    if (rc == ETIMEDOUT || !sd->worker_ready) {
+    if (rc == ETIMEDOUT && !sd->worker_ready) {
       // if we timedout then die
       projectM_deinit(inst);
       return WEED_ERROR_INIT_ERROR;
     }
 
     inited = 1;
-
     weed_set_string_array(iparamgui, "choices", sd->nprs, (char **)sd->prnames);
   } else sd = statsd;
 
@@ -463,7 +492,6 @@ static int projectM_init(weed_plant_t *inst) {
 }
 
 
-
 static int projectM_process(weed_plant_t *inst, weed_timecode_t timestamp) {
   int error;
 
@@ -574,7 +602,6 @@ static int projectM_process(weed_plant_t *inst, weed_timecode_t timestamp) {
 }
 
 
-
 weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
   weed_plant_t *plugin_info = weed_plugin_info_init(weed_boot, num_versions, api_versions);
   if (plugin_info != NULL) {
@@ -613,22 +640,22 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 }
 
 
-
-
 void weed_desetup(void) {
-  std::cout << "ProjectM EXITING3" << std::endl;
   if (inited && statsd != NULL) {
     statsd->die = true;
     pthread_join(statsd->thread, NULL);
     if (statsd->fbuffer != NULL) weed_free(statsd->fbuffer);
     if (statsd->audio != NULL) weed_free(statsd->audio);
-    if (statsd->prnames != NULL) weed_free(statsd->prnames);
+    if (statsd->prnames != NULL) {
+      for (int i = 0; i < statsd->nprs; i++) {
+        free(statsd->prnames[i]);
+      }
+      weed_free(statsd->prnames);
+    }
     pthread_mutex_destroy(&statsd->mutex);
     pthread_mutex_destroy(&statsd->pcm_mutex);
-    pthread_mutex_destroy(&cond_mutex);
-    pthread_cond_destroy(&cond);
     weed_free(statsd);
     statsd = NULL;
   }
-  std::cout << "ProjectM EXITING4" << std::endl;
+  inited = 0;
 }
