@@ -82,7 +82,7 @@ typedef struct {
   off_t seek;
   weed_timecode_t start_tc;
 
-  double arate;
+  int arate;
 
   ssize_t bytesize; // file in/out length in bytes [write by server in case of eof]
 
@@ -111,13 +111,14 @@ typedef struct {
   int32_t **buffer32; ///< sample data in 32 bit format (or NULL)
   float   **bufferf; ///< sample data in float format (or NULL)
 
+  // input values
   boolean s8_signed;
   boolean s16_signed;
   boolean s24_signed;
   boolean s32_signed;
 
   // ring buffer
-  size_t samples_filled; ///< number of samples filled (readonly client)
+  volatile size_t samples_filled; ///< number of samples filled (readonly client)
   size_t start_sample; ///< used for reading (readonly server)
 
   // private fields (used by server)
@@ -178,6 +179,7 @@ int64_t render_audio_segment(int nfiles, int *from_files, int to_file, double *a
 void aud_fade(int fileno, double startt, double endt, double startv, double endv); ///< fade in/fade out
 
 typedef enum {
+  RECA_MONITOR = 0,
   RECA_WINDOW_GRAB,
   RECA_NEW_CLIP,
   RECA_EXISTING,
@@ -213,6 +215,8 @@ void free_pulse_audio_buffers(void);
 
 void audio_free_fnames(void);
 
+boolean is_realtime_aplayer(int ptype);
+
 lives_audio_buf_t *audio_cache_init(void);
 void audio_cache_end(void);
 lives_audio_buf_t *audio_cache_get_buffer(void);
@@ -221,10 +225,10 @@ boolean apply_rte_audio_init(void);
 void apply_rte_audio_end(boolean del);
 boolean apply_rte_audio(int nframes);
 
-void init_audio_frame_buffer(short aplayer);
+void init_audio_frame_buffers(short aplayer);
 void free_audio_frame_buffer(lives_audio_buf_t *abuf);
-void append_to_audio_bufferf(lives_audio_buf_t *abuf, float *src, uint64_t nsamples, int channum);
-void append_to_audio_buffer16(lives_audio_buf_t *abuf, void *src, uint64_t nsamples, int channum);
+void append_to_audio_bufferf(float *src, uint64_t nsamples, int channum);
+void append_to_audio_buffer16(void *src, uint64_t nsamples, int channum);
 boolean push_audio_to_channel(weed_plant_t *achan, lives_audio_buf_t *abuf);
 
 boolean start_audio_stream(void);
@@ -234,4 +238,8 @@ void audio_stream(void *buff, size_t nbytes, int fd);
 
 char *lives_get_audio_file_name(int fnum);
 
+char *get_achannel_name(int totchans, int idx) WARN_UNUSED;
+const char *audio_player_get_display_name(const char *aplayer);
+
+lives_cancel_t handle_audio_timeout(void);
 #endif
