@@ -7,6 +7,8 @@
 #ifndef HAS_LIVES_CALLBACKS_H
 #define HAS_LIVES_CALLBACKS_H
 
+#include "interface.h"
+
 void lives_exit(int signum);
 
 void lives_notify(int msgnumber, const char *msgstring);
@@ -63,9 +65,9 @@ void on_import_theme_activate(LiVESMenuItem *, livespointer);
 
 void on_quit_activate(LiVESMenuItem *, livespointer);
 
-void on_undo_activate(LiVESMenuItem *, livespointer);
+void on_undo_activate(LiVESWidget *, livespointer);
 
-void on_redo_activate(LiVESMenuItem *, livespointer);
+void on_redo_activate(LiVESWidget *, livespointer);
 
 void on_paste_as_new_activate(LiVESMenuItem *, livespointer);
 
@@ -129,9 +131,7 @@ void open_sel_range_activate(void);
 
 void on_location_select(LiVESButton *, livespointer);
 
-void on_utube_select(LiVESButton *, livespointer);
-
-void on_autoreload_toggled(LiVESToggleButton *, livespointer);
+void on_utube_select(lives_remote_clip_request_t *req);
 
 void on_opensel_range_ok_clicked(LiVESButton *, livespointer);
 
@@ -167,7 +167,7 @@ void on_resize_activate(LiVESMenuItem *, livespointer);
 
 void on_rename_activate(LiVESMenuItem *, livespointer);
 
-void on_rename_set_name(LiVESButton *, livespointer);
+void on_rename_clip_name(LiVESButton *, livespointer);
 
 void on_spinbutton_start_value_changed(LiVESSpinButton *, livespointer);
 
@@ -199,11 +199,11 @@ void on_spin_step_value_changed(LiVESSpinButton *, livespointer);
 
 void on_spin_end_value_changed(LiVESSpinButton *, livespointer);
 
-EXPOSE_FN_PROTOTYPE(expose_vid_event)
+EXPOSE_FN_PROTOTYPE(expose_vid_event);
 
-EXPOSE_FN_PROTOTYPE(expose_laud_event)
+EXPOSE_FN_PROTOTYPE(expose_laud_event);
 
-EXPOSE_FN_PROTOTYPE(expose_raud_event)
+EXPOSE_FN_PROTOTYPE(expose_raud_event);
 
 void on_preview_clicked(LiVESButton *, livespointer);
 
@@ -323,19 +323,19 @@ void on_showfct_activate(LiVESMenuItem *, livespointer);
 
 void on_boolean_toggled(LiVESObject *, livespointer);
 
-void after_audio_toggled(LiVESWidget *, livespointer);
+void on_audio_toggled(LiVESWidget *, livespointer);
 
 void on_showsubs_toggled(LiVESObject *, livespointer);
 
 void on_show_messages_activate(LiVESMenuItem *, livespointer);
 
-boolean on_hrule_enter(LiVESWidget *, LiVESXEventCrossing *, livespointer);
-
+#ifdef ENABLE_GIW_3
+void on_hrule_value_changed(LiVESWidget *widget, livespointer user_data);
+#else
 boolean on_hrule_update(LiVESWidget *, LiVESXEventMotion *, livespointer);
-
-boolean on_hrule_reset(LiVESWidget *, LiVESXEventButton *, livespointer);
-
 boolean on_hrule_set(LiVESWidget *, LiVESXEventButton *, livespointer);
+boolean on_hrule_reset(LiVESWidget *, LiVESXEventButton *, livespointer);
+#endif
 
 void on_rewind_activate(LiVESMenuItem *, livespointer);
 
@@ -383,6 +383,8 @@ void on_show_keys_activate(LiVESMenuItem *, livespointer);
 
 void on_vj_reset_activate(LiVESMenuItem *, livespointer);
 
+void on_vj_realize_activate(LiVESMenuItem *, livespointer);
+
 void on_prv_link_toggled(LiVESToggleButton *, livespointer);
 
 boolean on_del_audio_activate(LiVESMenuItem *, livespointer);
