@@ -448,10 +448,9 @@ boolean realize_all_frames(int clipno, const char *msg, boolean enough) {
     mainw->current_file = clipno;
     cfile->progress_start = 1;
     cfile->progress_end = count_virtual_frames(cfile->frame_index, 1, cfile->frames);
-    cfile->opening_only_audio = enough; // force "Enough" button to be shown
+    if (enough) mainw->cancel_type = CANCEL_SOFT; // force "Enough" button to be shown
     do_threaded_dialog((char *)msg, TRUE);
-    cfile->opening_only_audio = FALSE;
-    if (enough) lives_widget_hide(cfile->proc_ptr->cancel_button); // leave only "Enough" button showing
+    mainw->cancel_type = CANCEL_KILL;
     retb = virtual_to_images(mainw->current_file, 1, cfile->frames, TRUE, NULL);
     end_threaded_dialog();
     mainw->current_file = current_file;
