@@ -114,6 +114,10 @@ typedef enum {
 #define WEED_LEAF_PIXEL_ASPECT_RATIO "pixel_aspect_ratio"
 #define WEED_LEAF_ROWSTRIDE_ALIGNMENT_HINT "rowstride_alignment_hint"
 #define WEED_LEAF_MAX_REPEATS "max_repeats"
+#ifdef TEST_GAMMA
+#define WEED_LEAF_GAMMA_TYPE "gamma_type"
+// values will be WEED_GAMMA_TYPE_SRGB, WEED_GAMMA_TYPE_LINEAR, WEED_GAMMA_TYPE_MONITOR
+#endif
 
 // params / param tmpl
 #define WEED_LEAF_IN_PARAMETERS "in_parameters"
@@ -180,16 +184,23 @@ typedef enum {
 #define WEED_LEAF_HOST_REFS "host_refs" // host ref counting
 #define WEED_LEAF_HOST_REPEATS "host_repeats" // host channel repeats
 #define WEED_LEAF_HOST_INITED "host_inited" // inited or not
+#define WEED_LEAF_HOST_PLUGIN_PATH "host_plugin_path" // plugin path
+#define WEED_LEAF_HOST_HANDLE "host_handle" // dll handle
+#define WEED_LEAF_HOST_FILTER_LIST "host_filter_list" // host usable filters
+#define WEED_LEAF_HOST_NORECORD "host_norecord" // do not record parameter changes for this instance
+
+#define WEED_LEAF_HOST_INSTANCE "host_instance" // special value for text widgets
+#define WEED_LEAF_HOST_IDX "host_idx" // special value for text widgets
+
+#define WEED_LEAF_HOST_SCRAP_FILE_OFFSET "scrap_file_offset" // special value for scrap_file
+
+// compound plugins
 #define WEED_LEAF_HOST_INTERNAL_CONNECTION "host_internal_connection" // for chain plugins
 #define WEED_LEAF_HOST_INTERNAL_CONNECTION_AUTOSCALE "host_internal_connection_autoscale" // for chain plugins
 #define WEED_LEAF_HOST_NEXT_INSTANCE "host_next_instance" // for chain plugins
 #define WEED_LEAF_HOST_COMPOUND_CLASS "host_compound_class" // for chain plugins
-#define WEED_LEAF_HOST_PLUGIN_PATH "host_plugin_path" // plugin path
-#define WEED_LEAF_HOST_HANDLE "host_handle" // dll handle
-#define WEED_LEAF_HOST_FILTER_LIST "host_filter_list" // host usable filters
+#define WEED_LEAF_HOST_CHANNEL_CONNECTION "host_channel_connection" // special value for text widgets
 
-#define WEED_LEAF_HOST_INSTANCE "host_instance" // special value for text widgets
-#define WEED_LEAF_HOST_IDX "host_idx" // special value for text widgets
 
 // layer only values
 
@@ -254,8 +265,9 @@ char *cd_to_plugin_dir(weed_plant_t *filter);
 boolean weed_init_effect(int hotkey); ///< hotkey starts at 1
 void weed_deinit_effect(int hotkey); ///< hotkey starts at 1
 weed_plant_t *weed_instance_from_filter(weed_plant_t *filter);
-void weed_instance_ref(weed_plant_t *inst);
-void weed_instance_unref(weed_plant_t *inst);
+int _wood_instance_ref(weed_plant_t *inst);
+int _wood_instance_unref(weed_plant_t *inst);
+weed_plant_t *_wood_instance_obtain(int line, char *file, int key, int mode);
 void weed_in_parameters_free(weed_plant_t *inst);
 void weed_in_params_free(weed_plant_t **parameters, int num_parameters);
 void add_param_connections(weed_plant_t *inst);
@@ -286,11 +298,12 @@ boolean weed_parameter_has_variable_elements_strict(weed_plant_t *inst, weed_pla
 boolean interpolate_param(weed_plant_t *inst, int i, void *pchain, weed_timecode_t tc);
 boolean interpolate_params(weed_plant_t *inst, void **pchains, weed_timecode_t tc);
 
-void filter_mutex_lock(int key);  // 0 based key
-void filter_mutex_unlock(int key); // 0 based key
+int filter_mutex_lock(int key);  // 0 based key
+int filter_mutex_trylock(int key);  // 0 based key
+int filter_mutex_unlock(int key); // 0 based key
 
-boolean weed_plant_serialise(int fd, weed_plant_t *plant, unsigned char **mem);
-weed_plant_t *weed_plant_deserialise(int fd, unsigned char **mem);
+size_t weed_plant_serialise(int fd, weed_plant_t *plant, unsigned char **mem);
+weed_plant_t *weed_plant_deserialise(int fd, unsigned char **mem, weed_plant_t *plant);
 
 /// record a parameter value change in our event_list
 void rec_param_change(weed_plant_t *inst, int pnum);
@@ -358,7 +371,7 @@ char *rte_keymode_get_type(int key, int mode) WARN_UNUSED;  ///< returns a strin
 lives_fx_cat_t rte_keymode_get_category(int key, int mode);
 #endif
 
-weed_plant_t *rte_keymode_get_instance(int key, int mode); ///< returns filter_instance bound to key/mode (or NULL)
+weed_plant_t *rte_keymode_get_instance(int key, int mode); ///< returns refcounted filter_instance bound to key/mode (or NULL)
 weed_plant_t *rte_keymode_get_filter(int key, int mode); ///< returns filter_class bound to key/mode (or NULL)
 
 boolean weed_delete_effectkey(int key, int mode);  ///< unbinds a filter_class from a key/mode
@@ -369,19 +382,15 @@ int weed_add_effectkey_by_idx(int key, int idx);  ///< see description
 int rte_key_getmode(int key);  ///< returns current active mode for a key (or -1)
 int rte_key_getmaxmode(int key); ///< returns highest mode which is set
 
-weed_plant_t *get_new_inst_for_keymode(int key, int mode); ///< get new inst (during recording playback)
+weed_plant_t *get_new_inst_for_keymode(int key, int mode); ///< get new refcounted inst (during recording playback)
 
 boolean rte_key_setmode(int key, int newmode);  ///< set mode for a given key; if key==0 then the active key is used
 
 ///< returns -1 if the filter is not found; it will match the first name found - returns -2 if you try to switch a generator/non-generator
 int rte_switch_keymode(int key, int mode, const char *hashname);
 
-//boolean rte_key_is_enabled(int key);
-
 /////////////////////////////////////////////////////////////
 
-int rte_getmodespk(void) __attribute__((__pure__));
-
 LiVESList *weed_get_all_names(lives_fx_list_t list_type);
 int rte_get_numfilters(boolean inc_dupes);
 
@@ -445,4 +454,27 @@ void fill_param_vals_to(weed_plant_t *param, weed_plant_t *ptmpl, int fill_slot)
 
 int weed_general_error;
 
+//#define DEBUG_FILTER_MUTEXES
+#ifdef DEBUG_FILTER_MUTEXES
+#define filter_mutex_lock(key) {g_print ("lock %d at line %d in file %s\n",key,__LINE__,__FILE__); if (key >= 0 && key < FX_KEYS_MAX) pthread_mutex_lock(&mainw->fx_mutex[key]); g_print("done\n");}
+#define filter_mutex_unlock(key) {g_print ("unlock %d at line %d in file %s\n\n",key,__LINE__,__FILE__); if (key >= 0 && key < FX_KEYS_MAX) pthread_mutex_unlock(&mainw->fx_mutex[key]); g_print("done\n");}
+#endif
+
+//#define DEBUG_REFCOUNT
+#ifdef DEBUG_REFCOUNT
+#define weed_instance_ref(a) {g_print ("ref %p at line %d in file %s\n",a,__LINE__,__FILE__); _weed_instance_ref(a);}
+#define weed_instance_unref(a) {g_print ("unref %p at line %d in file %s\n",a,__LINE__,__FILE__); _weed_instance_unref(a);}
+#define weed_instance_obtain(a,b) _weed_instance_obtain(__LINE__, __FILE__, a, b)
+#endif
+
+int _weed_instance_ref(weed_plant_t *inst);
+int _weed_instance_unref(weed_plant_t *inst);
+weed_plant_t *_weed_instance_obtain(int line, char *file, int key, int mode);
+
+#ifndef DEBUG_REFCOUNT
+int weed_instance_ref(weed_plant_t *inst);
+int weed_instance_unref(weed_plant_t *inst);
+weed_plant_t *weed_instance_obtain(int key, int mode);
+#endif
+
 #endif
