@@ -1,9 +1,11 @@
 // effects.c
 // LiVES (lives-exe)
-// (c) G. Finch 2003 - 2016
+// (c) G. Finch <salsaman+lives@gmail.com> 2003 - 2018
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
+#include "main.h"
+
 #include <fcntl.h>
 #include <string.h>
 #include <unistd.h>
@@ -21,8 +23,8 @@
 #include "../libweed/weed-host.h"
 #endif
 
-#include "main.h"
 #include "effects.h"
+#include "effects-weed.h"
 #include "interface.h"
 #include "paramwindow.h"
 #include "support.h"
@@ -39,8 +41,6 @@
 
 #include "rte_window.h"
 
-static int framecount;
-
 static weed_plant_t *resize_instance = NULL;
 
 static boolean apply_audio_fx;
@@ -138,6 +138,8 @@ char *lives_fx_cat_to_text(lives_fx_cat_t cat, boolean plural) {
 // Rendered effects
 
 boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
+  // apply a rendered effect to the current file
+
   // returns FALSE if the user cancelled
   // leave_info_file is set if a preview turned into actual processing: ie. no params were changed after the preview
   // preview generates .pre files instead of .mgk, so needs special post-processing
@@ -157,6 +159,8 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
   boolean got_no_frames = FALSE;
   char *tmp;
 
+  if (!CURRENT_CLIP_IS_VALID) return FALSE;
+
   if (rfx->num_in_channels == 0 && !is_preview) current_file = mainw->pre_src_file;
 
   if (is_preview) {
@@ -370,14 +374,14 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
 
   mainw->show_procd = TRUE;
 
-  if (rfx->props & RFX_PROPS_MAY_RESIZE || rfx->num_in_channels == 0) {
-    // get new frame size
-    if (rfx->status != RFX_STATUS_WEED) {
-      int numtok = get_token_count(mainw->msg, '|');
-
-      if (numtok > 1) {
-        char **array = lives_strsplit(mainw->msg, "|", numtok);
-        // [0] is "completed"
+  if (rfx->status != RFX_STATUS_WEED) {
+    int numtok = get_token_count(mainw->msg, '|');
+    if (numtok > 1) {
+      char **array = lives_strsplit(mainw->msg, "|", numtok);
+      // [0] is "completed"
+      cfile->end = cfile->progress_end = cfile->start + atoi(array[4]) - 1;
+      if (rfx->props & RFX_PROPS_MAY_RESIZE || rfx->num_in_channels == 0) {
+        // get new frame size
         cfile->hsize = atoi(array[1]);
         cfile->vsize = atoi(array[2]);
         if (rfx->num_in_channels == 0) {
@@ -386,31 +390,31 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
           cfile->end = cfile->frames = atoi(array[4]);
           cfile->bpp = cfile->img_type == IMG_TYPE_JPEG ? 24 : 32;
         }
-        lives_strfreev(array);
       }
-      if (rfx->num_in_channels == 0) {
-        cfile->progress_start = 1;
-        cfile->progress_end = cfile->frames;
-      }
-    } else {
-      int error;
-      weed_plant_t *first_out = get_enabled_channel((weed_plant_t *)rfx->source, 0, FALSE);
-      weed_plant_t *first_ot = weed_get_plantptr_value(first_out, WEED_LEAF_TEMPLATE, &error);
-      cfile->hsize = weed_get_int_value(first_ot, WEED_LEAF_HOST_WIDTH, &error);
-      cfile->vsize = weed_get_int_value(first_ot, WEED_LEAF_HOST_HEIGHT, &error);
+      lives_strfreev(array);
     }
+    if (rfx->num_in_channels == 0) {
+      cfile->progress_start = 1;
+      cfile->progress_end = cfile->frames;
+    }
+  } else {
+    int error;
+    weed_plant_t *first_out = get_enabled_channel((weed_plant_t *)rfx->source, 0, FALSE);
+    weed_plant_t *first_ot = weed_get_plantptr_value(first_out, WEED_LEAF_TEMPLATE, &error);
+    cfile->hsize = weed_get_int_value(first_ot, WEED_LEAF_HOST_WIDTH, &error);
+    cfile->vsize = weed_get_int_value(first_ot, WEED_LEAF_HOST_HEIGHT, &error);
+  }
 
-    if (rfx->num_in_channels > 0) {
-      if (cfile->hsize == cfile->ohsize && cfile->vsize == cfile->ovsize) cfile->undo_action = UNDO_EFFECT;
-      else {
-        boolean bad_header = FALSE;
-        save_clip_value(mainw->current_file, CLIP_DETAILS_WIDTH, &cfile->hsize);
-        if (mainw->com_failed || mainw->write_failed) bad_header = TRUE;
-        save_clip_value(mainw->current_file, CLIP_DETAILS_HEIGHT, &cfile->vsize);
-        if (mainw->com_failed || mainw->write_failed) bad_header = TRUE;
-        cfile->undo_action = UNDO_RESIZABLE;
-        if (bad_header) do_header_write_error(mainw->current_file);
-      }
+  if (rfx->num_in_channels > 0) {
+    if (cfile->hsize == cfile->ohsize && cfile->vsize == cfile->ovsize) cfile->undo_action = UNDO_EFFECT;
+    else {
+      boolean bad_header = FALSE;
+      save_clip_value(mainw->current_file, CLIP_DETAILS_WIDTH, &cfile->hsize);
+      if (mainw->com_failed || mainw->write_failed) bad_header = TRUE;
+      save_clip_value(mainw->current_file, CLIP_DETAILS_HEIGHT, &cfile->vsize);
+      if (mainw->com_failed || mainw->write_failed) bad_header = TRUE;
+      cfile->undo_action = UNDO_RESIZABLE;
+      if (bad_header) do_header_write_error(mainw->current_file);
     }
   }
 
@@ -419,6 +423,7 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
 
   if (mainw->keep_pre) {
     // this comes from a preview which then turned into processing
+    // the processing is the same as usual, except we use a different file extension (.pre instead of .mgk)
     char *com = lives_strdup_printf("%s mv_pre \"%s\" %d %d \"%s\"", prefs->backend_sync, cfile->handle, cfile->progress_start,
                                     cfile->progress_end, get_image_ext_for_type(cfile->img_type));
 
@@ -461,7 +466,7 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
   if (rfx->num_in_channels == 0) {
     if (rfx->props & RFX_PROPS_BATCHG) {
       // batch mode generators need some extra processing
-      char *imgdir = lives_strdup_printf("%s%s", prefs->workdir, cfile->handle);
+      char *imgdir = lives_build_path(prefs->workdir, cfile->handle, NULL);
       int img_file = mainw->current_file;
 
       mainw->suppress_dprint = TRUE;
@@ -474,7 +479,8 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
 
         lives_snprintf(mainw->files[new_file]->name, CLIP_NAME_MAXLEN, "%s", cfile->name);
         lives_snprintf(mainw->files[new_file]->file_name, PATH_MAX, "%s", cfile->file_name);
-        set_menu_text(mainw->files[new_file]->menuentry, cfile->name, FALSE);
+
+        lives_menu_item_set_text(mainw->files[new_file]->menuentry, cfile->name, FALSE);
 
         mainw->files[new_file]->fps = mainw->files[new_file]->pb_fps = cfile->fps;
       } else got_no_frames = TRUE;
@@ -519,17 +525,16 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
       lives_memcpy(clipboard, cfile, sizeof(lives_clip_t));
       cfile->is_loaded = TRUE;
       mainw->suppress_dprint = TRUE;
-      mainw->only_close = TRUE;
+      mainw->close_keep_frames = TRUE;
 
       close_current_file(current_file);
 
       mainw->suppress_dprint = FALSE;
-      mainw->only_close = FALSE;
+      mainw->close_keep_frames = FALSE;
 
       new_file = current_file;
 
       mainw->untitled_number--;
-
     } else {
       if (!(rfx->props & RFX_PROPS_BATCHG)) {
         // gen to new file
@@ -556,8 +561,8 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
   if (!mainw->gen_to_clipboard) cfile->changed = TRUE;
   if (mainw->multitrack == NULL) {
     if (new_file != -1) {
-      lives_sync();
-      switch_to_file((mainw->current_file = 0), new_file);
+      lives_sync(1);
+      switch_clip(1, new_file, TRUE);
     }
   } else {
     mainw->current_file = mainw->multitrack->render_file;
@@ -613,17 +618,18 @@ lives_render_error_t realfx_progress(boolean reset) {
   if (mainw->effects_paused) return LIVES_RENDER_EFFECTS_PAUSED;
 
   // sig_progress...
-  lives_snprintf(mainw->msg, 256, "%d", i);
+  lives_snprintf(mainw->msg, MAINW_MSG_SIZE, "%d", i);
   // load, effect, save frame
 
   // skip resizing virtual frames
   if (resize_instance != NULL && is_virtual_frame(mainw->current_file, i)) {
+    mainw->rowstride_alignment_hint = 1;
     if (++i > cfile->end) {
       mainw->internal_messaging = FALSE;
       lives_snprintf(mainw->msg, 9, "completed");
+      return LIVES_RENDER_COMPLETE;
     }
-    mainw->rowstride_alignment_hint = 1;
-    return LIVES_RENDER_COMPLETE;
+    return LIVES_RENDER_PROCESSING;
   }
 
   if (has_video_filters(FALSE) || resize_instance != NULL) {
@@ -634,7 +640,7 @@ lives_render_error_t realfx_progress(boolean reset) {
     layer = weed_layer_new_for_frame(mainw->current_file, i);
     if (!pull_frame(layer, get_image_ext_for_type(cfile->img_type), frameticks)) {
       // do_read_failed_error_s() cannot be used here as we dont know the filename
-      lives_snprintf(mainw->msg, 256, "error|missing image %d", i);
+      lives_snprintf(mainw->msg, MAINW_MSG_SIZE, "error|missing image %d", i);
       return LIVES_RENDER_WARNING_READ_FRAME;
     }
 
@@ -749,7 +755,6 @@ boolean on_realfx_activate_inner(int type, lives_rfx_t *rfx) {
   else resize_instance = NULL;
 
   mainw->internal_messaging = TRUE;
-  framecount = 0;
 
   mainw->rowstride_alignment_hint = 1;
 
@@ -785,18 +790,19 @@ boolean on_realfx_activate_inner(int type, lives_rfx_t *rfx) {
     reget_afilesize(mainw->current_file);
   }
 
+  mainw->internal_messaging = FALSE;
   resize_instance = NULL;
   return retval;
 }
 
 
-void on_realfx_activate(LiVESMenuItem *menuitem, livespointer user_data) {
+void on_realfx_activate(LiVESMenuItem *menuitem, livespointer rfx) {
   int type = 1;
 
   boolean has_lmap_error = FALSE;
 
   // type can be 0 - apply current realtime effects
-  // 1 - resize (using weed filter)
+  // 1 - resize (using weed filter) [menuitem == NULL]
 
   if (menuitem != NULL) {
     type = 0;
@@ -826,23 +832,27 @@ void on_realfx_activate(LiVESMenuItem *menuitem, livespointer user_data) {
     }
   }
 
-  if (!on_realfx_activate_inner(type, (lives_rfx_t *)user_data)) return;
+  if (!on_realfx_activate_inner(type, (lives_rfx_t *)rfx)) return;
 
   if (has_lmap_error) popup_lmap_errors(NULL, NULL);
 }
 
 
 weed_plant_t *on_rte_apply(weed_plant_t *layer, int opwidth, int opheight, weed_timecode_t tc) {
-  // realtime effects
+  // apply realtime effects to a layer
+  // mainw->filter_map is used as a guide
+  // mainw->pchains holds the parameter values for interpolation
+  // creates a temporary mix layer from mainw->blend_file (correcting its value if necessary)
+
+  // returns the effected layer
+
   weed_plant_t **layers, *retlayer;
   int i;
 
   if (mainw->foreign) return NULL;
 
   layers = (weed_plant_t **)lives_malloc(3 * sizeof(weed_plant_t *));
-
   layers[2] = NULL;
-
   layers[0] = layer;
 
   if (mainw->blend_file > -1 && mainw->num_tr_applied > 0 && (mainw->files[mainw->blend_file] == NULL ||
@@ -859,11 +869,12 @@ weed_plant_t *on_rte_apply(weed_plant_t *layer, int opwidth, int opheight, weed_
   } else layers[1] = NULL;
 
   if (resize_instance != NULL) {
+    lives_filter_error_t ret;
     weed_plant_t *init_event = weed_plant_new(WEED_PLANT_EVENT);
     weed_set_int_value(init_event, WEED_LEAF_IN_TRACKS, 0);
     weed_set_int_value(init_event, WEED_LEAF_OUT_TRACKS, 0);
 
-    weed_apply_instance(resize_instance, init_event, layers, 0, 0, tc);
+    (void)(ret = weed_apply_instance(resize_instance, init_event, layers, 0, 0, tc));
 
     retlayer = layers[0];
     weed_plant_free(init_event);
@@ -884,7 +895,7 @@ weed_plant_t *on_rte_apply(weed_plant_t *layer, int opwidth, int opheight, weed_
 void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc) {
   weed_plant_t **layers;
 
-  weed_plant_t *deint_filter, *deint_instance, *next_inst, *init_event;
+  weed_plant_t *deint_filter, *deint_instance, *next_inst, *init_event, *orig_instance;
 
   int deint_idx, error;
 
@@ -895,7 +906,7 @@ void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc) {
 
   deint_filter = get_weed_filter(deint_idx);
 
-  deint_instance = weed_instance_from_filter(deint_filter);
+  orig_instance = deint_instance = weed_instance_from_filter(deint_filter);
 
   layers = (weed_plant_t **)lives_malloc(2 * sizeof(weed_plant_t *));
 
@@ -923,6 +934,8 @@ deint1:
     goto deint1;
   }
 
+  weed_instance_unref(orig_instance);
+
   weed_plant_free(init_event);
 
   lives_free(layers);
@@ -963,6 +976,10 @@ weed_plant_t *get_blend_layer(weed_timecode_t tc) {
 
 boolean rte_on_off_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t keyval, LiVESXModifierType mod, livespointer user_data) {
   // this is the callback which happens when a rte is keyed
+  // key is 1 based, but if < 0 then this indicates auto mode (set via data connection)
+  // in automode we don't add the effect parameters in ce_thumbs mode
+  // if non-automode, the user overrides effect toggling
+
   int key = LIVES_POINTER_TO_INT(user_data);
   uint64_t new_rte;
 
@@ -977,9 +994,11 @@ boolean rte_on_off_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t k
     key = -key;
   }
 
-  new_rte = GU641 << (key - 1);
+  key--;
 
-  if (key == EFFECT_NONE) {
+  new_rte = GU641 << (key);
+
+  if (key == (EFFECT_NONE - 1)) {
     // switch up/down keys to default (fps change)
     weed_deinit_all(FALSE);
   } else {
@@ -990,53 +1009,62 @@ boolean rte_on_off_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t k
     if (!(mainw->rte & new_rte)) {
       // switch is ON
       // WARNING - if we start playing because a generator was started, we block here
-      if (!(weed_init_effect(key - 1))) {
+      filter_mutex_lock(key);
+      if (!(weed_init_effect(key))) {
         // ran out of instance slots, no effect assigned, or some other error
+        // or gen started playback and then stopped
         pthread_mutex_lock(&mainw->event_list_mutex);
         if (mainw->rte & new_rte) mainw->rte ^= new_rte;
         pthread_mutex_unlock(&mainw->event_list_mutex);
-        if (rte_window != NULL) rtew_set_keych(key - 1, FALSE);
-        if (mainw->ce_thumbs) ce_thumbs_set_keych(key - 1, FALSE);
+        if (rte_window != NULL) rtew_set_keych(key, FALSE);
+        if (mainw->ce_thumbs) ce_thumbs_set_keych(key, FALSE);
         mainw->osc_block = FALSE;
+        filter_mutex_unlock(key);
         return TRUE;
       }
 
-      if (!mainw->gen_started_play) {
+      if (!mainw->gen_started_play) { // should always be the case
+        pthread_mutex_lock(&mainw->event_list_mutex);
         if (!(mainw->rte & new_rte)) mainw->rte |= new_rte;
+        pthread_mutex_unlock(&mainw->event_list_mutex);
 
-        mainw->last_grabbable_effect = key - 1;
-        if (rte_window != NULL) rtew_set_keych(key - 1, TRUE);
+        mainw->last_grabbable_effect = key;
+        if (rte_window != NULL) rtew_set_keych(key, TRUE);
         if (mainw->ce_thumbs) {
-          ce_thumbs_set_keych(key - 1, TRUE);
+          ce_thumbs_set_keych(key, TRUE);
 
           // if effect was auto (from ACTIVATE data connection), leave all param boxes
           // otherwise, remove any which are not "pinned"
-          if (!mainw->fx_is_auto) ce_thumbs_add_param_box(key - 1, !mainw->fx_is_auto);
+          if (!mainw->fx_is_auto) ce_thumbs_add_param_box(key, !mainw->fx_is_auto);
         }
       }
+      filter_mutex_unlock(key);
     } else {
       // effect is OFF
-      weed_deinit_effect(key - 1);
+      filter_mutex_lock(key);
+      weed_deinit_effect(key);
       pthread_mutex_lock(&mainw->event_list_mutex);
       if (mainw->rte & new_rte) mainw->rte ^= new_rte;
       pthread_mutex_unlock(&mainw->event_list_mutex);
-      if (rte_window != NULL) rtew_set_keych(key - 1, FALSE);
-      if (mainw->ce_thumbs) ce_thumbs_set_keych(key - 1, FALSE);
+      filter_mutex_unlock(key);
+      if (rte_window != NULL) rtew_set_keych(key, FALSE);
+      if (mainw->ce_thumbs) ce_thumbs_set_keych(key, FALSE);
     }
   }
 
   mainw->osc_block = FALSE;
-  mainw->fx_is_auto = FALSE;
 
   if (mainw->current_file > 0 && cfile->play_paused && !mainw->noswitch) {
     load_frame_image(cfile->frameno);
   }
 
-  if (mainw->playing_file == -1 && mainw->current_file > 0 && ((has_video_filters(FALSE) && !has_video_filters(TRUE)) ||
-      (cfile->achans > 0 && prefs->audio_src == AUDIO_SRC_INT && has_audio_filters(AF_TYPE_ANY)) ||
-      mainw->agen_key != 0)) {
-    lives_widget_set_sensitive(mainw->rendered_fx[0].menuitem, TRUE);
-  } else lives_widget_set_sensitive(mainw->rendered_fx[0].menuitem, FALSE);
+  if (mainw->rendered_fx != NULL) {
+    if (mainw->playing_file == -1 && mainw->current_file > 0 && ((has_video_filters(FALSE) && !has_video_filters(TRUE)) ||
+        (cfile->achans > 0 && prefs->audio_src == AUDIO_SRC_INT && has_audio_filters(AF_TYPE_ANY)) ||
+        mainw->agen_key != 0)) {
+      lives_widget_set_sensitive(mainw->rendered_fx[0].menuitem, TRUE);
+    } else lives_widget_set_sensitive(mainw->rendered_fx[0].menuitem, FALSE);
+  }
 
   if (key > 0 && !mainw->fx_is_auto) {
     // user override any ACTIVATE data connection
@@ -1046,6 +1074,7 @@ boolean rte_on_off_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t k
     end_override_if_activate_output(key);
   }
 
+  mainw->fx_is_auto = FALSE;
   return TRUE;
 }
 
@@ -1067,7 +1096,9 @@ boolean grabkeys_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t key
   if (rte_window != NULL) {
     if (group != NULL) rtew_set_keygr(mainw->rte_keys);
   }
+  filter_mutex_lock(mainw->rte_keys);
   mainw->blend_factor = weed_get_blend_factor(mainw->rte_keys);
+  filter_mutex_unlock(mainw->rte_keys);
   mainw->osc_block = FALSE;
   return TRUE;
 }
@@ -1091,7 +1122,9 @@ boolean rtemode_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t keyv
   // "m" mode key
   if (mainw->rte_keys == -1) return TRUE;
   rte_key_setmode(0, -1);
+  filter_mutex_lock(mainw->rte_keys);
   mainw->blend_factor = weed_get_blend_factor(mainw->rte_keys);
+  filter_mutex_unlock(mainw->rte_keys);
   return TRUE;
 }
 
@@ -1133,3 +1166,63 @@ boolean swap_fg_bg_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t k
 }
 
 //////////////////////////////////////////////////////////////
+
+
+LIVES_GLOBAL_INLINE boolean rte_key_is_enabled(int key) {
+  // key starts at 1
+  return !((mainw->rte & (GU641 << --key)) == 0ll);
+}
+
+
+LIVES_GLOBAL_INLINE int rte_getmodespk(void) {
+  return prefs->max_modes_per_key;
+}
+
+
+LIVES_GLOBAL_INLINE boolean rte_key_toggle(int key) {
+  // key is 1 based, or < 0 for auto mode
+  rte_on_off_callback(NULL, NULL, 0, (LiVESXModifierType)0, LIVES_INT_TO_POINTER(key));
+  if (key < 0) key = -key;
+  return rte_key_is_enabled(key);
+}
+
+
+LIVES_GLOBAL_INLINE void rte_keys_reset(void) {
+  // switch off all realtime effects
+  rte_on_off_callback(NULL, NULL, 0, (LiVESXModifierType)0, LIVES_INT_TO_POINTER(EFFECT_NONE));
+}
+
+
+static int backup_key_modes[FX_KEYS_MAX_VIRTUAL];
+static uint64_t backup_rte = 0;
+
+void rte_keymodes_backup(int nkeys) {
+  // backup the current key/mode state
+  int i;
+
+  backup_rte = mainw->rte;
+
+  for (i = 0; i < nkeys; i++) {
+    backup_key_modes[i] = rte_key_getmode(i + 1);
+  }
+}
+
+
+void rte_keymodes_restore(int nkeys) {
+  int i;
+
+  rte_keys_reset();
+  mainw->rte = backup_rte;
+
+  for (i = 0; i < nkeys; i++) {
+    // set the mode
+    rte_key_setmode(i + 1, backup_key_modes[i]);
+    // activate the key
+    if ((mainw->rte & GU641 << (i)) != 0) rte_key_toggle(i + 1);
+  }
+  if (mainw->rte_keys != -1) {
+    filter_mutex_lock(mainw->rte_keys);
+    mainw->blend_factor = weed_get_blend_factor(mainw->rte_keys);
+    filter_mutex_unlock(mainw->rte_keys);
+  }
+}
