@@ -86,4 +86,15 @@ weed_plant_t *on_rte_apply(weed_plant_t *main_layer, int opwidth, int opheight,
 
 void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc);
 
+// rte keys state
+void rte_keymodes_backup(int nkeys);
+void rte_keymodes_restore(int nkeys);
+
+boolean rte_key_toggle(int key);
+void rte_keys_reset(void);
+
+boolean rte_key_is_enabled(int key);
+
+int rte_getmodespk(void);
+
 #endif
