@@ -66,6 +66,8 @@
 
 #define WEED_LEAF_HOST_AUDIO_TRANSITION "host_audio_transition"
 
+#define WEED_LEAF_HOST_TAG_COPY "host_tag_copy"
+
 #define LIVES_TRACK_ANY -1000000
 
 weed_plant_t *append_frame_event(weed_plant_t *event_list, weed_timecode_t tc, int numframes,
@@ -199,7 +201,6 @@ render_details *rdet;
 //// UI stuff ///////
 
 LiVESWidget *events_rec_dialog(boolean allow_mt);
-int get_render_choice(void);
 boolean deal_with_render_choice(boolean add_deinit);
 
 #define RENDER_CHOICE_DISCARD 0
