@@ -29,11 +29,25 @@ void jack_pb_stop(void);  /** pause playback transport master */
 
 #include "audio.h"
 
+#ifndef IS_DARWIN
+#define JACK_DRIVER_NAME "alsa"
+#else
+#ifdef IS_SOLARIS
+// use OSS on Solaris
+#define JACK_DRIVER_NAME "oss"
+#else
+// use coreaudio on Darwin
+#define JACK_DRIVER_NAME "coreaudio"
+#endif
+#endif
+
 #define JACK_MAX_OUTPUT_PORTS 10
 #define JACK_MAX_INPUT_PORTS 10
 
 #define ERR_PORT_NOT_FOUND 10
 
+#define JACK_DEFAULT_SERVER_NAME "(default)"
+
 typedef jack_nframes_t nframes_t;
 
 
@@ -44,8 +58,8 @@ typedef jack_nframes_t nframes_t;
 
 typedef struct {
   int      dev_idx;                      /**< id of this device ??? */
-  int64_t     sample_out_rate;                   /**< samples(frames) per second */
-  int64_t     sample_in_rate;                   /**< samples(frames) per second */
+  int     sample_out_rate;                   /**< samples(frames) per second */
+  volatile int     sample_in_rate;                   /**< samples(frames) per second */
   uint64_t    num_input_channels;            /**< number of input channels(1 is mono, 2 stereo etc..) */
   uint64_t    num_output_channels;           /**< number of output channels(1 is mono, 2 stereo etc..) */
   uint64_t    bytes_per_channel;
@@ -72,6 +86,7 @@ typedef struct {
   volatile aserver_message_t   *msgq;          /**< linked list of messages we are sending to the callback process */
 
   off_t seek_pos;
+  volatile off_t real_seek_pos;
   off_t seek_end;
   boolean usigned;
   boolean reverse_endian;
@@ -83,8 +98,8 @@ typedef struct {
   boolean             jackd_died;                    /**< true if jackd has died and we should try to restart it */
 
   boolean play_when_stopped; ///< if we should play audio even when jack transport is stopped
-  volatile uint64_t
-  audio_ticks; ///< ticks when we did the last seek, used to calculate current ticks from audio in tandem with frames_written
+
+  volatile jack_nframes_t nframes_start;
   volatile uint64_t frames_written;
 
   int out_chans_available;
@@ -120,11 +135,11 @@ jack_driver_t *jack_get_driver(int dev_idx, boolean is_output); ///< get driver
 int jack_audio_init(void); ///< init jack for host output
 int jack_audio_read_init(void); ///< init jack for host input
 
-int jack_open_device(jack_driver_t *); ///< open device for host output
-int jack_open_device_read(jack_driver_t *); ///< open device for host input
+boolean jack_create_client_writer(jack_driver_t *); ///< open device for host output
+boolean jack_create_client_reader(jack_driver_t *); ///< open device for host input
 
-int jack_driver_activate(jack_driver_t *);  ///< activate for host playback
-int jack_read_driver_activate(jack_driver_t *, boolean autocon);  ///< activate for host recording
+boolean jack_write_driver_activate(jack_driver_t *);  ///< activate for host playback
+boolean jack_read_driver_activate(jack_driver_t *, boolean autocon);  ///< activate for host recording
 
 void jack_close_device(jack_driver_t *);
 
@@ -132,9 +147,12 @@ boolean jack_try_reconnect(void);
 
 void jack_aud_pb_ready(int fileno);
 
+size_t jack_flush_read_data(size_t rbytes, void *data);
+
 // utils
 volatile aserver_message_t *jack_get_msgq(jack_driver_t *); ///< pull last msg from msgq, or return NULL
-uint64_t lives_jack_get_time(jack_driver_t *, boolean absolute); ///< get time from jack, in 10^-8 seconds
+void jack_time_reset(jack_driver_t *jackd, int64_t offset);
+uint64_t lives_jack_get_time(jack_driver_t *); ///< get time from jack, in 10^-8 seconds
 boolean jack_audio_seek_frame(jack_driver_t *, int frame);  ///< seek to (video) frame
 int64_t jack_audio_seek_bytes(jack_driver_t *, int64_t bytes);  ///< seek to byte position
 
