@@ -1,6 +1,6 @@
 // lbindings.c
 // LiVES (lives-exe)
-// (c) G. Finch <salsaman+lives@gmail.com> 2015 - 2017
+// (c) G. Finch <salsaman+lives@gmail.com> 2015 - 2019
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -131,7 +131,6 @@ typedef struct {
   double time;
 } mblockdata;
 
-
 /////////////////////////////////////////
 /// extern functions with no headers
 
@@ -262,7 +261,7 @@ boolean start_player(void) {
   *vargs = lives_strdup(",");
   arglen = padup(vargs, arglen);
 
-  // this will set our idlefunc and return
+  // this will set our idlefunc and return (TODO: don't set osc_auto)
   ret = lives_osc_cb_play(NULL, arglen, (const void *)(*vargs), OSCTT_CurrentTime(), NULL);
   if (ret) {
     while (!mainw->error && mainw->playing_file < 0) lives_usleep(prefs->sleep_time);
@@ -282,13 +281,13 @@ enum {
 };
 
 
-inline int trans_rev(int consta, int a, int b) {
+LIVES_LOCAL_INLINE int trans_rev(int consta, int a, int b) {
   if (consta == a) return b;
   else return consta;
 }
 
 
-int trans_constant(int consta, int domain) {
+static int trans_constant(int consta, int domain) {
   int nconsta;
   if (domain == const_domain_notify) {
     nconsta = trans_rev(consta, LIVES_CALLBACK_FRAME_SYNCH, LIVES_OSC_NOTIFY_FRAME_SYNCH);
@@ -580,7 +579,7 @@ static boolean call_set_ping_pong(livespointer data) {
 static boolean call_set_pref_bool(livespointer data) {
   bpref *bdata = (bpref *)data;
   if (mainw != NULL && !mainw->go_away) {
-    pref_factory_bool(bdata->prefidx, bdata->val);
+    pref_factory_bool(bdata->prefidx, bdata->val, TRUE);
     ext_caller_return_int(bdata->id, TRUE);
   } else ext_caller_return_int(bdata->id, FALSE);
   lives_free(bdata->prefidx);
@@ -592,7 +591,7 @@ static boolean call_set_pref_bool(livespointer data) {
 static boolean call_set_pref_int(livespointer data) {
   ipref *idata = (ipref *)data;
   if (mainw != NULL && !mainw->go_away) {
-    pref_factory_int(idata->prefidx, idata->val);
+    pref_factory_int(idata->prefidx, idata->val, TRUE);
     ext_caller_return_int(idata->id, TRUE);
   } else ext_caller_return_int(idata->id, FALSE);
   lives_free(data);
@@ -603,7 +602,7 @@ static boolean call_set_pref_int(livespointer data) {
 static boolean call_set_pref_bitmapped(livespointer data) {
   bmpref *bmdata = (bmpref *)data;
   if (mainw != NULL && !mainw->go_away) {
-    pref_factory_bitmapped(bmdata->prefidx, bmdata->bitfield, bmdata->val);
+    pref_factory_bitmapped(bmdata->prefidx, bmdata->bitfield, bmdata->val, TRUE);
     ext_caller_return_int(bmdata->id, TRUE);
   } else ext_caller_return_int(bmdata->id, FALSE);
   lives_free(data);
@@ -720,7 +719,6 @@ static boolean call_set_current_time(livespointer data) {
     } else {
       if (mainw->current_file > 0 && idata->stime >= 0. && idata->stime <= CLIP_TOTAL_TIME(mainw->current_file)) {
         cfile->pointer_time = lives_ce_update_timeline(0, idata->stime);
-        get_play_times();
       }
     }
     ext_caller_return_int(idata->id, TRUE);
@@ -877,7 +875,6 @@ static boolean call_render_layout(livespointer data) {
 }
 
 
-
 static boolean call_reload_layout(livespointer data) {
   msginfo *mdata = (msginfo *)data;
   boolean resp = FALSE;
@@ -926,8 +923,6 @@ static boolean call_save_layout(livespointer data) {
 }
 
 
-
-
 static boolean call_set_current_fps(livespointer data) {
   opfidata *idata = (opfidata *)data;
   if (mainw != NULL && !mainw->go_away && !mainw->is_processing && mainw->playing_file > -1 && mainw->multitrack == NULL) {
@@ -1076,7 +1071,6 @@ static boolean call_insert_block(livespointer data) {
 }
 
 
-
 static boolean call_remove_block(livespointer data) {
   mblockdata *rdata = (mblockdata *)data;
 
@@ -1129,20 +1123,21 @@ static boolean call_fx_enable(livespointer data) {
         else {
           int count = enabled_in_channels(filter, FALSE);
           if (mainw->playing_file == -1 && count == 0) {
+            // TODO: use osc, but don't set osc_auto
             // return value first because...
             ext_caller_return_int(fxdata->id, (int)TRUE);
             // ...we are going to hang here until playback ends
-            ret = rte_on_off_callback_hook(NULL, LIVES_INT_TO_POINTER(effect_key));
+            ret = rte_key_toggle(effect_key);
             return FALSE;
           } else {
-            ret = rte_on_off_callback_hook(NULL, LIVES_INT_TO_POINTER(effect_key));
+            ret = rte_key_toggle(effect_key);
             mainw->last_grabbable_effect = grab;
           }
         }
       }
     } else {
       if (mainw->rte & (GU641 << (effect_key - 1))) {
-        ret = rte_on_off_callback_hook(NULL, LIVES_INT_TO_POINTER(effect_key));
+        ret = rte_key_toggle(effect_key);
       }
     }
   }
