@@ -1039,6 +1039,7 @@ int free_n_msgs(int frval);
 void d_print_cancelled(void);
 void d_print_failed(void);
 void d_print_done(void);
+void d_print_enough(int frames);
 void d_print_file_error_failed(void);
 
 // general
@@ -1402,7 +1403,7 @@ boolean save_srt_subtitles(lives_clip_t *sfile, double start_time, double end_ti
 
 #define LIVES_TV_CHANNEL1 "http://www.serverwillprovide.com/sorteal/livestvclips/livestv.ogm"
 
-char *dummychar;
+const char *dummychar;
 
 void break_me(void);
 
