@@ -1,16 +1,17 @@
 // merge.c
 // LiVES (lives-exe)
-// (c) G. Finch 2003 - 2018 (salsaman+lives@gmail.com)
+// (c) G. Finch 2003 - 2019 (salsaman+lives@gmail.com)
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
+#include "main.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "main.h"
 #include "callbacks.h"
 #include "merge.h"
 #include "paramwindow.h"
@@ -91,10 +92,6 @@ void create_merge_dialog(void) {
   accel_group = LIVES_ACCEL_GROUP(lives_accel_group_new());
   lives_window_add_accel_group(LIVES_WINDOW(merge_opts->merge_dialog), accel_group);
 
-  if (prefs->show_gui) {
-    lives_window_set_transient_for(LIVES_WINDOW(merge_opts->merge_dialog), LIVES_WINDOW(mainw->LiVES));
-  }
-
   dialog_vbox = lives_dialog_get_content_area(LIVES_DIALOG(merge_opts->merge_dialog));
 
   vbox = lives_vbox_new(FALSE, 0);
@@ -202,14 +199,13 @@ void create_merge_dialog(void) {
 
   // done !
 
-  cancelbutton = lives_standard_button_new_from_stock(LIVES_STOCK_CANCEL, NULL);
-  lives_dialog_add_action_widget(LIVES_DIALOG(merge_opts->merge_dialog), cancelbutton, LIVES_RESPONSE_CANCEL);
-  lives_widget_set_can_focus(cancelbutton, TRUE);
+  cancelbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(merge_opts->merge_dialog), LIVES_STOCK_CANCEL, NULL,
+                 LIVES_RESPONSE_CANCEL);
 
-  okbutton = lives_standard_button_new_from_stock(LIVES_STOCK_OK, NULL);
-  lives_dialog_add_action_widget(LIVES_DIALOG(merge_opts->merge_dialog), okbutton, LIVES_RESPONSE_OK);
-  lives_widget_set_can_focus_and_default(okbutton);
-  lives_widget_grab_default_special(okbutton);
+  okbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(merge_opts->merge_dialog), LIVES_STOCK_OK, NULL,
+             LIVES_RESPONSE_OK);
+
+  lives_button_grab_default_special(okbutton);
 
   lives_signal_connect(LIVES_GUI_OBJECT(cancelbutton), LIVES_WIDGET_CLICKED_SIGNAL,
                        LIVES_GUI_CALLBACK(on_merge_cancel_clicked),
@@ -248,7 +244,7 @@ void on_trans_method_changed(LiVESCombo *combo, livespointer user_data) {
 
   int idx;
 
-  if (!strlen(txt)) {
+  if (strlen(txt) == 0) {
     lives_free(txt);
     return;
   }
@@ -258,7 +254,7 @@ void on_trans_method_changed(LiVESCombo *combo, livespointer user_data) {
   lives_container_foreach(LIVES_CONTAINER(merge_opts->param_vbox), bang, NULL);
   on_paramwindow_cancel_clicked(NULL, rfx);
 
-  idx = lives_list_strcmp_index(merge_opts->trans_list, txt);
+  idx = lives_list_strcmp_index(merge_opts->trans_list, txt, TRUE);
 
   lives_free(txt);
 
@@ -309,10 +305,15 @@ void on_merge_cancel_clicked(LiVESButton *button, livespointer user_data) {
 
 
 void on_merge_ok_clicked(LiVESButton *button, livespointer user_data) {
+  lives_rfx_t *rfx;
+
+  char *com;
+
   int start, end;
 
   int cb_start = 1;
   boolean cb_video_change = FALSE;
+
   int current_file = mainw->current_file;
   int old_frames = clipboard->frames;
 
@@ -322,7 +323,6 @@ void on_merge_ok_clicked(LiVESButton *button, livespointer user_data) {
   int cb_end, excess_frames;
   int times_to_loop = 1;
 
-  lives_rfx_t *rfx;
 
   if (merge_opts->spinbutton_loops != NULL)
     mainw->last_transition_loops = lives_spin_button_get_value_as_int(LIVES_SPIN_BUTTON(merge_opts->spinbutton_loops));
@@ -368,6 +368,11 @@ void on_merge_ok_clicked(LiVESButton *button, livespointer user_data) {
 
   d_print(_("Merging clipboard with selection..."));
 
+  // clear up any leftover old files
+  com = lives_strdup_printf("%s clear_tmp_files \"%s\"", prefs->backend, cfile->handle);
+  lives_system(com, FALSE);
+  lives_free(com);
+
   excess_frames = clipboard->frames - (cfile->end - cfile->start + 1);
   if (excess_frames < 0) excess_frames = 0;
 
