@@ -30,7 +30,7 @@
 #define FX_BLOCK_WIDTH ((int)(80.*widget_opts.scale))
 #define FX_BLOCK_HEIGHT ((int)(20.*widget_opts.scale))
 
-#define MT_TRACK_HEIGHT ((int)(25.*widget_opts.scale))
+#define MT_TRACK_HEIGHT ((int)(35.*widget_opts.scale))
 
 #define TIMECODE_LENGTH 14 ///< length of timecode text entry, must be >12
 
@@ -113,10 +113,10 @@ typedef enum {
   POLY_CLIPS,
   POLY_IN_OUT,
   POLY_FX_STACK,
+  POLY_PARAMS,
   POLY_EFFECTS,
   POLY_TRANS,
-  POLY_COMP,
-  POLY_PARAMS
+  POLY_COMP
 } lives_mt_poly_state_t;
 
 typedef enum {
@@ -163,6 +163,8 @@ struct _mt_opts {
   boolean render_vidp; ///< render video
   boolean render_audp; ///< render audio
   boolean normalise_audp; ///< normalise audio
+  int hpaned_pos;
+  int vpaned_pos;
   LiVESList *aparam_view_list;
 };
 
@@ -170,6 +172,7 @@ struct _mt {
   // widgets
   LiVESWidget *menubar;
   LiVESWidget *top_vbox;
+  LiVESWidget *xtravbox;
   LiVESWidget *hbox;
   LiVESWidget *play_blank;
   LiVESWidget *play_box;
@@ -179,8 +182,11 @@ struct _mt {
   LiVESWidget *in_out_box;
   LiVESWidget *in_hbox;
   LiVESWidget *out_hbox;
+  LiVESWidget *in_frame;
+  LiVESWidget *out_frame;
   LiVESWidget *in_image;
   LiVESWidget *out_image;
+  LiVESWidget *scrolledwindow;
   LiVESWidget *context_box;
   LiVESWidget *context_scroll;
   LiVESWidget *sep_image;
@@ -209,6 +215,10 @@ struct _mt {
   LiVESWidget *jumpback;
   LiVESWidget *render;
   LiVESWidget *prerender_aud;
+  LiVESWidget *message_box;
+  LiVESWidget *msg_area;
+  LiVESWidget *msg_scrollbar;
+  LiVESAdjustment *msg_adj;
   LiVESWidget *fx_block;
   LiVESWidget *fx_blockv;
   LiVESWidget *fx_blocka;
@@ -231,7 +241,7 @@ struct _mt {
   LiVESWidget *mm_select;
   LiVESWidget *ins_menuitem;
   LiVESWidget *ins_normal;
-  LiVESToolItem *grav_menuitem;
+  LiVESWidget *grav_menuitem;
   LiVESWidget *grav_label;
   LiVESWidget *grav_normal;
   LiVESWidget *grav_left;
@@ -358,7 +368,7 @@ struct _mt {
   LiVESToolItem *sep1;
   LiVESToolItem *sep2;
   LiVESToolItem *sep3;
-  LiVESToolItem *sep4;
+  LiVESWidget *sep4;
 
   LiVESWidget *btoolbar;
   LiVESWidget *btoolbar2;
@@ -369,6 +379,17 @@ struct _mt {
   LiVESWidget *hseparator;
   LiVESWidget *hseparator2;
 
+  LiVESWidget *files_menu;
+  LiVESWidget *edit_menu;
+  LiVESWidget *play_menu;
+  LiVESWidget *effects_menu;
+  LiVESWidget *tracks_menu;
+  LiVESWidget *selection_menu;
+  LiVESWidget *tools_menu;
+  LiVESWidget *render_menu;
+  LiVESWidget *view_menu;
+  LiVESWidget *help_menu;
+
   LiVESWidget *open_menu;
 #ifdef HAVE_WEBM
   LiVESWidget *open_loc_menu;
@@ -380,10 +401,7 @@ struct _mt {
   LiVESWidget *device_menu;
 #endif
   LiVESWidget *recent_menu;
-  LiVESWidget *recent1;
-  LiVESWidget *recent2;
-  LiVESWidget *recent3;
-  LiVESWidget *recent4;
+  LiVESWidget *recent[N_RECENT_FILES];
 
   LiVESWidget *time_scrollbar;
   LiVESWidget *show_layout_errors;
@@ -446,6 +464,8 @@ struct _mt {
 
   ulong tc_func;
 
+  ulong sw_func;
+
   weed_plant_t *event_list;
 
   weed_plant_t *init_event;   ///< current editable values
@@ -491,6 +511,8 @@ struct _mt {
 
   boolean mt_frame_preview;
 
+  boolean in_sensitise;
+
   lives_rfx_t *current_rfx;
 
   char layout_name[PATH_MAX];
@@ -577,6 +599,10 @@ struct _mt {
 
   int exact_preview;
 
+  int preview_layer;
+
+  weed_plant_t *solo_inst; ///< instance to view solo in the frame preview
+
   LiVESList *tl_marks;
 
   weed_plant_t *pb_start_event; ///< FRAME event from which we start playback
@@ -661,6 +687,8 @@ struct _lives_amixer_t {
 /// blocks MUST only contain frames from a single clip. They MAY NOT contain blank frames.
 ///
 /// start and end events MUST be FRAME events
+
+/// TODO: add scrap_file_offset so we can treat scrapfile like an ordinary clip
 struct _track_rect {
   ulong uid;
 
@@ -876,6 +904,7 @@ void on_fx_insa_clicked(LiVESWidget *button, livespointer mt);
 // utils
 uint32_t event_list_get_byte_size(lives_mt *, weed_plant_t *event_list, int *num_events);  ///< returns bytes and sets num_events
 boolean event_list_rectify(lives_mt *, weed_plant_t *event_listy);
+void reset_renumbering(void);
 boolean make_backup_space(lives_mt *, size_t space_needed);
 void activate_mt_preview(lives_mt *); ///< sensitize Show Preview and Apply buttons
 void **mt_get_pchain(void);
@@ -923,7 +952,7 @@ void remove_current_from_affected_layouts(lives_mt *);
 
 // auto backup
 uint32_t mt_idle_add(lives_mt *);
-void recover_layout(void);
+boolean recover_layout(void);
 void recover_layout_cancelled(boolean is_startup);
 boolean write_backup_layout_numbering(lives_mt *);
 boolean mt_auto_backup(livespointer mt);
@@ -966,14 +995,8 @@ typedef enum {
 
 // array max: TODO - use dynamic arrays to bypass track limits
 
-#ifdef IS_MINGW
-#define MAX_TRACKS 32768
-#define MAX_VIDEO_TRACKS 32768
-#define MAX_AUDIO_TRACKS 32768
-#else
 #define MAX_TRACKS 65536
 #define MAX_VIDEO_TRACKS 65536
 #define MAX_AUDIO_TRACKS 65536
-#endif
 
 #endif
