@@ -30,9 +30,15 @@
 static int font_cmp(const void *p1, const void *p2);
 #endif
 
-//
-// code things from scribbler.c
-//
+
+static void fill_bckg(lives_painter_t *cr, double x, double y, double dx, double dy) {
+  lives_painter_new_path(cr);
+  lives_painter_rectangle(cr, x, y, dx, dy);
+  lives_painter_fill(cr);
+  lives_painter_new_path(cr);
+}
+
+
 static void getxypos(LingoLayout *layout, double *px, double *py, int width, int height, boolean cent, double *pw, double *ph) {
   // calc coords of text, text will fit so it goes to bottom. Set cent to center text.
 
@@ -47,35 +53,311 @@ static void getxypos(LingoLayout *layout, double *px, double *py, int width, int
   lingo_layout_get_size(layout, &w_, &h_, width, height);
 
   // scale width, height to pixels
-  if (pw)
-    *pw = ((double)w_) / LINGO_SCALE;
-  if (ph)
-    *ph = ((double)h_) / LINGO_SCALE;
-
-  // xpos (left or centered)
-
-  if (cent) {
-    d = ((double)w_) / LINGO_SCALE;
-    d /= 2.0;
-    d = (width >> 1) - d;
-  } else d = 0.0;
-
-  if (px) *px = d;
+  d = ((double)h_) / (double)LINGO_SCALE;
+  if (ph) *ph = d;
 
   // ypos (adjusted so text goes to bottom)
+  if (py) *py = height - (int)d;
 
-  d = ((double)h_) / LINGO_SCALE;
-  d = height - d;
+  d = ((double)w_) / (double)LINGO_SCALE;
+  if (pw) *pw = d;
 
-  if (py) *py = d;
+  if (px) *px = cent ? (double)(width >> 1) - d / 2. : 0.;
 }
 
 
-static void fill_bckg(lives_painter_t *cr, double x, double y, double dx, double dy) {
-  lives_painter_new_path(cr);
-  lives_painter_rectangle(cr, x, y, dx, dy);
-  lives_painter_fill(cr);
+static char *rewrap_text(char *text) {
+  // find the longest line and move the last word to the following line
+  // if there is no following line we add one
+  // if there are no spaces in the line we truncate
+  size_t maxlen = 0;
+
+  char **lines;
+  char *jtext, *tmp;
+#ifdef REFLOW_TEXT
+  char *first, *second;
+  int j;
+#endif
+  int i;
+  int numlines;
+  int maxline = -1;
+  boolean needs_nl = FALSE;
+
+  if (text == NULL || strlen(text) == 0) return NULL;
+
+  jtext = lives_strdup("");
+  numlines = get_token_count(text, '\n');
+  lines = lives_strsplit(text, "\n", numlines);
+
+  for (i = 0; i < numlines; i++) {
+    if (strlen(lines[i]) > maxlen) {
+      maxlen = strlen(lines[i]);
+      maxline = i;
+    }
+  }
+  if (maxlen < 2) {
+    lives_strfreev(lines);
+    return jtext;
+  }
+  for (i = 0; i < numlines; i++) {
+    if (i == maxline) {
+#ifdef REFLOW_TEXT
+      for (j = maxlen - 1; j > 0; j--) {
+        // skip the final character - if it's a space we aren't going to move it yet
+        // if it's not a space we aren't going to move it yet
+        if (lines[i][j - 1] == ' ') {
+          // up to and including space
+          first = lives_strndup(lines[i], j);
+          // after space
+          second = lines[i] + j;
+          tmp = lives_strdup_printf("%s%s%s\n%s", jtext, needs_nl ? "\n" : "", first, second);
+          lives_free(first);
+          lives_free(jtext);
+          jtext = tmp;
+          needs_nl = FALSE;
+          break;
+        }
+        // no space in line, truncate last char
+        lines[i][maxlen - 1] = 0;
+        if (maxlen > 3)
+          lives_snprintf(&lines[i][maxlen - 4], 4, "%s", "...");
+        needs_nl = TRUE;
+      }
+#endif
+      //g_print("maxlen %ld for %s\n", maxlen, lines[i]);
+      lines[i][maxlen - 1] = 0;
+      if (maxlen > 5)
+        lives_snprintf(&lines[i][maxlen - 4], 4, "%s", "...");
+      //g_print("Trying with %s\n", lines[i]);
+    }
+    tmp = lives_strdup_printf("%s%s%s", jtext, needs_nl ? "\n" : "", lines[i]);
+    lives_free(jtext);
+    jtext = tmp;
+    needs_nl = TRUE;
+  }
+  lives_strfreev(lines);
+  return jtext;
+}
+
+
+#ifdef MUST_FIT
+static char *remove_first_line(char *text) {
+  int i;
+  size_t tlen = strlen(text);
+  for (i = 0; i < tlen; i++) {
+    if (text[i] == '\n') return strdup(text + i + 1);
+  }
+  return NULL;
+}
+#endif
+
+
+void layout_to_lives_painter(LingoLayout *layout, lives_painter_t *cr, lives_text_mode_t mode, lives_colRGBA64_t *fg,
+                             lives_colRGBA64_t *bg, int dwidth, int dheight, double x_bg, double y_bg, double x_text, double y_text) {
+  double b_alpha = 1.;
+  double f_alpha = 1.;
+
+  if (bg != NULL) b_alpha = (double)bg->alpha / 65535.;
+  if (fg != NULL) f_alpha = (double)fg->alpha / 65535.;
+
+  if (cr == NULL) return;
+
+  switch (mode) {
+  case LIVES_TEXT_MODE_BACKGROUND_ONLY:
+    lingo_layout_set_text(layout, "", -1);
+  case LIVES_TEXT_MODE_FOREGROUND_AND_BACKGROUND:
+    lives_painter_set_source_rgba(cr, (double)bg->red / 66535., (double)bg->green / 66535., (double)bg->blue / 66535., b_alpha);
+    fill_bckg(cr, x_bg, y_bg, dwidth, dheight);
+    break;
+  default:
+    break;
+  }
+
   lives_painter_new_path(cr);
+  lives_painter_move_to(cr, x_text, y_text);
+  lives_painter_set_source_rgba(cr, (double)fg->red / 66535., (double)fg->green / 66535., (double)fg->blue / 66535., f_alpha);
+}
+
+
+//#define DEBUG_MSGS
+LingoLayout *layout_nth_message_at_bottom(int n, int width, int height, LiVESWidget *widget, int *linecount) {
+  // create a layout, using text properties for widget
+  //
+  // nth message in mainw->messages should be at the bottom
+  // or if there are insufficient messages then we render from message 0
+
+  // also we want to justify text, splitting on words so that it fits width
+
+#ifdef GUI_GTK
+  LingoLayout *layout;
+  LingoContext *ctx;
+  weed_plant_t *msg;
+
+  char *readytext, *testtext, *newtext = NULL, *tmp, *xx;
+  int w = 0, h = 0, pw;
+  int error;
+  int totlines = 0;
+  int whint = 0;
+  int slen;
+
+  boolean heightdone = FALSE;
+  boolean needs_newline = FALSE;
+
+  if (width < 32 || height < 32) return NULL;
+
+  ctx = lives_widget_create_lingo_context(widget);
+  if (ctx == NULL || !LINGO_IS_CONTEXT(ctx)) return NULL;
+
+  layout = lingo_layout_new(ctx);
+  if (layout == NULL || !LINGO_IS_LAYOUT(layout)) {
+    lives_object_unref(ctx);
+    return NULL;
+  }
+
+  readytext = lives_strdup("");
+
+  msg = get_nth_info_message(n);
+
+#ifdef DEBUG_MSGS
+  g_print("Want msg number %d at bottom\n%s", n, weed_get_string_value(msg, WEED_LEAF_LIVES_MESSAGE_STRING, &error));
+#endif
+
+  while (1) {
+    if (newtext == NULL) {
+      if (msg == NULL) break;
+      newtext = weed_get_string_value(msg, WEED_LEAF_LIVES_MESSAGE_STRING, &error);
+      if (error != WEED_NO_ERROR) break;
+      if (newtext == NULL) break;
+      totlines += get_token_count(newtext, '\n');
+#ifdef DEBUG_MSGS
+      g_print("Got msg:%s\ntotal is now %d lines\n", newtext, totlines);
+#endif
+      if (msg == mainw->msg_list) msg = NULL;
+      else msg = weed_get_plantptr_value(msg, WEED_LEAF_PREVIOUS, &error);
+    }
+
+    testtext = lives_strdup_printf("%s%s%s", newtext, needs_newline ? "\n" : "", readytext);
+    needs_newline = TRUE;
+    lingo_layout_set_text(layout, testtext, -1);
+    lingo_layout_get_size(layout, &w, &h, 0, 0);
+
+    h /= LINGO_SCALE;
+    w /= LINGO_SCALE;
+
+#ifdef DEBUG_MSGS
+    g_print("Sizes %d %d window, %d %d layout (%s)\n", width, height, w, h, testtext);
+#endif
+
+    if (h > height) {
+#ifdef DEBUG_MSGS
+      g_print("Too high !\n");
+#endif
+
+#ifdef MUST_FIT
+      while (h > height) {
+        // text was too high, start removing lines from the top until it fits
+        tmp = remove_first_line(newtext);
+        lives_free(newtext);
+        newtext = tmp;
+        totlines--;
+        if (newtext == NULL) break; // no more to remove, we are done !
+#ifdef DEBUG_MSGS
+        g_print("Retry with (%d) |%s|\n", totlines, newtext);
+#endif
+        lives_free(testtext);
+        testtext = lives_strdup_printf("%s%s", newtext, readytext);
+#ifdef DEBUG_MSGS
+        g_print("Testing with:%s:\n", testtext);
+#endif
+        pango_layout_set_width(layout, width * LINGO_SCALE);
+        lingo_layout_set_text(layout, testtext, -1);
+        lingo_layout_get_size(layout, NULL, &h, 0, 0);
+
+        h /= LINGO_SCALE;
+      }
+#endif
+      heightdone = TRUE;
+    }
+
+    // height was ok, now let's check the width
+    if (w > width) {
+      int jumpval = 1, dirn = -1, tjump = 0;
+      //double nscale = 2.;
+      // text was too wide
+#ifdef DEBUG_MSGS
+      g_print("Too wide !!!\n");
+#endif
+      totlines -= get_token_count(newtext, '\n');
+      slen = (int)strlen(newtext);
+      while (1) {
+        // for now we just truncate and elipsise lines
+        tjump = dirn * jumpval;
+        /* if (tjump >= slen && dirn == -1) { */
+        /*   jumpval = slen / 2; */
+        /*   tjump = dirn * jumpval; */
+        /* } */
+        /* g_print("pt b %d %d %d\n", tjump, dirn, jumpval); */
+        if (whint == 0 || whint + 4 > slen) {
+          xx = lives_strndup(newtext, slen + tjump);
+        } else {
+          xx = lives_strndup(newtext, whint + 4 + tjump);
+        }
+        tmp = rewrap_text(xx);
+        lives_free(xx);
+#ifdef DEBUG_MSGS
+        g_print("Retry with (%d) |%s|\n", totlines, xx);
+#endif
+        if (tmp == NULL) break;
+        // check width again, just looking at new part
+        lingo_layout_set_text(layout, tmp, -1);
+        lingo_layout_get_size(layout, &pw, NULL, 0, 0);
+        w = pw / LINGO_SCALE;
+        if (w >= width) {
+          //dirn = -1;
+          jumpval++;
+          if (whint <= 0 || (int)strlen(tmp) < whint) whint = (int)strlen(tmp);
+        } else {
+          break;
+        }
+        /*   if (jumpval == 1) break; */
+        /*   dirn = 1; */
+        /*   nscale = 0.5; */
+        /* } */
+        /* if (jumpval > 1) jumpval = (int)(jumpval * nscale + .9); */
+        lives_free(tmp);
+      }
+#ifdef DEBUG_MSGS
+      g_print("Width OK now\n");
+#endif
+      lives_free(newtext);
+      newtext = tmp;
+      totlines += get_token_count(newtext, '\n');
+      continue; // width is OK again, need to recheck height
+    }
+
+    lives_free(newtext);
+    newtext = NULL;
+    lives_free(readytext);
+    readytext = testtext;
+#ifdef DEBUG_MSGS
+    g_print("|%s| passed size tests\n", readytext);
+#endif
+    if (heightdone) break;
+  }
+
+  // result is now in readytext
+  lingo_layout_set_text(layout, readytext, -1);
+
+  if (linecount != NULL) *linecount = totlines;
+
+#ifdef DEBUG_MSGS
+  g_print("|%s| FINAL !!\n", readytext);
+#endif
+  lives_free(readytext);
+  lives_object_unref(ctx);
+  return layout;
+#endif
+  return NULL;
 }
 
 
@@ -135,9 +417,11 @@ static int font_cmp(const void *p1, const void *p2) {
 
 LingoLayout *render_text_to_cr(LiVESWidget *widget, lives_painter_t *cr, const char *text, const char *fontname,
                                double size, lives_text_mode_t mode, lives_colRGBA64_t *fg, lives_colRGBA64_t *bg,
-                               boolean center, boolean rising, double top, int offs_x, int width, int height) {
+                               boolean center, boolean rising, double top, int offs_x, double dwidth, double dheight) {
   // fontname may be eg. "Sans"
 
+  // size is in device units, i.e. pixels
+
   // ypos:
   // if "rising" is TRUE, text will be aligned to fit to bottom
   // if "rising" is FALSE,  "top" (0.0 -> 1.0) is used
@@ -152,14 +436,6 @@ LingoLayout *render_text_to_cr(LiVESWidget *widget, lives_painter_t *cr, const c
   LingoLayout *layout;
 
   double x_pos = 0., y_pos = 0.;
-  double x_text, y_text;
-  double dwidth = width, dheight = height;
-
-  double b_alpha = 1.;
-  double f_alpha = 1.;
-
-  if (bg != NULL) b_alpha = (double)bg->alpha / 65535.;
-  if (fg != NULL) f_alpha = (double)fg->alpha / 65535.;
 
   if (cr == NULL) return NULL;
 
@@ -177,7 +453,7 @@ LingoLayout *render_text_to_cr(LiVESWidget *widget, lives_painter_t *cr, const c
 
     pango_layout_set_font_description(layout, font);
   }
-  pango_layout_set_text(layout, text, -1);
+  pango_layout_set_markup(layout, text, -1);
 #endif
 
 #ifdef GUI_QT
@@ -187,15 +463,12 @@ LingoLayout *render_text_to_cr(LiVESWidget *widget, lives_painter_t *cr, const c
 #ifndef GUI_QT
   if (rising || center || mode == LIVES_TEXT_MODE_FOREGROUND_AND_BACKGROUND)
 #endif
-    getxypos(layout, &x_pos, &y_pos, width, height, center, &dwidth, &dheight);
+    getxypos(layout, &x_pos, &y_pos, dwidth, dheight, center, &dwidth, &dheight);
 
-  if (!rising) y_pos = y_text = height * top;
+  if (!rising) y_pos = dheight * top;
 
   if (!center) x_pos += offs_x;
 
-  x_text = x_pos;
-  y_text = y_pos;
-
   /*  lives_painter_new_path(cr);
   lives_painter_rectangle(cr,offs_x,0,width,height);
   lives_painter_clip(cr);*/
@@ -203,24 +476,7 @@ LingoLayout *render_text_to_cr(LiVESWidget *widget, lives_painter_t *cr, const c
   if (center) lingo_layout_set_alignment(layout, LINGO_ALIGN_CENTER);
   else lingo_layout_set_alignment(layout, LINGO_ALIGN_LEFT);
 
-  switch (mode) {
-  case LIVES_TEXT_MODE_BACKGROUND_ONLY:
-    lingo_layout_set_text(layout, "", -1);
-  case LIVES_TEXT_MODE_FOREGROUND_AND_BACKGROUND:
-    lives_painter_set_source_rgba(cr, (double)bg->red / 66535., (double)bg->green / 66535., (double)bg->blue / 66535., b_alpha);
-    fill_bckg(cr, x_pos, y_pos, dwidth, dheight);
-    break;
-  default:
-    break;
-  }
-
-  lives_painter_new_path(cr);
-  lives_painter_move_to(cr, x_text, y_text);
-  lives_painter_set_source_rgba(cr, (double)fg->red / 66535., (double)fg->green / 66535., (double)fg->blue / 66535., f_alpha);
-
-#ifdef GUI_QT
-  lingo_layout_set_coords(layout, x_pos, y_pos, dwidth, dheight);
-#endif
+  layout_to_lives_painter(layout, cr, mode, fg, bg, dwidth, dheight, x_pos, y_pos, x_pos, y_pos);
 
 #ifdef GUI_GTK
   if (font != NULL) {
@@ -240,10 +496,6 @@ weed_plant_t *render_text_to_layer(weed_plant_t *layer, const char *text, const
 
   lives_painter_t *cr;
 
-  lives_painter_surface_t *surface;
-
-  void *src;
-
   LingoLayout *layout;
 
   int width, height, error;
@@ -257,8 +509,7 @@ weed_plant_t *render_text_to_layer(weed_plant_t *layer, const char *text, const
   if (cr == NULL) return layer; ///< error occured
 
   layout = render_text_to_cr(NULL, cr, text, fontname, size, mode, fg_col, bg_col, center, rising, top, 0, width, height);
-
-  lingo_painter_show_layout(cr, layout);
+  if (layout != NULL && LINGO_IS_LAYOUT(layout)) lingo_painter_show_layout(cr, layout);
 
   // do not !!
   //lives_painter_paint(cr);
@@ -267,14 +518,14 @@ weed_plant_t *render_text_to_layer(weed_plant_t *layer, const char *text, const
 
   if (layout) lives_object_unref(layout);
 
-  surface = lives_painter_get_target(cr);
-  src = lives_painter_image_surface_get_data(surface);
-  lives_free(src);
+  //surface = lives_painter_get_target(cr);
+  //src = lives_painter_image_surface_get_data(surface);
+  //lives_free(src);
+
   lives_painter_destroy(cr);
   return layer;
 }
 
-
 static const char *cr_str = "\x0D";
 static const char *lf_str = "\x0A";
 
