@@ -1,6 +1,6 @@
 // paramspecial.c
 // LiVES
-// (c) G. Finch 2004 - 2018 <salsaman+lives@gmail.com>
+// (c) G. Finch 2004 - 2019 <salsaman+lives@gmail.com>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -28,6 +28,7 @@
 static lives_special_aspect_t aspect;
 static lives_special_framedraw_rect_t framedraw;
 static LiVESList *fileread;
+static LiVESList *filewrite;
 static LiVESList *passwd_widgets;
 
 
@@ -44,6 +45,7 @@ void init_special(void) {
   mergealign.start_param = mergealign.end_param = NULL;
   passwd_widgets = NULL;
   fileread = NULL;
+  filewrite = NULL;
 }
 
 
@@ -134,6 +136,13 @@ void add_to_special(const char *sp_string, lives_rfx_t *rfx) {
     int idx = atoi(array[1]);
     fileread = lives_list_append(fileread, (livespointer)&rfx->params[idx]);
 
+    // ensure we get an entry and not a text_view
+    if ((int)rfx->params[idx].max > RFX_TEXT_MAGIC) rfx->params[idx].max = (double)RFX_TEXT_MAGIC;
+  } else if (!strcmp(array[0], "filewrite")) {
+    int idx = atoi(array[1]);
+    filewrite = lives_list_append(filewrite, (livespointer)&rfx->params[idx]);
+    rfx->params[idx].edited = TRUE;
+
     // ensure we get an entry and not a text_view
     if ((int)rfx->params[idx].max > RFX_TEXT_MAGIC) rfx->params[idx].max = (double)RFX_TEXT_MAGIC;
   } else if (!strcmp(array[0], "password")) {
@@ -319,6 +328,7 @@ void check_for_special(lives_rfx_t *rfx, lives_param_t *param, LiVESBox *pbox) {
   while (slist != NULL) {
     if (param == (lives_param_t *)(slist->data)) {
       int epos;
+      char *def_dir;
 
       param->special_type = LIVES_PARAM_SPECIAL_TYPE_FILEREAD;
 
@@ -332,14 +342,29 @@ void check_for_special(lives_rfx_t *rfx, lives_param_t *param, LiVESBox *pbox) {
 
       if (box == NULL) return;
 
+      def_dir = lives_get_current_dir();
+
+      if (LIVES_IS_ENTRY(param->widgets[0])) {
+        if (strlen(lives_entry_get_text(LIVES_ENTRY(param->widgets[0])))) {
+          char dirnamex[PATH_MAX];
+          lives_snprintf(dirnamex, PATH_MAX, "%s", lives_entry_get_text(LIVES_ENTRY(param->widgets[0])));
+          get_dirname(dirnamex);
+          lives_free(def_dir);
+          def_dir = lives_strdup(dirnamex);
+        }
+      }
+
       epos = get_box_child_index(LIVES_BOX(box), param->widgets[0]);
 
-      buttond = lives_standard_file_button_new(FALSE, lives_get_current_dir());
+      param->widgets[1] = buttond = lives_standard_file_button_new(FALSE, def_dir);
+      lives_free(def_dir);
       lives_box_pack_start(LIVES_BOX(box), buttond, FALSE, FALSE, widget_opts.packing_width);
       lives_box_reorder_child(LIVES_BOX(box), buttond, epos); // insert after label, before textbox
       lives_signal_connect(buttond, LIVES_WIDGET_CLICKED_SIGNAL, LIVES_GUI_CALLBACK(on_filesel_button_clicked), (livespointer)param->widgets[0]);
 
       if (!lives_widget_is_sensitive(param->widgets[0])) lives_widget_set_sensitive(buttond, FALSE);
+      lives_widget_set_show_hide_with(param->widgets[0], buttond);
+      lives_widget_set_sensitive_with(param->widgets[0], buttond);
 
       if (LIVES_IS_ENTRY(param->widgets[0])) {
         lives_entry_set_editable(LIVES_ENTRY(param->widgets[0]), FALSE);
@@ -354,6 +379,49 @@ void check_for_special(lives_rfx_t *rfx, lives_param_t *param, LiVESBox *pbox) {
     slist = slist->next;
   }
 
+  slist = filewrite;
+  while (slist != NULL) {
+    if (param == (lives_param_t *)(slist->data)) {
+      int epos;
+
+      param->special_type = LIVES_PARAM_SPECIAL_TYPE_FILEWRITE;
+
+      if (param->widgets[0] == NULL) continue;
+
+      box = lives_widget_get_parent(param->widgets[0]);
+
+      while (box != NULL && !LIVES_IS_HBOX(box)) {
+        box = lives_widget_get_parent(box);
+      }
+
+      if (box == NULL) return;
+
+      epos = get_box_child_index(LIVES_BOX(box), param->widgets[0]);
+
+      param->widgets[1] = buttond = lives_standard_file_button_new(FALSE, NULL);
+
+      lives_widget_object_set_data(LIVES_WIDGET_OBJECT(buttond), "filesel_type", (livespointer)LIVES_FILE_SELECTION_SAVE);
+      lives_box_pack_start(LIVES_BOX(box), buttond, FALSE, FALSE, widget_opts.packing_width);
+      lives_box_reorder_child(LIVES_BOX(box), buttond, epos); // insert after label, before textbox
+      lives_signal_connect(buttond, LIVES_WIDGET_CLICKED_SIGNAL, LIVES_GUI_CALLBACK(on_filesel_button_clicked), (livespointer)param->widgets[0]);
+
+      if (!lives_widget_is_sensitive(param->widgets[0])) lives_widget_set_sensitive(buttond, FALSE);
+      lives_widget_set_show_hide_with(param->widgets[0], buttond);
+      lives_widget_set_sensitive_with(param->widgets[0], buttond);
+
+      if (LIVES_IS_ENTRY(param->widgets[0])) {
+        lives_entry_set_editable(LIVES_ENTRY(param->widgets[0]), TRUE);
+        if (param->widgets[1] != NULL &&
+            LIVES_IS_LABEL(param->widgets[1]) &&
+            lives_label_get_mnemonic_widget(LIVES_LABEL(param->widgets[1])) != NULL)
+          lives_label_set_mnemonic_widget(LIVES_LABEL(param->widgets[1]), buttond);
+        lives_entry_set_max_length(LIVES_ENTRY(param->widgets[0]), PATH_MAX);
+      }
+    }
+
+    slist = slist->next;
+  }
+
   // password fields
 
   slist = passwd_widgets;
@@ -418,8 +486,36 @@ void after_aspect_height_changed(LiVESToggleButton *spinbutton, livespointer use
 }
 
 
-void special_cleanup(void) {
+boolean check_filewrite_overwrites(void) {
+  // check all writeable files which were user edited (param->edited), to make sure we don't overwrite without permission
+  // if the value was set by the file button we would have checked there, and param->edited will be FALSE
+
+  if (filewrite != NULL) {
+    LiVESList *slist = filewrite;
+    while (slist != NULL) {
+      lives_param_t *param = (lives_param_t *)(slist->data);
+      if (param->edited) {
+        // check for overwrite
+        if (LIVES_IS_ENTRY(param->widgets[0])) {
+          if (strlen(lives_entry_get_text(LIVES_ENTRY(param->widgets[0])))) {
+            if (!check_file(lives_entry_get_text(LIVES_ENTRY(param->widgets[0])), TRUE)) {
+              return FALSE;
+            }
+          }
+        }
+      }
+      slist = slist->next;
+    }
+  }
+  return TRUE;
+}
+
+
+boolean special_cleanup(void) {
   // free some memory now
+  boolean ret = TRUE;
+
+  if (!check_filewrite_overwrites()) return FALSE;
 
   mainw->framedraw = mainw->framedraw_reset = NULL;
   mainw->framedraw_spinbutton = NULL;
@@ -433,11 +529,19 @@ void special_cleanup(void) {
   mainw->framedraw_preview = NULL;
 
   if (framedraw.extra_params != NULL) lives_free(framedraw.extra_params);
+  framedraw.extra_params = NULL;
 
   if (fileread != NULL) lives_list_free(fileread);
+  fileread = NULL;
+
+  if (filewrite != NULL) lives_list_free(filewrite);
+  filewrite = NULL;
+
   if (passwd_widgets != NULL) lives_list_free(passwd_widgets);
+  passwd_widgets = NULL;
 
   framedraw.added = FALSE;
+  return ret;
 }
 
 
