@@ -372,6 +372,7 @@ typedef enum {
 
   // text widget types
   LIVES_PARAM_SPECIAL_TYPE_FILEREAD,
+  LIVES_PARAM_SPECIAL_TYPE_FILEWRITE,
   LIVES_PARAM_SPECIAL_TYPE_PASSWORD,
 
   // misc types
@@ -429,6 +430,7 @@ typedef struct {
   boolean onchange; ///< is there a trigger ?
 
   boolean changed;
+  boolean edited;
 
   boolean change_blocked;
 
@@ -556,6 +558,7 @@ typedef struct {
   LiVESWidget *dialog;
   LiVESWidget *spinbuttonh;
   LiVESWidget *spinbuttonw;
+  LiVESWidget *apply_fx;
   LiVESWidget *fps_entry;
   LiVESWidget *pal_entry;
   lives_rfx_t *rfx;
