@@ -1,6 +1,6 @@
 // pulse.h
 // LiVES (lives-exe)
-// (c) G. Finch 2005 - 2012
+// (c) G. Finch 2005 - 2019
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -28,26 +28,25 @@ typedef struct {
   void *data;
 } audio_buffer_t;
 
-
 typedef struct {
   pa_threaded_mainloop *mloop;
   pa_context *con;
   pa_stream *pstream;
   pa_proplist *pa_props;
 
-  pa_usec_t usec_start;
+  volatile pa_usec_t usec_start;
 
   int str_idx;
 
-  pa_context_state_t state;
+  pa_stream_state_t state;
 
   // app side
-  int64_t in_arate; /**< samples(frames) per second */
+  volatile int in_arate; /**< samples(frames) per second */
   uint64_t in_achans; /**< number of input channels(1 is mono, 2 stereo etc..) */
   uint64_t in_asamps;
 
   // server side
-  int64_t out_arate; /**< samples(frames) per second */
+  int out_arate; /**< samples(frames) per second */
   uint64_t out_achans; /**< number of output channels(1 is mono, 2 stereo etc..) */
   uint64_t out_asamps;
 
@@ -66,7 +65,9 @@ typedef struct {
 
   uint8_t *sound_buffer; ///< transformed data
 
-  float volume[PULSE_MAX_OUTPUT_CHANS]; ///< amount volume, 1.0 is full volume
+  pa_cvolume volume;
+
+  //float volume[PULSE_MAX_OUTPUT_CHANS]; ///< amount volume, 1.0 is full volume (currently unused, see volume_linear)
 
   boolean in_use; /**< true if this device is currently in use */
   boolean mute;
@@ -78,11 +79,9 @@ typedef struct {
 
   boolean is_paused;
 
-  volatile int64_t
-  audio_ticks; ///< ticks when we did the last seek, used to calculate current ticks from audio, in tandem with frames_written
-
   int fd; /**< if >0 we are playing from a lives_clip_t */
-  volatile off_t seek_pos;
+  off_t seek_pos;
+  volatile off_t real_seek_pos;
   off_t seek_end;
   boolean usigned;
   boolean reverse_endian;
@@ -101,15 +100,15 @@ typedef struct {
 
   uint64_t chunk_size;
 
+  double volume_linear; ///< TODO: use perchannel volume[]
+
   volatile int astream_fd;
 
   volatile float abs_maxvol_heard;
 
-  volatile boolean waitforop;
+  volatile boolean is_corked;
 } pulse_driver_t;
 
-
-
 // TODO - rationalise names
 
 boolean lives_pulse_init(short startup_phase);  ///< init server, mainloop and context
@@ -129,28 +128,28 @@ void pulse_aud_pb_ready(int fileno);
 size_t pulse_flush_read_data(pulse_driver_t *, int fileno, size_t rbytes, boolean rev_endian, void *data);
 
 void pulse_driver_uncork(pulse_driver_t *);
+void pulse_driver_cork(pulse_driver_t *);
 
 boolean pulse_try_reconnect(void);
 
 // utils
 volatile aserver_message_t *pulse_get_msgq(pulse_driver_t *); ///< pull last msg from msgq, or return NULL
 
-int64_t pulse_audio_seek_bytes(pulse_driver_t *, int64_t bytes);  ///< seek to byte position
+int64_t pulse_audio_seek_bytes(pulse_driver_t *pulsed, int64_t bytes, lives_clip_t *sfile);
 
-int64_t lives_pulse_get_time(pulse_driver_t *, boolean absolute); ///< get time from pa, in 10^-8 seconds
+void pa_time_reset(pulse_driver_t *pulsed, int64_t offset);
+
+uint64_t lives_pulse_get_time(pulse_driver_t *); ///< get time from pa, in 10^-8 seconds
 
 double lives_pulse_get_pos(pulse_driver_t *);
 
 void pa_mloop_lock(void);
 void pa_mloop_unlock(void);
 
-
 //////////////////////
 
 boolean pulse_audio_seek_frame(pulse_driver_t *, int frame);  ///< seek to (video) frame
 
 void pulse_get_rec_avals(pulse_driver_t *);
 
-
-
 #endif
