@@ -1,6 +1,6 @@
 // resample.c
 // LiVES
-// (c) G. Finch 2004 - 2018 <salsaman+lives@gmail.com>
+// (c) G. Finch 2004 - 2019 <salsaman+lives@gmail.com>
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
@@ -555,7 +555,6 @@ weed_plant_t *quantise_events(weed_plant_t *in_list, double qfps, boolean allow_
       // event is before slot, note it and get next event
       if (out_clips != NULL) lives_free(out_clips);
       if (out_frames != NULL) lives_free(out_frames);
-
       numframes = weed_leaf_num_elements(event, WEED_LEAF_CLIPS);
       out_clips = weed_get_int_array(event, WEED_LEAF_CLIPS, &error);
       out_frames = weed_get_int_array(event, WEED_LEAF_FRAMES, &error);
@@ -574,6 +573,7 @@ weed_plant_t *quantise_events(weed_plant_t *in_list, double qfps, boolean allow_
 
       //  in some cases we allow a gap before writing our first FRAME out event
       if (!(is_first && allow_gap)) {
+        if (last_audio_event == event && needs_audio) add_audio = TRUE;
         if (in_tc - (out_tc + tl) < nearest_tc) {
           if (event != NULL) {
             if (out_clips != NULL) lives_free(out_clips);
@@ -582,7 +582,6 @@ weed_plant_t *quantise_events(weed_plant_t *in_list, double qfps, boolean allow_
             numframes = weed_leaf_num_elements(event, WEED_LEAF_CLIPS);
             out_clips = weed_get_int_array(event, WEED_LEAF_CLIPS, &error);
             out_frames = weed_get_int_array(event, WEED_LEAF_FRAMES, &error);
-            if (last_audio_event == event && needs_audio) add_audio = TRUE;
             if (error == WEED_ERROR_MEMORY_ALLOCATION) {
               do_memory_error_dialog();
               event_list_free(out_list);
@@ -596,6 +595,9 @@ weed_plant_t *quantise_events(weed_plant_t *in_list, double qfps, boolean allow_
             event_list_free(out_list);
             return NULL;
           }
+          if (weed_plant_has_leaf(event, WEED_LEAF_HOST_SCRAP_FILE_OFFSET)) {
+            weed_set_int64_value(shortcut, WEED_LEAF_HOST_SCRAP_FILE_OFFSET, weed_get_int64_value(event, WEED_LEAF_HOST_SCRAP_FILE_OFFSET, &error));
+          }
           if (add_audio) {
             weed_set_int_array(shortcut, WEED_LEAF_AUDIO_CLIPS, num_aclips, aclips);
             weed_set_double_array(shortcut, WEED_LEAF_AUDIO_SEEKS, num_aclips, aseeks);
@@ -645,6 +647,10 @@ weed_plant_t *quantise_events(weed_plant_t *in_list, double qfps, boolean allow_
         return NULL;
       }
       if (get_first_event(out_list) == NULL) weed_set_voidptr_value(out_list, WEED_LEAF_FIRST, get_last_event(out_list));
+      if (weed_plant_has_leaf(event, WEED_LEAF_HOST_SCRAP_FILE_OFFSET)) {
+        weed_set_int64_value(get_first_frame_event(out_list), WEED_LEAF_HOST_SCRAP_FILE_OFFSET, weed_get_int64_value(event,
+                             WEED_LEAF_HOST_SCRAP_FILE_OFFSET, &error));
+      }
     }
   }
 
@@ -836,6 +842,7 @@ void on_resaudio_ok_clicked(LiVESButton *button, LiVESEntry *entry) {
   if (cfile->audio_waveform != NULL) {
     for (i = 0; i < cfile->achans; lives_freep((void **)&cfile->audio_waveform[i++]));
     lives_freep((void **)&cfile->audio_waveform);
+    lives_freep((void **)&cfile->aw_sizes);
   }
 
   cfile->arate = arate;
@@ -892,7 +899,7 @@ void on_resaudio_ok_clicked(LiVESButton *button, LiVESEntry *entry) {
 
 static void on_resaudw_achans_changed(LiVESWidget *widg, livespointer user_data) {
   _resaudw *resaudw = (_resaudw *)user_data;
-  char *tmp;
+  //char *tmp;
 
   if (!lives_toggle_button_get_active(LIVES_TOGGLE_BUTTON(widg))) {
     lives_widget_set_sensitive(resaudw->rb_signed, FALSE);
@@ -905,11 +912,7 @@ static void on_resaudw_achans_changed(LiVESWidget *widg, livespointer user_data)
     if (prefsw != NULL) {
       lives_widget_set_sensitive(prefsw->pertrack_checkbutton, FALSE);
       lives_widget_set_sensitive(prefsw->backaudio_checkbutton, FALSE);
-      lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(prefsw->pertrack_checkbutton), FALSE);
-      lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(prefsw->backaudio_checkbutton), FALSE);
     } else if (rdet != NULL) {
-      lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(rdet->pertrack_checkbutton), FALSE);
-      lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(rdet->backaudio_checkbutton), FALSE);
       lives_widget_set_sensitive(rdet->pertrack_checkbutton, FALSE);
       lives_widget_set_sensitive(rdet->backaudio_checkbutton, FALSE);
     }
@@ -929,11 +932,6 @@ static void on_resaudw_achans_changed(LiVESWidget *widg, livespointer user_data)
       lives_widget_set_sensitive(rdet->backaudio_checkbutton, TRUE);
       lives_widget_set_sensitive(rdet->pertrack_checkbutton, TRUE);
     }
-
-    tmp = lives_strdup_printf("%d", DEFAULT_AUDIO_CHANS);
-    lives_entry_set_text(LIVES_ENTRY(resaudw->entry_achans), tmp);
-    lives_free(tmp);
-
   }
 }
 
@@ -1096,14 +1094,14 @@ void on_resample_vid_ok(LiVESButton *button, LiVESEntry *entry) {
 _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox) {
   // type 1 == resample
   // type 2 == insert silence
-  // type 3 == enter multitrack or encode or render to clip
+  // type 3 == enter multitrack
   // type 4 == prefs/multitrack
   // type 5 == new clip record/record to selection with no existing audio
   // type 6 == record to clip with no existing audio
   // type 7 == record to clip with existing audio (show time only)
   // type 8 == grab external window, with audio
   // type 9 == grab external, no audio
-  // type 10 == change inside multitrack
+  // type 10 == change inside multitrack / render to clip (embedded) [resets to 3]
   // type 11 == rte audio gen as rfx
 
   LiVESWidget *dialog_vbox = NULL;
@@ -1155,7 +1153,7 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
   _resaudw *resaudw = (_resaudw *)(lives_malloc(sizeof(_resaudw)));
 
   if (type == 10) {
-    if (mainw->multitrack != NULL) chans_fixed = TRUE; // TODO *
+    chans_fixed = TRUE;
     type = 3;
   }
 
@@ -1202,10 +1200,6 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     accel_group = LIVES_ACCEL_GROUP(lives_accel_group_new());
     lives_window_add_accel_group(LIVES_WINDOW(resaudw->dialog), accel_group);
 
-    if (prefs->show_gui) {
-      lives_window_set_transient_for(LIVES_WINDOW(resaudw->dialog), LIVES_WINDOW(mainw->LiVES));
-    }
-
     dialog_vbox = lives_dialog_get_content_area(LIVES_DIALOG(resaudw->dialog));
 
     vboxx = lives_vbox_new(FALSE, 0);
@@ -1309,20 +1303,25 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     lives_free(tmp);
 
     if (type == 4) lives_box_pack_start(LIVES_BOX(vboxx), frame, FALSE, FALSE, widget_opts.packing_height);
-    else lives_box_pack_start(LIVES_BOX(vboxx), frame, TRUE, TRUE, 0);
+    else lives_box_pack_start(LIVES_BOX(vboxx), frame, FALSE, TRUE, 0);
 
     vbox2 = lives_vbox_new(FALSE, 0);
     lives_container_add(LIVES_CONTAINER(frame), vbox2);
 
-    if (type > 2 && type < 5) {
+    if (type > 2 && type < 5 && !chans_fixed) {
       resaudw->aud_hbox = lives_hbox_new(FALSE, 0);
       lives_box_pack_start(LIVES_BOX(vbox2), resaudw->aud_hbox, FALSE, FALSE, 0);
 
       resaudw->aud_checkbutton = lives_standard_check_button_new(_("_Enable audio"), FALSE, LIVES_BOX(resaudw->aud_hbox), NULL);
 
       if (rdet != NULL) lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(resaudw->aud_checkbutton), rdet->achans > 0);
-      else lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(resaudw->aud_checkbutton), prefs->mt_def_achans > 0);
-    } else resaudw->aud_checkbutton = lives_check_button_new();
+      else lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(resaudw->aud_checkbutton),
+                                            mainw->multitrack == NULL ? prefs->mt_def_achans > 0 : cfile->achans > 0);
+      if (type == 4) {
+        lives_signal_connect(LIVES_GUI_OBJECT(resaudw->aud_checkbutton), LIVES_WIDGET_TOGGLED_SIGNAL,
+                             LIVES_GUI_CALLBACK(apply_button_set_enabled), NULL);
+      }
+    }
 
     hbox2 = lives_hbox_new(FALSE, 0);
     lives_box_pack_start(LIVES_BOX(vbox2), hbox2, FALSE, FALSE, widget_opts.packing_height);
@@ -1345,10 +1344,15 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     if (type < 3 || (type > 4 && type < 8) || type == 11) tmp = lives_strdup_printf("%d", (int)mainw->fx1_val);
     else if (type == 8) tmp = lives_strdup_printf("%d", DEFAULT_AUDIO_RATE);
     else if (type == 3) tmp = lives_strdup_printf("%d", rdet->arate);
-    else tmp = lives_strdup_printf("%d", prefs->mt_def_arate);
+    else tmp = mainw->multitrack == NULL ||
+                 cfile->achans == 0 ? lives_strdup_printf("%d", prefs->mt_def_arate) : lives_strdup_printf("%d", cfile->arate);
     lives_entry_set_text(LIVES_ENTRY(resaudw->entry_arate), tmp);
     lives_free(tmp);
 
+    if (type == 4) {
+      lives_signal_connect(LIVES_GUI_OBJECT(combo4), LIVES_WIDGET_CHANGED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled), NULL);
+    }
+
     combo5 = lives_standard_combo_new((type >= 3 && type != 11 ? (_("_Channels")) : (_("Channels"))),
                                       channels, LIVES_BOX(hbox), NULL);
 
@@ -1360,7 +1364,8 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     if (type < 3 || (type > 4 && type < 8) || type == 11) tmp = lives_strdup_printf("%d", (int)mainw->fx2_val);
     else if (type == 8) tmp = lives_strdup_printf("%d", DEFAULT_AUDIO_CHANS);
     else if (type == 3) tmp = lives_strdup_printf("%d", rdet->achans);
-    else tmp = lives_strdup_printf("%d", prefs->mt_def_achans == 0 ? DEFAULT_AUDIO_CHANS : prefs->mt_def_achans);
+    else tmp = lives_strdup_printf("%d", mainw->multitrack == NULL ||
+                                     cfile->achans == 0 ? (prefs->mt_def_achans == 0 ? DEFAULT_AUDIO_CHANS : prefs->mt_def_achans) : cfile->achans);
     lives_entry_set_text(LIVES_ENTRY(resaudw->entry_achans), tmp);
     lives_free(tmp);
 
@@ -1369,6 +1374,10 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
       lives_widget_set_sensitive(combo5, FALSE);
     }
 
+    if (type == 4) {
+      lives_signal_connect(LIVES_GUI_OBJECT(combo5), LIVES_WIDGET_CHANGED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled), NULL);
+    }
+
     combo6 = lives_standard_combo_new((type >= 3 && type != 11 ? (_("_Sample Size")) : (_("Sample Size"))),
                                       sampsize, LIVES_BOX(hbox), NULL);
 
@@ -1382,7 +1391,7 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     if (type < 3 || (type > 4 && type < 8) || type == 11) tmp = lives_strdup_printf("%d", (int)mainw->fx3_val);
     else if (type == 8) tmp = lives_strdup_printf("%d", DEFAULT_AUDIO_SAMPS);
     else if (type == 3) tmp = lives_strdup_printf("%d", rdet->asamps);
-    else tmp = lives_strdup_printf("%d", prefs->mt_def_asamps);
+    else tmp = lives_strdup_printf("%d", mainw->multitrack == NULL || cfile->achans == 0 ? prefs->mt_def_asamps : cfile->asampsize);
     lives_entry_set_text(LIVES_ENTRY(resaudw->entry_asamps), tmp);
 
     if (!strcmp(tmp, "8")) is_8bit = TRUE;
@@ -1390,6 +1399,10 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
 
     lives_free(tmp);
 
+    if (type == 4) {
+      lives_signal_connect(LIVES_GUI_OBJECT(combo6), LIVES_WIDGET_CHANGED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled), NULL);
+    }
+
     vseparator = lives_vseparator_new();
     if (type != 4) lives_box_pack_start(LIVES_BOX(hbox2), vseparator, FALSE, FALSE, widget_opts.packing_width);
 
@@ -1414,7 +1427,7 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     if (type < 3 || (type > 4 && type < 8) || type == 11) aendian = mainw->fx4_val;
     else if (type == 8) aendian = DEFAULT_AUDIO_SIGNED16 | ((capable->byte_order == LIVES_BIG_ENDIAN) ? AFORM_BIG_ENDIAN : 0);
     else if (type == 3) aendian = rdet->aendian;
-    else aendian = prefs->mt_def_signed_endian;
+    else aendian = mainw->multitrack == NULL || cfile->achans == 0 ? prefs->mt_def_signed_endian : cfile->signed_endian;
 
     if (aendian & AFORM_UNSIGNED) {
       lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(resaudw->rb_unsigned), TRUE);
@@ -1422,6 +1435,12 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
       lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(resaudw->rb_signed), TRUE);
     }
 
+    if (type == 4) {
+      lives_signal_connect(LIVES_GUI_OBJECT(resaudw->rb_signed), LIVES_WIDGET_TOGGLED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled), NULL);
+      lives_signal_connect(LIVES_GUI_OBJECT(resaudw->rb_unsigned), LIVES_WIDGET_TOGGLED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled),
+                           NULL);
+    }
+
     vseparator = lives_vseparator_new();
     lives_box_pack_start(LIVES_BOX(hbox2), vseparator, FALSE, FALSE, widget_opts.packing_width);
 
@@ -1458,6 +1477,12 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
                          NULL);
   }
 
+  if (type == 4) {
+    lives_signal_connect(LIVES_GUI_OBJECT(resaudw->rb_littleend), LIVES_WIDGET_TOGGLED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled),
+                         NULL);
+    lives_signal_connect(LIVES_GUI_OBJECT(resaudw->rb_bigend), LIVES_WIDGET_TOGGLED_SIGNAL, LIVES_GUI_CALLBACK(apply_button_set_enabled), NULL);
+  }
+
   if (type > 7 && type != 11) {
     frame = lives_standard_frame_new(_("Video"), 0., FALSE);
     lives_box_pack_start(LIVES_BOX(vboxx), frame, TRUE, TRUE, 0);
@@ -1482,9 +1507,9 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
       resaudw->hour_spinbutton = lives_standard_spin_button_new(_(" hours  "), hours,
                                  0., hours > 23 ? hours : 23, 1., 1., 0, LIVES_BOX(hbox), NULL);
 
-      resaudw->minute_spinbutton = lives_standard_spin_button_new(_(" minutes  "), mins, 0., 59., 1., 1., 0, LIVES_BOX(hbox), NULL);
+      resaudw->minute_spinbutton = lives_standard_spin_button_new(_(" minutes  "), mins, 0., 59., 1., 10., 0, LIVES_BOX(hbox), NULL);
 
-      resaudw->second_spinbutton = lives_standard_spin_button_new(_(" seconds  "), secs, 0., 59., 1., 1., 0, LIVES_BOX(hbox), NULL);
+      resaudw->second_spinbutton = lives_standard_spin_button_new(_(" seconds  "), secs, 0., 59., 1., 10., 0, LIVES_BOX(hbox), NULL);
 
       hbox = lives_hbox_new(FALSE, 0);
       lives_box_pack_start(LIVES_BOX(dialog_vbox), hbox, TRUE, TRUE, widget_opts.packing_height);
@@ -1510,19 +1535,17 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
   }
 
   if (type < 3 || type > 4) {
-    cancelbutton = lives_standard_button_new_from_stock(LIVES_STOCK_CANCEL, NULL);
-
-    lives_dialog_add_action_widget(LIVES_DIALOG(resaudw->dialog), cancelbutton, LIVES_RESPONSE_CANCEL);
-    lives_widget_set_can_focus_and_default(cancelbutton);
+    cancelbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(resaudw->dialog), LIVES_STOCK_CANCEL, NULL,
+                   LIVES_RESPONSE_CANCEL);
 
     if (accel_group != NULL) lives_widget_add_accelerator(cancelbutton, LIVES_WIDGET_CLICKED_SIGNAL, accel_group,
           LIVES_KEY_Escape, (LiVESXModifierType)0, (LiVESAccelFlags)0);
 
-    okbutton = lives_standard_button_new_from_stock(LIVES_STOCK_OK, NULL);
 
-    lives_dialog_add_action_widget(LIVES_DIALOG(resaudw->dialog), okbutton, LIVES_RESPONSE_OK);
-    lives_widget_set_can_focus_and_default(okbutton);
-    lives_widget_grab_default_special(okbutton);
+    okbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(resaudw->dialog), LIVES_STOCK_OK, NULL,
+               LIVES_RESPONSE_OK);
+
+    lives_button_grab_default_special(okbutton);
 
     if (type < 8 || type == 11) {
       lives_signal_connect(LIVES_GUI_OBJECT(cancelbutton), LIVES_WIDGET_CLICKED_SIGNAL,
@@ -1550,9 +1573,8 @@ _resaudw *create_resaudw(short type, render_details *rdet, LiVESWidget *top_vbox
     }
 
     lives_widget_show_all(resaudw->dialog);
-
   } else {
-    if (type > 2 && type < 5) {
+    if (resaudw->aud_checkbutton != NULL) {
       lives_signal_connect_after(LIVES_GUI_OBJECT(resaudw->aud_checkbutton), LIVES_WIDGET_TOGGLED_SIGNAL,
                                  LIVES_GUI_CALLBACK(on_resaudw_achans_changed),
                                  (livespointer)resaudw);
@@ -1610,23 +1632,19 @@ void create_new_pb_speed(short type) {
   accel_group = LIVES_ACCEL_GROUP(lives_accel_group_new());
   lives_window_add_accel_group(LIVES_WINDOW(new_pb_speed), accel_group);
 
-  if (prefs->show_gui) {
-    lives_window_set_transient_for(LIVES_WINDOW(new_pb_speed), LIVES_WINDOW(mainw->LiVES));
-  }
-
   dialog_vbox = lives_dialog_get_content_area(LIVES_DIALOG(new_pb_speed));
 
   vbox = lives_vbox_new(FALSE, 0);
 
-  lives_box_pack_start(LIVES_BOX(dialog_vbox), vbox, TRUE, TRUE, widget_opts.packing_height * 2);
+  lives_box_pack_start(LIVES_BOX(dialog_vbox), vbox, TRUE, TRUE, 0);
 
   if (type == 1) {
     lives_snprintf(label_text, 256,
-                   _("\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the desired playback speed\nin _frames per second"),
+                   _("Current playback speed is %.3f frames per second.\n\nPlease enter the desired playback speed\nin _frames per second"),
                    cfile->fps);
   } else if (type == 2) {
     lives_snprintf(label_text, 256,
-                   _("\n\nCurrent playback speed is %.3f frames per second.\n\nPlease enter the _resampled rate\nin frames per second"),
+                   _("Current playback speed is %.3f frames per second.\n\nPlease enter the _resampled rate\nin frames per second"),
                    cfile->fps);
   }
 
@@ -1645,8 +1663,15 @@ void create_new_pb_speed(short type) {
 
     spinbutton_pb_speed = lives_standard_spin_button_new(NULL, cfile->fps, 1., FPS_MAX, .01, .1, 3, LIVES_BOX(hbox), NULL);
 
+    hbox = lives_hbox_new(FALSE, 0);
+    lives_box_pack_start(LIVES_BOX(vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
+
+    add_fill_to_box(LIVES_BOX(hbox));
+
     label2 = lives_standard_label_new_with_mnemonic_widget(_("OR enter the desired clip length in _seconds"), NULL);
-    lives_box_pack_start(LIVES_BOX(vbox), label2, TRUE, TRUE, widget_opts.packing_height);
+    lives_box_pack_start(LIVES_BOX(hbox), label2, TRUE, TRUE, widget_opts.packing_width);
+
+    add_fill_to_box(LIVES_BOX(hbox));
 
     hbox = lives_hbox_new(FALSE, 0);
     lives_box_pack_start(LIVES_BOX(vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
@@ -1662,23 +1687,24 @@ void create_new_pb_speed(short type) {
 
   lives_label_set_mnemonic_widget(LIVES_LABEL(label), spinbutton_pb_speed);
 
+  add_fill_to_box(LIVES_BOX(vbox));
+
   ca_hbox = lives_hbox_new(FALSE, 0);
   change_audio_speed = lives_standard_check_button_new
                        (_("Change the _audio speed as well"), FALSE, LIVES_BOX(ca_hbox), NULL);
 
   lives_box_pack_start(LIVES_BOX(vbox), ca_hbox, TRUE, TRUE, widget_opts.packing_height);
 
-  cancelbutton = lives_standard_button_new_from_stock(LIVES_STOCK_CANCEL, NULL);
-  lives_dialog_add_action_widget(LIVES_DIALOG(new_pb_speed), cancelbutton, LIVES_RESPONSE_CANCEL);
-  lives_widget_set_can_focus(cancelbutton, TRUE);
+  cancelbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(new_pb_speed), LIVES_STOCK_CANCEL, NULL,
+                 LIVES_RESPONSE_CANCEL);
 
   lives_widget_add_accelerator(cancelbutton, LIVES_WIDGET_CLICKED_SIGNAL, accel_group,
                                LIVES_KEY_Escape, (LiVESXModifierType)0, (LiVESAccelFlags)0);
 
-  change_pb_ok = lives_standard_button_new_from_stock(LIVES_STOCK_OK, NULL);
-  lives_dialog_add_action_widget(LIVES_DIALOG(new_pb_speed), change_pb_ok, LIVES_RESPONSE_OK);
-  lives_widget_set_can_focus_and_default(change_pb_ok);
-  lives_widget_grab_default_special(change_pb_ok);
+  change_pb_ok = lives_dialog_add_button_from_stock(LIVES_DIALOG(new_pb_speed), LIVES_STOCK_OK, NULL,
+                 LIVES_RESPONSE_OK);
+
+  lives_button_grab_default_special(change_pb_ok);
   lives_widget_grab_focus(spinbutton_pb_speed);
 
   reorder_leave_back = FALSE;
@@ -1842,7 +1868,12 @@ void on_change_speed_ok_clicked(LiVESButton *button, livespointer user_data) {
   cfile->undo_action = UNDO_CHANGE_SPEED;
 
   if (mainw->fx1_val == 0.) mainw->fx1_val = 1.;
+
+  // update the frame rate
   cfile->pb_fps = cfile->fps = mainw->fx1_val;
+
+  lives_spin_button_set_value(LIVES_SPIN_BUTTON(mainw->spinbutton_pb_fps), cfile->pb_fps);
+
   if (mainw->fx1_bool) {
     cfile->arate = (int)(arate * cfile->fps + .5);
     msg = lives_strdup_printf(_("Changed playback speed to %.3f frames per second and audio to %d Hz.\n"), cfile->fps, cfile->arate);
@@ -2022,7 +2053,7 @@ int deorder_frames(int old_frames, boolean leave_bak) {
 boolean resample_clipboard(double new_fps) {
   // resample the clipboard video - if we already did it once, it is
   // quicker the second time
-  char *msg, *com;
+  char *com;
   int current_file = mainw->current_file;
 
   mainw->no_switch_dprint = TRUE;
@@ -2063,10 +2094,9 @@ boolean resample_clipboard(double new_fps) {
     cfile->frames = new_frames;
     cfile->undo_action = UNDO_RESAMPLE;
     cfile->fps = cfile->undo1_dbl;
+    lives_spin_button_set_value(LIVES_SPIN_BUTTON(mainw->spinbutton_pb_fps), cfile->fps);
     cfile->undo1_dbl = old_fps;
-    msg = lives_strdup_printf(_("Clipboard was resampled to %d frames.\n"), cfile->frames);
-    d_print(msg);
-    lives_free(msg);
+    d_print(_("Clipboard was resampled to %d frames.\n"), cfile->frames);
     mainw->current_file = current_file;
   } else {
     if (clipboard->undo1_dbl < clipboard->fps) {
