@@ -1,6 +1,6 @@
 // startup.c
 // LiVES
-// (c) G. Finch 2010 - 2018 <salsaman+lives@gmail.com>
+// (c) G. Finch 2010 - 2019 <salsaman+lives@gmail.com>
 // released under the GNU GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -9,59 +9,51 @@
 #include "main.h"
 #include "interface.h"
 
+static boolean allpassed;
 
-static boolean prompt_existing_dir(char *dirname, uint64_t freespace, boolean wrtable) {
-  char *msg;
-  boolean res = FALSE;
-
+static boolean prompt_existing_dir(const char *dirname, uint64_t freespace, boolean wrtable) {
+  boolean res;
   if (wrtable) {
     char *fspstr = lives_format_storage_space_string(freespace);
-    msg = lives_strdup_printf
-          (_("A directory named\n%s\nalready exists. Do you wish to use this directory ?\n\n(Free space = %s)\n"), dirname, fspstr);
+    if (dirs_equal(dirname, capable->home_dir)) {
+      if (!do_yesno_dialog(
+            _("You have chosen to use your home directory as the LiVES working directory.\nThis is NOT recommended as it will likely result in the loss of unrelated files.\nClick Yes if you REALLY want to continue, or No to create or select another directory.\n")))
+        return FALSE;
+    }
+
+    res = do_yesno_dialogf(_("A directory named\n%s\nalready exists. Do you wish to use this directory ?\n\n(Free space = %s)\n"), dirname,
+                           fspstr);
     lives_free(fspstr);
-    res = do_yesno_dialog(msg);
   } else {
-    msg = lives_strdup_printf
-          (_("A directory named\n%s\nalready exists.\nLiVES could not write to this directory or read its free space.\nPlease select another location.\n"),
-           dirname);
-    do_error_dialog(msg);
+    res = do_error_dialogf(
+            _("A directory named\n%s\nalready exists.\nLiVES could not write to this directory or read its free space.\nPlease select another location.\n"),
+            dirname);
   }
-  lives_free(msg);
   return res;
 }
 
 
 static boolean prompt_new_dir(char *dirname, uint64_t freespace, boolean wrtable) {
-  boolean res = FALSE;
-  char *msg;
+  boolean res;
   if (wrtable) {
     char *fspstr = lives_format_storage_space_string(freespace);
-    msg = lives_strdup_printf(_("\nCreate the directory\n%s\n?\n\n(Free space = %s)"), dirname, fspstr);
+    res = do_warning_dialogf(_("\nCreate the directory\n%s\n?\n\n(Free space = %s)"), dirname, fspstr);
     lives_free(fspstr);
-    res = do_warning_dialog(msg);
   } else {
-    msg = lives_strdup_printf(_("\nLiVES could not write to the directory\n%s\nPlease try again and choose a different location.\n"), dirname);
-    do_error_dialog(msg);
+    res = do_error_dialogf(_("\nLiVES could not write to the directory\n%s\nPlease try again and choose a different location.\n"), dirname);
   }
-
-  lives_free(msg);
   return res;
 }
 
 
 void close_file(int current_file, boolean tshoot) {
-  char *com;
-
   if (tshoot) close_current_file(current_file);
   else {
 #ifdef IS_MINGW
     // kill any active processes: for other OSes the backend does this
     lives_kill_subprocesses(cfile->handle, TRUE);
 #endif
-
-    com = lives_strdup_printf("%s close \"%s\"", prefs->backend_sync, cfile->handle);
-    lives_system(com, TRUE);
-    lives_free(com);
+    mainw->current_file = close_temp_handle(mainw->current_file, current_file);
   }
 }
 
@@ -70,15 +62,12 @@ boolean do_workdir_query(void) {
   uint64_t freesp;
 
   int response;
-  boolean ok = FALSE;
 
   char *dirname;
 
-top:
-
   renamew = create_rename_dialog(6);
 
-  while (!ok) {
+  while (1) {
     response = lives_dialog_run(LIVES_DIALOG(renamew->dialog));
 
     if (response == LIVES_RESPONSE_CANCEL) return FALSE;
@@ -132,18 +121,18 @@ top:
         }
       }
     }
-    ok = TRUE;
+    if (!lives_make_writeable_dir(dirname)) {
+      do_blocking_error_dialogf(
+        _("LiVES could not write to the directory\n%s\nPlease check permissions for the parent directory,\nand try again.\n"), dirname);
+      lives_free(dirname);
+      continue;
+    }
+    break;
   }
 
   lives_widget_destroy(renamew->dialog);
   lives_freep((void **)&renamew);
 
-  mainw->com_failed = FALSE;
-
-  if (!lives_file_test(dirname, LIVES_FILE_TEST_IS_DIR)) {
-    if (lives_mkdir_with_parents(dirname, capable->umask) == -1) goto top;
-  }
-
   lives_snprintf(prefs->workdir, PATH_MAX, "%s", dirname);
   lives_snprintf(future_prefs->workdir, PATH_MAX, "%s", prefs->workdir);
 
@@ -153,6 +142,7 @@ top:
   lives_snprintf(mainw->first_info_file, PATH_MAX, "%s"LIVES_DIR_SEP LIVES_INFO_FILE_NAME".%d", prefs->workdir, capable->mainpid);
 
   lives_free(dirname);
+
   return TRUE;
 }
 
@@ -174,18 +164,12 @@ static void on_init_aplayer_toggled(LiVESToggleButton *tbutton, livespointer use
   case AUD_PLAYER_SOX:
     set_pref(PREF_AUDIO_PLAYER, AUDIO_PLAYER_SOX);
     break;
-  case AUD_PLAYER_MPLAYER:
-    set_pref(PREF_AUDIO_PLAYER, AUDIO_PLAYER_MPLAYER);
-    break;
-  case AUD_PLAYER_MPLAYER2:
-    set_pref(PREF_AUDIO_PLAYER, AUDIO_PLAYER_MPLAYER2);
-    break;
   }
 }
 
 
 boolean do_audio_choice_dialog(short startup_phase) {
-  LiVESWidget *dialog, *dialog_vbox, *radiobutton2, *radiobutton3, *radiobutton4, *label;
+  LiVESWidget *dialog, *dialog_vbox, *radiobutton2, *label;
   LiVESWidget *okbutton, *cancelbutton;
   LiVESWidget *hbox;
 
@@ -201,7 +185,7 @@ boolean do_audio_choice_dialog(short startup_phase) {
 
   LiVESSList *radiobutton_group = NULL;
 
-  char *txt0, *txt1, *txt2, *txt3, *txt4, *txt5, *txt6, *txt7, *msg;
+  char *txt0, *txt1, *txt2, *txt3, *txt4, *txt5, *txt6, *msg;
 
   int response;
 
@@ -219,7 +203,7 @@ boolean do_audio_choice_dialog(short startup_phase) {
 #else
   if (!capable->has_pulse_audio) {
     txt2 = lives_strdup(
-             _(", but you do not have pulse audio installed on your system.\n You are advised to install pulse audio first before running LiVES.\n\n"));
+             _(", but you do not have pulseaudio installed on your system.\n You are advised to install pulseaudio first before running LiVES.\n\n"));
   } else txt2 = lives_strdup(".\n\n");
 #endif
 
@@ -240,18 +224,12 @@ boolean do_audio_choice_dialog(short startup_phase) {
   txt5 = lives_strdup(_("SOX may be used if neither of the preceding players work, "));
 
   if (capable->has_sox_play) {
-    txt6 = lives_strdup(_("but some audio features will be disabled.\n\n"));
+    txt6 = lives_strdup(_("but many audio features will be disabled.\n\n"));
   } else {
     txt6 = lives_strdup(_("but you do not have sox installed.\nYou are advised to install it before running LiVES.\n\n"));
   }
 
-  if (capable->has_mplayer || capable->has_mplayer2) {
-    txt7 = lives_strdup(_("The MPLAYER/MPLAYER2 audio player is only recommended for testing purposes.\n\n"));
-  } else {
-    txt7 = lives_strdup("");
-  }
-
-  msg = lives_strdup_printf("%s%s%s%s%s%s%s%s", txt0, txt1, txt2, txt3, txt4, txt5, txt6, txt7);
+  msg = lives_strdup_printf("%s%s%s%s%s%s%s", txt0, txt1, txt2, txt3, txt4, txt5, txt6);
 
   lives_free(txt0);
   lives_free(txt1);
@@ -260,7 +238,6 @@ boolean do_audio_choice_dialog(short startup_phase) {
   lives_free(txt4);
   lives_free(txt5);
   lives_free(txt6);
-  lives_free(txt7);
 
   dialog = lives_standard_dialog_new(_("Choose an audio player"), FALSE, -1, -1);
 
@@ -269,6 +246,7 @@ boolean do_audio_choice_dialog(short startup_phase) {
 
   dialog_vbox = lives_dialog_get_content_area(LIVES_DIALOG(dialog));
 
+  // TODO: add_param_label_to_box()
   label = lives_standard_label_new(msg);
   lives_container_add(LIVES_CONTAINER(dialog_vbox), label);
   lives_free(msg);
@@ -277,7 +255,7 @@ boolean do_audio_choice_dialog(short startup_phase) {
   hbox = lives_hbox_new(FALSE, 0);
   lives_box_pack_start(LIVES_BOX(dialog_vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
 
-  radiobutton0 = lives_standard_radio_button_new(_("Use _pulse audio player"), &radiobutton_group, LIVES_BOX(hbox), NULL);
+  radiobutton0 = lives_standard_radio_button_new(_("Use _pulseaudio player"), &radiobutton_group, LIVES_BOX(hbox), NULL);
 
   if (prefs->audio_player == -1) prefs->audio_player = AUD_PLAYER_PULSE;
 
@@ -328,54 +306,16 @@ boolean do_audio_choice_dialog(short startup_phase) {
                          LIVES_INT_TO_POINTER(AUD_PLAYER_SOX));
   }
 
-  if (capable->has_mplayer) {
-    hbox = lives_hbox_new(FALSE, 0);
-    lives_box_pack_start(LIVES_BOX(dialog_vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
-
-    radiobutton3 = lives_standard_radio_button_new(_("Use _mplayer audio player"), &radiobutton_group, LIVES_BOX(hbox), NULL);
-
-    if (prefs->audio_player == -1) prefs->audio_player = AUD_PLAYER_MPLAYER;
-
-    if (prefs->audio_player == AUD_PLAYER_MPLAYER) {
-      lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(radiobutton3), TRUE);
-      set_pref(PREF_AUDIO_PLAYER, AUDIO_PLAYER_MPLAYER);
-    }
-
-    lives_signal_connect(LIVES_GUI_OBJECT(radiobutton3), LIVES_WIDGET_TOGGLED_SIGNAL,
-                         LIVES_GUI_CALLBACK(on_init_aplayer_toggled),
-                         LIVES_INT_TO_POINTER(AUD_PLAYER_MPLAYER));
-  }
-
-  if (capable->has_mplayer2) {
-    hbox = lives_hbox_new(FALSE, 0);
-    lives_box_pack_start(LIVES_BOX(dialog_vbox), hbox, FALSE, FALSE, widget_opts.packing_height);
-
-    radiobutton4 = lives_standard_radio_button_new(_("Use _mplayer2 audio player"), &radiobutton_group, LIVES_BOX(hbox), NULL);
-
-    if (prefs->audio_player == -1) prefs->audio_player = AUD_PLAYER_MPLAYER2;
-
-    if (prefs->audio_player == AUD_PLAYER_MPLAYER2) {
-      lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(radiobutton4), TRUE);
-      set_pref(PREF_AUDIO_PLAYER, AUDIO_PLAYER_MPLAYER2);
-    }
-
-    lives_signal_connect(LIVES_GUI_OBJECT(radiobutton4), LIVES_WIDGET_TOGGLED_SIGNAL,
-                         LIVES_GUI_CALLBACK(on_init_aplayer_toggled),
-                         LIVES_INT_TO_POINTER(AUD_PLAYER_MPLAYER2));
-  }
-
-  cancelbutton = lives_standard_button_new_from_stock(LIVES_STOCK_CANCEL, NULL);
-
-  lives_dialog_add_action_widget(LIVES_DIALOG(dialog), cancelbutton, LIVES_RESPONSE_CANCEL);
+  cancelbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_CANCEL, NULL,
+                 LIVES_RESPONSE_CANCEL);
 
   lives_widget_add_accelerator(cancelbutton, LIVES_WIDGET_CLICKED_SIGNAL, accel_group,
                                LIVES_KEY_Escape, (LiVESXModifierType)0, (LiVESAccelFlags)0);
 
-  okbutton = lives_standard_button_new_from_stock(LIVES_STOCK_GO_FORWARD, _("_Next"));
+  okbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_GO_FORWARD, _("_Next"),
+             LIVES_RESPONSE_OK);
 
-  lives_dialog_add_action_widget(LIVES_DIALOG(dialog), okbutton, LIVES_RESPONSE_OK);
-  lives_widget_set_can_focus_and_default(okbutton);
-  lives_widget_grab_default_special(okbutton);
+  lives_button_grab_default_special(okbutton);
   lives_widget_grab_focus(okbutton);
 
   if (prefs->audio_player == -1) {
@@ -395,7 +335,7 @@ boolean do_audio_choice_dialog(short startup_phase) {
     lives_widget_hide(mainw->recaudio_submenu);
   }
 
-  lives_widget_context_update();
+  lives_widget_process_updates(mainw->LiVES, TRUE);
 
   return (response == LIVES_RESPONSE_OK);
 }
@@ -419,7 +359,7 @@ static void add_test(LiVESWidget *table, int row, char *ttext, boolean noskip) {
     lives_widget_show(image);
   }
 
-  lives_widget_context_update();
+  lives_widget_process_updates(mainw->LiVES, TRUE);
 }
 
 
@@ -439,7 +379,7 @@ static boolean pass_test(LiVESWidget *table, int row) {
   lives_table_attach(LIVES_TABLE(table), image, 2, 3, row, row + 1, (LiVESAttachOptions)0, (LiVESAttachOptions)0, 0, 10);
   lives_widget_show(image);
 
-  lives_widget_context_update();
+  lives_widget_process_updates(mainw->LiVES, TRUE);
   return TRUE;
 }
 
@@ -466,13 +406,13 @@ static boolean fail_test(LiVESWidget *table, int row, char *ftext) {
   lives_table_attach(LIVES_TABLE(table), image, 2, 3, row, row + 1, (LiVESAttachOptions)0, (LiVESAttachOptions)0, 0, 10);
   lives_widget_show(image);
 
-  lives_widget_context_update();
-
+  lives_widget_process_updates(mainw->LiVES, TRUE);
+  allpassed = FALSE;
   return FALSE;
 }
 
 
-LIVES_INLINE char *get_resource(char *fname) {
+LIVES_LOCAL_INLINE char *get_resource(char *fname) {
   return lives_build_filename(prefs->prefix_dir, DATA_DIR, "resources", fname, NULL);
 }
 
@@ -509,6 +449,8 @@ boolean do_startup_tests(boolean tshoot) {
 
   int out_fd, info_fd;
 
+  allpassed = TRUE;
+
   mainw->suppress_dprint = TRUE;
   mainw->cancelled = CANCEL_NONE;
 
@@ -538,20 +480,19 @@ boolean do_startup_tests(boolean tshoot) {
   label = lives_standard_label_new(_("LiVES will now run some basic configuration tests\n"));
   lives_container_add(LIVES_CONTAINER(dialog_vbox), label);
 
-  cancelbutton = lives_standard_button_new_from_stock(LIVES_STOCK_CANCEL, NULL);
-
-  lives_dialog_add_action_widget(LIVES_DIALOG(dialog), cancelbutton, LIVES_RESPONSE_CANCEL);
+  cancelbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_CANCEL, NULL,
+                 LIVES_RESPONSE_CANCEL);
 
   lives_widget_add_accelerator(cancelbutton, LIVES_WIDGET_CLICKED_SIGNAL, accel_group,
                                LIVES_KEY_Escape, (LiVESXModifierType)0, (LiVESAccelFlags)0);
 
   if (!tshoot) {
-    okbutton = lives_standard_button_new_from_stock(LIVES_STOCK_GO_FORWARD, _("_Next"));
-  } else okbutton = lives_standard_button_new_from_stock(LIVES_STOCK_OK, NULL);
+    okbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_GO_FORWARD, _("_Next"),
+               LIVES_RESPONSE_OK);
+  } else okbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_OK, NULL,
+                      LIVES_RESPONSE_OK);
 
-  lives_dialog_add_action_widget(LIVES_DIALOG(dialog), okbutton, LIVES_RESPONSE_OK);
-  lives_widget_set_can_focus_and_default(okbutton);
-  lives_widget_grab_default_special(okbutton);
+  lives_button_grab_default_special(okbutton);
   lives_widget_grab_focus(okbutton);
 
   lives_widget_set_sensitive(okbutton, FALSE);
@@ -569,7 +510,7 @@ boolean do_startup_tests(boolean tshoot) {
 
   if (!capable->has_sox_sox) {
     success = fail_test(table, 0, _("You should install sox to be able to use all the audio features in LiVES"));
-
+    lives_widget_grab_focus(cancelbutton);
   } else {
     success = pass_test(table, 0);
   }
@@ -620,7 +561,7 @@ boolean do_startup_tests(boolean tshoot) {
       afile = lives_build_filename(prefs->workdir, cfile->handle, "testout.wav", NULL);
 
       mainw->com_failed = FALSE;
-      com = lives_strdup_printf("%s export_audio \"%s\" 0. 0. 44100 2 16 0 22050 \"%s\"", prefs->backend_sync, cfile->handle, afile);
+      com = lives_strdup_printf("%s export_audio \"%s\" 0. 0. 44100 2 16 1 22050 \"%s\"", prefs->backend_sync, cfile->handle, afile);
       lives_system(com, TRUE);
       if (mainw->com_failed) {
         tmp = lives_strdup_printf(_("Command failed: %s"), com);
@@ -638,7 +579,7 @@ boolean do_startup_tests(boolean tshoot) {
       if (info_fd != -1) {
         close(info_fd);
 
-        lives_sync();
+        lives_sync(1);
 
         fsize = sget_file_size(afile);
         lives_rm(afile);
@@ -692,7 +633,8 @@ boolean do_startup_tests(boolean tshoot) {
   if (!success2) {
     if (!success) {
       lives_widget_destroy(dialog);
-      lives_widget_context_update();
+      lives_widget_process_updates(mainw->LiVES, TRUE);
+
       do_no_mplayer_sox_error();
       close_file(current_file, tshoot);
       mainw->suppress_dprint = FALSE;
@@ -800,7 +742,7 @@ boolean do_startup_tests(boolean tshoot) {
     if (info_fd != -1) {
       close(info_fd);
 
-      lives_sync();
+      lives_sync(1);
 
       cfile->img_type = IMG_TYPE_PNG;
       get_frame_count(mainw->current_file);
@@ -894,6 +836,12 @@ boolean do_startup_tests(boolean tshoot) {
   mainw->current_file = current_file;
 
   lives_widget_set_sensitive(okbutton, TRUE);
+  if (!tshoot) {
+    if (allpassed)
+      lives_widget_grab_focus(okbutton);
+    else
+      lives_widget_grab_focus(cancelbutton);
+  }
 
   if (!capable->has_mplayer && !capable->has_mplayer2 && capable->has_mpv) {
     label = lives_standard_label_new(
@@ -977,11 +925,10 @@ void do_startup_interface_query(void) {
     lives_toggle_button_set_active(LIVES_TOGGLE_BUTTON(radiobutton), TRUE);
   }
 
-  okbutton = lives_standard_button_new_from_stock(LIVES_STOCK_GO_FORWARD, _("_Finish"));
+  okbutton = lives_dialog_add_button_from_stock(LIVES_DIALOG(dialog), LIVES_STOCK_GO_FORWARD, _("_Finish"),
+             LIVES_RESPONSE_OK);
 
-  lives_dialog_add_action_widget(LIVES_DIALOG(dialog), okbutton, LIVES_RESPONSE_OK);
-  lives_widget_set_can_focus_and_default(okbutton);
-  lives_widget_grab_default_special(okbutton);
+  lives_button_grab_default_special(okbutton);
   lives_widget_grab_focus(okbutton);
 
   lives_widget_show_all(dialog);
@@ -994,8 +941,7 @@ void do_startup_interface_query(void) {
   set_int_pref(PREF_STARTUP_INTERFACE, prefs->startup_interface);
 
   lives_widget_destroy(dialog);
-
-  lives_widget_context_update();
+  lives_widget_process_updates(mainw->LiVES, TRUE);
 }
 
 
