@@ -39,7 +39,7 @@ static boolean lives_wait_user_buffer(lives_vdev_t *ldev, unicap_data_buffer_t *
   int ncount;
 
   timer = timeout * TICKS_PER_SECOND;
-  stime = lives_get_current_ticks(0, 0);
+  stime = lives_get_current_ticks();
 
   while (1) {
     status = unicap_poll_buffer(ldev->handle, &ncount);
@@ -53,7 +53,7 @@ static boolean lives_wait_user_buffer(lives_vdev_t *ldev, unicap_data_buffer_t *
       return TRUE;
     }
 
-    dtime = lives_get_current_ticks(0, 0);
+    dtime = lives_get_current_ticks();
     if (dtime - stime > timer) return FALSE;
 
     lives_usleep(prefs->sleep_time);
@@ -69,10 +69,10 @@ static boolean lives_wait_system_buffer(lives_vdev_t *ldev, double timeout) {
   int64_t stime, dtime, timer;
 
   timer = timeout * TICKS_PER_SECOND;
-  stime = lives_get_current_ticks(0, 0);
+  stime = lives_get_current_ticks();
 
   while (ldev->buffer_ready == 0) {
-    dtime = lives_get_current_ticks(0, 0);
+    dtime = lives_get_current_ticks();
     if (dtime - stime > timer) return FALSE;
     lives_usleep(prefs->sleep_time);
     lives_widget_context_update();
@@ -110,7 +110,6 @@ boolean weed_layer_set_from_lvdev(weed_plant_t *layer, lives_clip_t *sfile, doub
 
   weed_set_int_value(layer, WEED_LEAF_WIDTH, sfile->hsize /
                      weed_palette_get_pixels_per_macropixel(ldev->current_palette));
-
   weed_set_int_value(layer, WEED_LEAF_HEIGHT, sfile->vsize);
   weed_set_int_value(layer, WEED_LEAF_CURRENT_PALETTE, ldev->current_palette);
   weed_set_int_value(layer, WEED_LEAF_YUV_SUBSPACE, WEED_YUV_SUBSPACE_YCBCR); // TODO - handle bt.709
@@ -462,33 +461,29 @@ boolean on_open_vdev_activate(LiVESMenuItem *menuitem, livespointer user_data) {
     status = unicap_enumerate_devices(NULL, &devices[i], i);
     if (!SUCCESS(status)) {
       if (i == 0) LIVES_INFO("Unicap failed to get any devices");
-      break;
     }
   }
 
-  for (i = 0; i < dev_count; i++) {
-    if (!unicap_is_stream_locked(&devices[i])) {
-      devlist = lives_list_prepend(devlist, devices[i].identifier);
-    }
-  }
-
-  if (devlist == NULL) {
-    do_locked_in_vdevs_error();
-    return FALSE;
-  }
-
   if (user_data == NULL) {
-    mainw->fx1_val = 0;
-    mainw->open_deint = FALSE;
-    card_dialog = create_combo_dialog(1, (livespointer)devlist);
-    response = lives_dialog_run(LIVES_DIALOG(card_dialog));
-    lives_list_free(devlist);
+    for (i = 0; i < dev_count; i++) {
+      if (!unicap_is_stream_locked(&devices[i])) {
+        devlist = lives_list_prepend(devlist, devices[i].identifier);
+      }
+    }
 
-    if (response == LIVES_RESPONSE_CANCEL) {
-      lives_widget_destroy(card_dialog);
+    if (devlist == NULL) {
+      do_locked_in_vdevs_error();
       return FALSE;
     }
 
+    mainw->fx1_val = 0;
+    mainw->open_deint = FALSE;
+    card_dialog = create_combo_dialog(1, (livespointer)devlist);
+    lives_list_free(devlist);
+    response = lives_dialog_run(LIVES_DIALOG(card_dialog));
+    if (response == LIVES_RESPONSE_CANCEL) {
+      return FALSE;
+    }
     lives_widget_destroy(card_dialog);
   } else {
     char *device = (char *)user_data;
