@@ -1,13 +1,12 @@
 #!/usr/bin/perl
 
-# (c) Salsaman (salsaman+lives@gmail.com) 2004 - 2018
+# (c) Salsaman (salsaman+lives@gmail.com) 2004 - 2019
 
 # released under the GPL 3 or later
 # see file COPYING or www.gnu.org for details
 
 #Communicate with LiVES and do auto VJing
 
-
 # syntax is autolives.pl host cmd_port status_port [options]
 # e.g. autolives.pl localhost 49999 49998
 # or just autolives.pl to use defaults 
@@ -17,6 +16,7 @@
 # -omc <notify_port>
 # -time <secs> (ignored if -omc is passed)
 # -mute
+# -loop (set continuous looping, ignored if -waitforplay is passed)
 # -debug
 
 $remote_host="localhost";
@@ -24,8 +24,19 @@ $remote_port=49999; #command port to app
 $local_port=49998; #status port from app
 $ctime = 1; # default change time
 $mute = 0;
+$loop = 0;
 $DEBUG = 0;
 
+$maxfx = 9; # e.g set to 7 if you want video gens and transitions to persist
+$allow_fxchanges = 1;
+$allow_clipswitch = 1;
+$allow_fgbgswap = 1;
+$allow_dirchange = 0; ## should be automatic, depending on whether current clip has virtual frames or not
+
+$play_dir = 0; ## assume forwards
+
+$timeout = 10;
+
 if (defined($ARGV[0])) {
     $remote_host=$ARGV[0];
 }
@@ -38,8 +49,9 @@ if (defined($ARGV[2])) {
     $local_port=$ARGV[2];
 }
 
-if ($^O eq "MSWin32") {
-    $sendOMC="perl sendOSC.pl $remote_host $remote_port";
+if ($^O eq "MSWin32" || $^O eq "msys") {
+    $is_mingw = 1;
+    $sendOMC="MSYS2_ARG_CONV_EXCL=\"*\" sendOSC -h $remote_host $remote_port >/dev/null 2>&1";
 }
 else {
     $sendOMC="sendOSC -h $remote_host $remote_port";
@@ -59,12 +71,15 @@ while (($opt = shift) ne "") {
     if ($opt eq "-waitforplay") {
 	$waitforplay = 1;
     }
+    if ($opt eq "-loop") {
+	$loop = 1;
+    }
     if ($opt eq "-debug") {
 	$DEBUG = 1;
     }
 }
 
-if ($^O eq "MSWin32") {
+if ($is_mingw) {
     use IO::Socket::INET;
 }
 else {
@@ -89,7 +104,7 @@ my $s=new IO::Select;
 
 chop($hostname = `hostname`);  
 (undef,undef,undef,undef,$myaddr) = gethostbyname($hostname);
- 
+
 @my_ip = unpack("C4", $myaddr);
 $my_ip_addr  = join(".", @my_ip);
 
@@ -97,18 +112,16 @@ if ($remote_host eq "localhost") {
     $my_ip_addr="localhost";
 }
 
-
 if ($DEBUG) {print STDERR "Opening status port UDP $local_port on $my_ip_addr...\n";}
 
-
 my $ip1=IO::Socket::INET->new(LocalPort => $local_port, Proto=>'udp',
-        LocalAddr => $my_ip_addr)
+			      LocalAddr => $my_ip_addr)
     or die "error creating UDP listener for $my_ip_addr  $@\n";
+
 $s->add($ip1);
 
 if ($DEBUG) {print STDERR "Status port ready.\n";}
 
-
 if ($noty) {
     if ($DEBUG) {print STDERR "Opening notify port UDP $local_port on $my_ip_addr...\n";}
     my $ip2=IO::Socket::INET->new(LocalPort => $notify_port, Proto=>'udp',
@@ -117,27 +130,16 @@ if ($noty) {
     $s->add($ip2);
 }
 
-
-$timeout=1;
-
-
-
 #################################################################
 # start sending OMC commands
 
 if ($DEBUG) {print STDERR "Beginning OMC handshake...\n";}
 
-if ($^O eq "MSWin32") {
-    `$sendOMC /lives/open_status_socket s $my_ip_addr i $local_port`;
-}
-else {
-    `$sendOMC /lives/open_status_socket,$my_ip_addr,$local_port`;
-}
-
+send_command("/lives/open_status_socket,$my_ip_addr,$local_port");
 
 if ($DEBUG) {print STDERR "Sent request to open status socket. Sending ping.\n";}
 
-`$sendOMC /lives/ping`;
+send_command("/lives/ping");
 my $retmsg=&get_newmsg;
 
 if ($DEBUG) {print STDERR "got $retmsg\n";}
@@ -147,39 +149,39 @@ unless ($retmsg eq "pong") {
     exit 2;
 }
 
+if ($allow_fxchanges) {
+    # get number of realtime effect keys
+    send_command("/effect_key/count");
 
-# get number of realtime effect keys
-`$sendOMC /effect_key/count`;
+    my $numeffectkeys=&get_newmsg;
 
-my $numeffectkeys=&get_newmsg;
+    if ($DEBUG) {print STDERR "LiVES has $numeffectkeys realtime keys !\n";}
 
+    if ($numeffectkeys > $maxfx) {
+	$numeffectkeys = $maxfx;
+	if ($DEBUG) {print STDERR "only messing with $numeffectkeys\n";}
+    }
+    
+    if ($DEBUG) {print STDERR "getting effect key layout...\n";}
 
-if ($DEBUG) {print STDERR "LiVES has $numeffectkeys realtime keys !\n";}
+    # get number of keymodes
 
-if ($DEBUG) {print STDERR "getting effect key layout...\n";}
+    send_command("/effect_key/maxmode/get");
 
+    $nummodes=&get_newmsg;
 
-# get number of keymodes
-
-`$sendOMC /effect_key/maxmode/get`;
-
-$nummodes=&get_newmsg;
-
-
-if ($DEBUG) {print STDERR "there are $nummodes modes per key\n";}
-
-
-if ($DEBUG) {&print_layout($numeffectkeys,$nummodes);}
-
-
-if ($DEBUG) {print STDERR "done !\n";}
+    if ($DEBUG) {
+	print STDERR "there are $nummodes modes per key\n";
+	&print_layout($numeffectkeys,$nummodes);
+	print STDERR "done !\n";
+    }
+}
 
 # get number of clips
-`$sendOMC /clip/count`;
+send_command("/clip/count");
 
 $numclips=&get_newmsg;
 
-
 if ($DEBUG) {print STDERR "LiVES has $numclips clips open !\n";}
 
 if (!$numclips) {
@@ -188,64 +190,44 @@ if (!$numclips) {
 }
 
 if ($noty) {
-    if ($^O eq "MSWin32") {
-	`$sendOMC /lives/open_notify_socket s $my_ip_addr i $notify_port`;
-    }
-    else {
-	`$sendOMC /lives/open_notify_socket,$my_ip_addr,$notify_port`;
-    }
+    send_command("/lives/open_notify_socket,$my_ip_addr,$notify_port");
 }
 
-
 # get some constants we need
-if ($^O eq "MSWin32") {
-    `$sendOMC /lives/constant/value/get s LIVES_STATUS_PLAYING`;
-}
-else {
-    `$sendOMC /lives/constant/value/get,LIVES_STATUS_PLAYING`;
-}
+send_command("/lives/constant/value/get,LIVES_STATUS_PLAYING");
 
 $playstat=&get_newmsg;
 
-if ($^O eq "MSWin32") {
-    `$sendOMC /lives/constant/value/get s LIVES_LOOP_MODE_CONTINUOUS`;
-}
-else {
-    `$sendOMC /lives/constant/value/get,LIVES_LOOP_MODE_CONTINUOUS`;
-}
-$loopconst=&get_newmsg;
+if ($loop) {
+    send_command("/lives/constant/value/get,LIVES_LOOP_MODE_CONTINUOUS");
 
-# get current loop mode
-`$sendOMC /video/loop/get`;
-$currloop=&get_newmsg;
+    $loopconst=&get_newmsg;
+
+    # get current loop mode
+    send_command("/video/loop/get");
+    $currloop=&get_newmsg;
+}
 
 if ($mute) {
     #mute the sound if requested
-    `$sendOMC /audio/mute/get`;
+    send_command("/audio/mute/get");
     $mute = 1 - &get_newmsg;
     if ($mute) {
-	if ($^O eq "MSWin32") {
-	    `$sendOMC /audio/mute/set i 1`;
-	}
-	else {
-	    `$sendOMC /audio/mute/set,1`;
-	}
+	send_command("/audio/mute/set,1");
     }
 }
 
 if (!$waitforplay) {
     #trigger playback
-    if ($^O eq "MSWin32") {
-	`$sendOMC /video/loop/set i $loopconst`;
-    } else {
-	`$sendOMC /video/loop/set,$loopconst`;
+    if ($loop) {
+	send_command("/video/loop/set,$loopconst");
     }
 
-    `$sendOMC /video/play`;
+    send_command("/video/play");
 
-    #wait 5 seconds for playback to start, else bail
-    for ($i=0;$i<5;$i++) {
-	`$sendOMC /lives/status/get`;
+    #wait $timeout seconds for playback to start, else bail
+    for ($i=0; $i<$timeout; $i++) {
+	send_command("/lives/status/get");
 	$status=&get_newmsg;
 	if ($status != $playstat) {
 	    sleep 1;
@@ -266,7 +248,6 @@ if (!$waitforplay) {
 
 if ($DEBUG) {print STDERR "Performing magic...\n";}
 
-
 while (1) {
     if ($noty) {
 	for ($ii=0;$ii<4;$ii++) {
@@ -278,7 +259,7 @@ while (1) {
     }
     
     if ($waitforplay) {
-	`$sendOMC /lives/status/get`;
+	send_command("/lives/status/get");
 	$status=&get_newmsg;
 	if ($status != $playstat) {
 	    sleep 1;
@@ -286,74 +267,64 @@ while (1) {
 	}
     }
 
-    $action=int(rand(18))+1;
+    $action=int(rand(20))+1;
     
     if ($action<6) {
 	# 1,2,3,4,5
 	# random clip switch
-	$nextclip=int(rand($numclips)+1);
-	if ($^O eq "MSWin32") {
-	    `$sendOMC /clip/select i $nextclip`;
-	} else {
-	    `$sendOMC /clip/select,$nextclip`;
+	if ($allow_clipswitch) {
+	    $nextclip=int(rand($numclips)+1);
+	    send_command("/clip/select,$nextclip");
 	}
     }
     elsif ($action<15) {
 	# mess with effects
-	$nexteffectkey=int(rand($numeffectkeys)+1);
-	`$sendOMC /clip/select,$nextclip`;
-	if ($action<11) {
-	    # 6,7,8,9,10
-	    if ($nexteffectkey!=$key_to_avoid) {
-		`$sendOMC /effect_key/disable,$nexteffectkey`;
-		if ($^O eq "MSWin32") {
-		    `$sendOMC /effect_key/disable i $nexteffectkey`;
-		} else {
-		    `$sendOMC /effect_key/disable,$nexteffectkey`;
+	if ($allow_fxchanges) {
+	    $nexteffectkey=int(rand($numeffectkeys)+1);
+	    if ($action<10) {
+		# 6,7,8,9
+		if ($nexteffectkey!=$key_to_avoid) {
+		    send_command("/effect_key/disable,$nexteffectkey");
 		}
 	    }
-	}
-	elsif ($action<13) {
-	    #11,12
-	    if ($nexteffectkey!=$key_to_avoid) {
-		if ($^O eq "MSWin32") {
-		    `$sendOMC /effect_key/enable i $nexteffectkey`;
-		} else {
-		    `$sendOMC /effect_key/enable,$nexteffectkey`;
+	    elsif ($action<13) {
+		#10,11,12
+		if ($nexteffectkey!=$key_to_avoid) {
+		    send_command("/effect_key/enable,$nexteffectkey");
 		}
 	    }
-	}
-	else {
-	    #13,14,15,16
-	    if ($nexteffectkey!=$key_to_avoid) {
-		if ($^O eq "MSWin32") {
-		    `$sendOMC /effect_key/maxmode/get i $nexteffectkey`;
-		} else {
-		    `$sendOMC /effect_key/maxmode/get,$nexteffectkey`;
-		}
+	    else {
+		#13,14,15,16
+		if ($nexteffectkey!=$key_to_avoid) {
+		    send_command("/effect_key/maxmode/get,$nexteffectkey");
+		    
+		    $maxmode=&get_newmsg;
+		    $newmode=int(rand($maxmode))+1;
 
-		$maxmode=&get_newmsg;
-		$newmode=int(rand($maxmode))+1;
-
-		if ($^O eq "MSWin32") {
-		    `$sendOMC /effect_key/mode/set i $nexteffectkey i $newmode`;
-		} else {
-		    `$sendOMC /effect_key/mode/set,$nexteffectkey,$newmode`;
+		    send_command("/effect_key/mode/set,$nexteffectkey,$newmode");
 		}
 	    }
 	}
     }
     elsif ($action==17) {
-	`$sendOMC /clip/foreground/background/swap`;
+	if ($allow_fgbgswap) {
+	    send_command("/clip/foreground/background/swap");
+	}
     }
-
     else {
-	#18
-	`$sendOMC /video/play/reverse`;
+	#18, #19, #20
+	if ($allow_dirchange) {
+	    if ($play_reversed || $action == 18) {
+		#make flipping from backwards to forwards more likely
+		# so that we actually make some progress in the clip
+		$play_reversed = !$play_reversed;
+		send_command("/video/play/reverse");
+	    }
+	}
     }
 
     if (!$waitforplay) {
-	`$sendOMC /lives/status/get`;
+	send_command("/lives/status/get");
 	$status=&get_newmsg;
 	if ($status != $playstat) {
 	    &finish;
@@ -362,30 +333,33 @@ while (1) {
 	    last;
 	}
     }
-	
 }
 
 exit 0;
 
+
 #####################################################################
+sub send_command {
+    if ($DEBUG) {print STDERR "Autolives sending: @_\n";}
+    my ($command)="$sendOMC @_";
+    if (!$is_mingw) {
+	`$command`;
+    }
+    else {
+	system("bash.exe", "-l", "-c", $command);
+    }
+}
+
 
 sub finish {
-
-    #reset loop mode
-    if ($^O eq "MSWin32") {
-	`$sendOMC /video/loop/set i $currloop`;
-    } else {
-	`$sendOMC /video/loop/set,$currloop`;
+    if ($loop) {
+	#reset loop mode
+	send_command("/video/loop/set,$currloop");
     }
 
     # reset mute status
     if ($mute) {
-	if ($^O eq "MSWin32") {
-	    `$sendOMC /audio/mute/set i 0`;
-	}
-	else {
-	    `$sendOMC /audio/mute/set,0`;
-	}
+	send_command("/audio/mute/set,0");
 	$mute = 0;
     }
 }
@@ -402,10 +376,10 @@ sub get_newmsg {
 	    #if ($DEBUG) {print STDERR "check $lport $local_port\n";}
 
 	    next if ($lport != $local_port);
-	
+	    
 	    # TODO - check from address is our host
 	    #if ($DEBUG) {print STDERR "FROM : ".inet_ntoa($ripaddr)."($rport)  ";
-	
+	    
 	    last;
 	}
 	#if ($DEBUG) {print STDERR "OK $lport $local_port : $newmsg\n";}
@@ -428,10 +402,10 @@ sub get_notify {
 	    #if ($DEBUG) {print STDERR "check $lport $notify_port $newmsg\n";}
 
 	    next if ($lport != $notify_port);
-	
+	    
 	    # TODO - check from address is our host
 	    #if ($DEBUG) {print STDERR "FROM : ".inet_ntoa($ripaddr)."($rport)  ";
-	
+	    
 	    last;
 	}
 	last if ($lport == $notify_port && $newmsg =~ /^32768\|(.*)/);
@@ -458,11 +432,7 @@ sub print_layout {
     my ($i,$j);
     for ($i=1;$i<=$keys;$i++) {
 	for ($j=1;$j<=$modes;$j++) {
-	    if ($^O eq "MSWin32") {
-		`$sendOMC /effect_key/name/get i $i i $j`;
-	    } else {
-		`$sendOMC /effect_key/name/get,$i,$j`;
-	    }
+	    send_command("/effect_key/name/get,$i,$j");
 	    $name=&get_newmsg;
 	    unless ($name eq "") {
 		if ($DEBUG) {print STDERR "key $i, mode $j: $name    ";}
